/* Library to add C-Functions to Scilab And/Or Matlab */
/* Bertrand Guiheneuf 1996 */

/* ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
/* Changes                                                     */
/* ----------------------------------------------------------- */
/* 6 March 1997 (Bertrand Guiheneuf):                          */
/* Added string management (no string matrix management!!!!)   */
/* ----------------------------------------------------------- */
/* ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */



#include "C_LAB_Interf.h"

#ifdef C_SCILAB


/****************************************************************/
/*       ERRORS MANAGEMENT                                      */


/********** InterfError **********/
/* errare humanum est */
void InterfError( mesg )

     char *mesg;

{
  cout("ERROR IN INTERFACE ROUTINE");
  cout( mesg );
  
}






/****************************************************************/
/*       MATRIX MANAGEMENT                                      */

/********** MatrixMemSize **********/
/* Returns the size of a matrix in bytes */
int MatrixMemSize( m )
     Matrix *m;

{
    integer *prefix;
    int error=0;
    int mem_siz=0;
    int type;
    int w,h; 

    prefix = (integer *)m; /* virtual variable to read the matrix properties */

    error=0; /* is there in error in the source matrix? for the moment, no */

    type = prefix[0]; /* matrix type */
    h = prefix[1];    /* matrix height */
    w = prefix[2];    /* matrix width */

    switch (type) 
      {
      case 1 :/* it is a numeric matrix */
	{
	  if ( prefix[3]==0 ) /* and it's real */
	    mem_siz = 4 * sizeof(integer) +w*h * sizeof(double);
	  else
	    mem_siz = 4 * sizeof(integer) + 2*w*h * sizeof(double);
	}
      break;
      
      case 10 :/* it is a string matrix */ 	
	{
	  mem_siz = (4 + w*h + prefix[4 + w*h]) * sizeof(integer);
	}
      break;

      default :
	{ error=1; }

      }



    if (error) 
      {
	InterfError("MatrixMemSize: matrix tupe is not valid\n" ) ;
	return(-1);
      }
    else
      {
	return(mem_siz);
      }


}






/********** MatrixCreate **********/
/* Create a Matrix with the good dimensions and the good type */
/* Be careful when used with a string type !!!!! */

Matrix *MatrixCreate( h,w,type )
     int h; /* number of lines */
     int w; /* number of columns */
     char *type; /* type of the matrix */

     /* type="real"    --> double (real part only )           */
     /* type="complex" --> double (real and imaginery part )  */

     /* type="sys-string" --> string */
     /* NOT DESIGNED FOR SCILAB/MATLAB PROGRAMMER USE                      */
     /* INTERNAL USE ONLY !!!!!!!!!!!!                                     */
     /*      WARNING: The string is contained in a (1x1) string matrix     */
     /*      In this case, on of the dimensions (h or w) must be 1 and the */
     /*      other one represents the size of the string !!!!!!!!!!!!!!    */
     
	
     
{

  int mat_size;
  Matrix *output = 0;
  integer *prefix;
  char Error[200];

  if (!strcmp(type,"real"))
    {
      mat_size = 4 * sizeof(integer) + w*h * sizeof(double);
      output = (Matrix *)malloc((size_t)mat_size); /* the final matrix */
      prefix = (integer *)output; /* to access the matrix properties */
      
      prefix[0] = 1;  /* it's a full numerical matrix */
      prefix[1] = h;  /* coding the dimensions ... */
      prefix[2] = w;  /* see Guide for Developpers in the SCILAB doc */
      prefix[3] = 0;  /* no imaginery part */
    }
  else
    {
      if (!strcmp("complex",type))
	{
	  mat_size = 4 * sizeof(integer) + 2*w*h * sizeof(double);
	  output = (Matrix *)malloc(mat_size); /* the final matrix */
	  prefix = (integer *)output; /* to access the matrix properties */
	  
	  prefix[0] = 1;  /* it's a full numeric matrix */
	  prefix[1] = h;  /* coding the dimensions ... */
	  prefix[2] = w;  /* see Guide for Developpers in the SCILAB doc */
	  prefix[3] = 1;  /* it's complex! */
	}
      else
	{
	  if (!strcmp("sys-string",type))
	    {
	      if ( ( (MIN(w,h)) == 1) )
		{
		  mat_size = (4 + 2  + w*h) * sizeof(integer);
		  output = (Matrix *)malloc((unsigned)mat_size); /* the final matrix */
		  prefix = (integer *)output; /* to access the matrix properties */
	  
		  prefix[0] = 10; /* it's a string matrix */
		  prefix[1] = 1;  /* dimensions */
		  prefix[2] = 1;  /* multiple strings matrices are not allowed for the moment */
		  prefix[3] = 0;  /* see Guide for Developpers in the SCILAB doc */

		  prefix[4] = 1;  /* pointer on the string */
		  prefix[5] = 1+w*h; /* pointer on the first char after the string */

		}
	      else 
		{
		  sprintf(Error,"The string matrix dimension must be 1xN or Nx1");
		  InterfError(Error);
		}
	    }
	  else 
	    {
	      sprintf(Error,"%s is not an implemented type",type);
	      InterfError(Error);
	    }
	
	}
    }

  

  return(output);

}











/********** MatrixGetType **********/
/* returns the type of a matrix */
void MatrixGetType( m,type )
     Matrix *m;  /* the matrix which we want to know the type */
     char *type; /* the return value */

     /* the possible return values are:        */
     /* "real" for a real numeric matrix       */
     /* "complex" for a complex numeric matrix */

{
  integer *prefix;    /* virtual variable to read the matrix properties */
  
  prefix = (integer *)m; /* we want to read the first four ints of m */

  switch (prefix[0]) /* the matrix is numeric */
    {
    case 1 :
      {
	if (prefix[3] == 0) /* is it complex ??? */
	  strcpy(type,"real"); 
	else
	  strcpy(type,"complex");
      }
    break;
      
    case 10 :
      {
	strcpy(type,"string");
      }
    break;
    
    default :
      {
	strcpy(type,"unknown");
      }
    
    }
  
  
}










/********** MatrixIsNumeric **********/
/* tests if a matrix is numeric ( real or complex ) */
int MatrixIsNumeric( m )
     Matrix *m; /* the matrix we want to test the type */

{

  integer *prefix;    /* virtual variable to read the matrix properties */
  
  prefix = (integer *)m;
  return( ( prefix[0] == 1 ) ); /* easy, no? */

}










/********** MatrixIsReal  **********/
/* tests if a matrix is real */
int MatrixIsReal( m )
     Matrix *m; /* the matrix we want to test the type */

{
  integer *prefix;    /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  /* type verification will come later */

  return( ( prefix[3] == 0 ) && ( prefix[0] == 1 )  ); /* easy again */

}









/********** MatrixIsComplex  **********/
/* tests if a matrix is complex */
int MatrixIsComplex( m )
     Matrix *m; /* the matrix we want to test the type */

{
  integer *prefix;    /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  /* type verification will come later */
  return( ( prefix[3] == 1 ) && ( prefix[0] == 1 )  );

}


/********** MatrixIsScalar  **********/
/* tests if a matrix is a real scalar */
int MatrixIsScalar( m )
     Matrix *m; /* the matrix we want to test the type */

{
  integer *prefix;    /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  return( ((prefix[0]==1) && (prefix[2]==1) && (prefix[2]==1) && (prefix[3]==0)) );

}

/********** MatrixIsString  **********/
/* tests if a matrix is (1x1) string matrix */
int MatrixIsString( m )
     Matrix *m; /* the matrix we want to test the type */

{
  integer *prefix;    /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  return( ((prefix[0]==10) && (prefix[2]==1) && (prefix[2]==1) && (prefix[3]==0)) );

}


/********** MatrixReadString  **********/
/* Reads the string contained in a string matrix */
/* Necessary because of scilab strange management of strings */
/* The returned pointer will need to be freed by the user !!!!!! */

char *MatrixReadString( m )
      Matrix *m; /* the matrix in which the string is stored */

{
  char *outstr;
  integer *prefix;    /* virtual variable to read the matrix properties */
  int strsiz;
  int op; /* to code the operarion when calling the fortran routine cvstr */

  prefix = (integer *)m;
  strsiz = prefix[5]-1;
  outstr = (char *)malloc((strsiz+1) * sizeof(char));
  op = 1; /* scilab code to ascii */

  F2C(cvstr)(&strsiz, prefix + 6, outstr, &op );
  outstr[strsiz]='\0';
		
  return(outstr);

}



/********** MatrixCreateString  **********/
/* Reads the string contained in a ascii  string and put it  */
/* into a (1x1) Scilab string Matrix                         */

Matrix *MatrixCreateString( s )
      char *s;   /* the string to write */

{
  Matrix *m; /* the matrix where we want to put the string */
  integer *prefix;    /* virtual variable to read the matrix properties */
  int strsiz;
  int op; /* to code the operartion when calling the fortran routine cvstr */
  
 
  strsiz = strlen(s);
  m = MatrixCreate(1, strsiz, "sys-string");

  prefix = (integer *)m;
  op = 0; /* ascii to scilab code */
  F2C(cvstr)(&strsiz, prefix + 6, s, &op);
  return(m); 

}












  

/********** MatrixFree **********/
/* Free a Matrix. For later compatibility */
void MatrixFree( m )
     Matrix *m; /* matrix to be freed */

{
  free(m);
}










/********** MatrixGetPr **********/
/* Returns a pointer on the real part of a numeric matrix */
double *MatrixGetPr( m )
     Matrix *m; /* matrix that we want to get the real part */

{
  if ( !MatrixIsNumeric(m) ) /* oh damn, what the f. is that "m" you gave me ??? */
    {
      InterfError("MatrixGetPr : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      return( &(m[2]) );
    }
}









/********** MatrixGetPi **********/
/* Returns a pointer on the imaginery part of a numeric complex  matrix */
double *MatrixGetPi( m )
     Matrix *m; /* matrix that we want to get the real part */

{
  integer *prefix;  /* virtual variable to read the matrix properties */
  prefix = (integer *)m;

  if ( !MatrixIsNumeric(m) ) /* oh damn, what the f. is that "m" you gave me ??? */
    {
      InterfError("MatrixGetPi : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      if ( !MatrixIsComplex(m) ) /* come on boy, don't try it with me .. */
	{
	  InterfError("MatrixGetPi : Matrix is must be complex\n");
	  return(0); /* returns a NULL pointer */
	}
      else /* i knew you could */
	{
	  return( &(m[2 + prefix[1]*prefix[2]]) ); /* refix[1/2] are the matrix dimensions */
	}
    }
}









/********** MatrixGetWidth **********/
/* returns the number of columns of the matrix */
int MatrixGetWidth( m )
     Matrix *m; /* The matrix ... */
     
{
  
  integer *prefix;  /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  
  return ( prefix[2] ); /* see Guide for Developpers in the SCILAB doc */

}





/********** MatrixGetHeight **********/
/* returns the number of lines of the matrix */
int MatrixGetHeight( m )
     Matrix *m; /* The matrix ... */
     
{
  
  integer *prefix;  /* virtual variable to read the matrix properties */

  prefix = (integer *)m;
  
  return ( prefix[1] ); /* see Guide for Developpers in the SCILAB doc */

}








/********** MatrixGetScalar **********/
/* return the real part of the (1,1) element of a matrix */
/* if offers a way to get the real parameters  without   */
/* handling all the matrix treatment */
double MatrixGetScalar( m )
     Matrix *m; /* the matrix containing the scalar */

{
  if ( !MatrixIsNumeric(m) ) /* It is not a numeric matrix !! */
    {
      InterfError("MatrixGetScalar : matrix must be numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* OK */
    {
      return( m[2] );
    }
}








/********** MatrixTranspose **********/
/* Transpose a matrix but without conjugating the complexes */
/* This routine is SLOW but permits to interface C programs */
/* without much efforts. Remember the elements of a matrix  */
/* are written line per line in C whereas they are writen   */
/* column after column in Fortran.                          */
/* Unfortunately ( for us C-coder ), Scilab codes the       */
/* in the Fortran-like style                                */   
/* Why doesn't everybody code in JAVA ?????                 */
/* anyway IT IS STRONGLY RECOMMENDED TO HANDLE THE PROBLEM  */
/* IN YOUR C-ROUTINES                                       */
void MatrixTranspose( m )
     Matrix *m; /* the matrix to transpose */
     
{
  double *buffer; /* the buffer used to transpose the matrix */
  /* LATER, this routine will be optimized */
  /* TO DO : Use the temporary memory of scilab when avalaible    */
  /*         When the matrix is squared, do not use any buffer(?) */

  integer *prefix; /* to get the matrix properties */
  int mat_w, mat_h, mat_size;
  int i,j;

  prefix = (integer *)m;
  mat_h = prefix[1];
  mat_w = prefix[2];
  mat_size = mat_h*mat_w;

  if ( !prefix[0]==1 ) /* the matrix is not numeric */
    InterfError("MatrixTranspose : Unfortunately, the matrix must be numeric\n");
  else 
    {
      buffer = (double *)malloc( mat_size * sizeof(double) );
      if (buffer == 0) /* not enough memory, sorry */
	InterfError("MatrixTranspose : Not enough memory to allocate the buffer\nTry to quit applications\n");
      else
	{
	  m+=2; /* to be on the real part */
	  for (i=0; i<mat_h; i++) /* transpose in the buffer */
	    for (j=0; j<mat_w; j++)
	      buffer[i*mat_w+j] = m[j*mat_h+i];
	  
	  for (i=0; i<mat_h; i++) /* copy the buffer in the matrix */
	    for (j=0; j<mat_w; j++)
	      m[i*mat_w+j] = buffer[i*mat_w+j];
	  
	  prefix[1] = mat_w;
	  prefix[2] = mat_h;

	  if (prefix[3] == 1) /* is it complex ? */
	    {
	      m+=mat_size;
	      for (i=0; i<mat_h; i++) /* transpose in the buffer */
		for (j=0; j<mat_w; j++)
		  buffer[i*mat_w+j] = m[j*mat_h+i];
	      
	      for (i=0; i<mat_h; i++) /* copy the buffer in the matrix */
		for (j=0; j<mat_w; j++)
		  m[i*mat_w+j] = buffer[i*mat_w+j];
	      
	    }
	 
	  free(buffer);
	}
    }
}


/********** MatrixCopy **********/
/* Copies a matrix from one place to another */
void MatrixCopy( m_source, m_dest )
     Matrix *m_source;
     Matrix *m_dest;

{

  int mem_siz;

  mem_siz = MatrixMemSize(m_source);
  if (mem_siz == -1) 
    {
      InterfError("MatrixCopy : Cannot determinate the matrix memory size\n");
      return;
    }
  else
    {
      memcpy(m_dest, m_source, mem_siz);
    }

}




/****************************************************************/
/*       STACK MANAGEMENT                                       */






/********** StkAvail **********/
/* Returns the Available memory in the stack   */
/* the answer is in BYTE, not in "stck indexes" */
int StkAvail()
{
  
  return( ((vstk_.lstk)[(vstk_.bot)-1]-(vstk_.lstk)[(vstk_.top)]-1) * sizeof(double) );
}


/********** InterfInit **********/
/* Allocate the memory to the Param array */
/* Puts the input parameters in the Param array   */
/* also computes the number of input and output   */
/* parameters. They are returned in the Interf.NbParamIn */
/* and Interf.NbParamOut */
/* Also puts the index of the called function */
/* within the interface in Interf.FuncIndex */
void InterfInit()
{

  int i;
  
  Interf.NbParamIn = (com_.rhs>0 ? com_.rhs : 0) ;  
  Interf.NbParamOut = (com_.lhs>0 ? com_.lhs : 0);
  Interf.FuncIndex = com_.fin;
  Interf.ReturnCounter = 0;
  
  Interf.ReturnIndex = vstk_.top+1 - Interf.NbParamIn;
  /* The first free index minus the number of Input Parameters */

  if (Interf.NbParamIn == 0 ) Interf.Param=0;
  else
    {
      Interf.Param = (Matrix **)malloc(Interf.NbParamIn * sizeof(Matrix *));
      
      for (i=0; i<Interf.NbParamIn; i++)
	(Interf.Param)[i] = (Matrix *)( &( (stack_.stk)[ ( (vstk_.lstk)[vstk_.top-com_.rhs+i] ) -1] ) ); 
      /* in fact, it's stk(top-1 -rhs +1) to correct the C/FORTRAN problem */
    }
  
  
}


/********** ReturnParam **********/
/* Push a parameter (matrix) on the return stack */
void ReturnParam(m)
     Matrix *m;
{


  int CurrIndex;
 
  if (Interf.ReturnCounter < Interf.NbParamOut)
    {
      CurrIndex =(vstk_.lstk)[Interf.ReturnIndex-1];
      /* Gets the index of the first free cell in the return stack */
      if ( MatrixMemSize(m)>StkAvail() )
	{
	  free(m);
	  InterfError("Sorry, No more Stack free");
	  return;
	}

      MatrixCopy( m , &((stack_.stk)[CurrIndex-1]) );
      /* Copies the matrix there */
      
      Interf.ReturnIndex++;
      /* as said Franco : lets's prepare the next push */
      (vstk_.lstk)[Interf.ReturnIndex-1] = CurrIndex + MatrixMemSize( m ) / sizeof(Matrix);
      /* We must divide by sizeof(Matrix) because we work in bytes */
      /* not in "stack elements" */
      /* Ok, it's far from neatness */
      /* Anyway, doesn't it work??? */
      Interf.ReturnCounter++;
      free(m);
      /* and then free the parameter */
    }
  
  
}




/********** InterfDone **********/
/* Frees the memory used by Interf.Param */
void InterfDone()
{

  
  vstk_.top += Interf.NbParamOut - Interf.NbParamIn;
  if (Interf.NbParamIn > 0) free(Interf.Param);
  
}









#endif  C_SCILAB 




/***********************************************************/
/***********************************************************/
/***********************************************************/





#ifdef C_MATLAB4 

/****************************************************************/
/*       ERRORS MANAGEMENT                                      */


/********** InterfError **********/
/* errare humanum est */
void InterfError( mesg )

     char *mesg;

{
  mexErrMsgTxt(mesg);
  
}


/****************************************************************/
/*       MATRIX MANAGEMENT                                      */

/********** MatrixMemSize **********/
/* Returns the size of a matrix in bytes */
int MatrixMemSize( m )
     Matrix *m;

{
    InterfError("MatrixMemSize is not implemented in C-MATLAB interface");
    return(-1);
}




Matrix *MatrixCreate( h,w,type )
     int h; /* number of lines */
     int w; /* number of columns */
     char *type; /* type of the matrix */

     /* type="real"    --> double (real part only )           */
     /* type="complex" --> double (real and imaginery part )  */
     
	
     
{
  Matrix *output=0;
  char Error[200];
  
  if (!strcmp(type,"real"))
    output = mxCreateFull(h, w, 0);
  else
    {
      if (!strcmp("complex",type))
	output = mxCreateFull(h, w, 1);
      else
	{
	  sprintf(Error,"%s is not an implemented type",type);
	  InterfError("Type not implemented in the interface");
	}
    }


  return(output);

}



/********** MatrixGetType **********/
/* returns the type of a matrix */
void MatrixGetType( m,type )
     Matrix *m;  /* the matrix which we want to know the type */
     char *type; /* the return value */

     /* the possible return values are:          */
     /* "real" for a real numeric matrix       */
     /* "complex" for a complex numeric matrix */
     /* "string" for a string matrix */

{


  if (mxIsNumeric(m)) /* the matrix is numeric */
    {
      if (!mxIsComplex(m)) /* is it complex ??? */
	strcpy(type,"real"); 
      else
	strcpy(type,"complex"); 
    }
  else
    {
      if (mxIsString(m)) /* the matrix is string */
	strcpy(type,"string"); 
    }
  
}



/********** MatrixIsNumeric **********/
/* tests if a matrix is numeric ( real or complex ) */
int MatrixIsNumeric( m )
     Matrix *m; /* the matrix we want to test the type */

{

  return(mxIsNumeric(m));

}


/********** MatrixIsReal  **********/
/* tests if a matrix is real */
int MatrixIsReal( m )
     Matrix *m; /* the matrix we want to test the type */

{
  
  return( mxIsNumeric(m) && !mxIsComplex(m) );

}


/********** MatrixIsComplex  **********/
/* tests if a matrix is complex */
int MatrixIsComplex( m )
     Matrix *m; /* the matrix we want to test the type */

{
  
  return( mxIsNumeric(m) && mxIsComplex(m) );

}



/********** MatrixIsScalar  **********/
/* tests if a matrix is a real scalar */
int MatrixIsScalar( m )
     Matrix *m; /* the matrix we want to test the type */

{

  return ( mxIsNumeric(m) && !mxIsComplex(m) && 
	   (mxGetN(m) ==1) && (mxGetM(m) ==1) );

}


/********** MatrixIsString **********/
/* tests if a matrix is a string (but not a vector of strings) */
int MatrixIsString( m )
     Matrix *m; /* the matrix we want to test the type */

{

   return ( mxIsString(m) && (mxGetM(m) ==1) );


}



/********** MatrixReadString  **********/
/* Reads the string contained in a string matrix */
/* Necessary because of scilab strange management of strings */
/* The returned pointer will need to be freed by the user !!!!!! */

char *MatrixReadString( m )
      Matrix *m; /* the matrix in which the string is stored */

{
  char *outstr;
  int strsiz;

  strsiz = mxGetN(m);
  outstr = (char *)malloc((strsiz+1) * sizeof(char));
  mxGetString(m, outstr, strsiz+1);	
  return(outstr);

}



/********** MatrixCreateString  **********/
/* Reads the string contained in a ascii  string and put it */
/* into a (1x1) Matlab string Matrix                        */

Matrix *MatrixCreateString( s )
      char *s;   /* the string to write */

{
  return ( mxCreateString(s) );
}







/********** MatrixFree **********/
/* Free a Matrix. For later compatibility */
void MatrixFree( m )
     Matrix *m; /* matrix to be freed */

{
  mxFreeMatrix(m);
}



/********** MatrixGetPr **********/
/* Returns a pointer on the real part of a numeric matrix */
double *MatrixGetPr( m )
     Matrix *m; /* matrix that we want to get the real part */

{
  if ( !MatrixIsNumeric(m) ) /* oh damn, what the f. is that "m" you gave me ??? */
    {
      InterfError("MatrixGetPr : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      return( mxGetPr(m) );
    }
}




/********** MatrixGetPi **********/
/* Returns a pointer on the imaginery part of a numeric complex  matrix */
double *MatrixGetPi( m )
     Matrix *m; /* matrix that we want to get the real part */

{

  if ( !MatrixIsNumeric(m) ) /* oh damn, what the f. is that "m" you gave me ??? */
    {
      InterfError("MatrixGetPi : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      if ( !MatrixIsComplex(m) ) /* come on boy, don't try it with me .. */
	{
	  InterfError("MatrixGetPi : Matrix is must be complex\n");
	  return(0); /* returns a NULL pointer */
	}
      else /* i knew you could */
	{
	  return( mxGetPi(m) ); 
	}
    }
}




/********** MatrixGetWidth **********/
/* returns the number of columns of the matrix */
int MatrixGetWidth( m )
     Matrix *m; /* The matrix ... */
     
{
  
  return ( mxGetN(m) );

}





/********** MatrixGetHeight **********/
/* returns the number of lines of the matrix */
int MatrixGetHeight( m )
     Matrix *m; /* The matrix ... */
     
{
  
  return ( mxGetM(m) );

}



/********** MatrixGetScalar **********/
/* return the real part of the (1,1) element of a matrix */
/* if offers a way to get the real parameters  without   */
/* handling all the matrix treatment */
double MatrixGetScalar( m )
     Matrix *m; /* the matrix containing the scalar */

{
  if ( !MatrixIsNumeric(m) ) /* It is not a numeric matrix !! */
    {
      InterfError("MatrixGetScalar : matrix must be numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* OK */
    {
      return( mxGetScalar(m) );
    }
}

/********** MatrixTranspose **********/
/* Transpose a matrix but without conjugating the complexes */
/* This routine is SLOW but permits to interface C programs */
/* without much efforts. Remember the elements of a matrix  */
/* are written line per line in C whereas they are writen   */
/* column after column in Fortran.                          */
/* Unfortunately ( for us C-coder ), Scilab codes the       */
/* in the Fortran-like style                                */   
/* Why doesn't everybody code in JAVA ?????                 */
/* anyway IT IS STRONGLY RECOMMENDED TO HANDLE THE PROBLEM  */
/* IN YOUR C-ROUTINES                                       */
void MatrixTranspose( m )
     Matrix *m; /* the matrix to transpose */
     
{
  double *buffer; /* the buffer used to transpose the matrix */
  /* LATER, this routine will be optimized */
  /* TO DO : Use the temporary memory of scilab when avalaible    */
  /*         When the matrix is squared, do not use any buffer(?) */

  double *pm;
  int Mat_w, Mat_h, mat_size;
  int i,j;


  Mat_h = mxGetM(m);
  Mat_w = mxGetN(m);
  mat_size = Mat_h*Mat_w;

  if ( !mxIsNumeric(m) ) /* the matrix is not numeric */
    InterfError("MatrixTranspose : Unfortunately, the matrix must be numeric\n");
  else 
    {
      buffer = (double *)malloc( mat_size * sizeof(double) );
      if (buffer == 0) /* not enough memory, sorry */
	InterfError("MatrixTranspose : Not enough memory to allocate the buffer\nTry to quit applications\n");
      else
	{
	  pm = mxGetPr(m); /* to be on the real part */
	  for (i=0; i<Mat_h; i++) /* transpose in the buffer */
	    for (j=0; j<Mat_w; j++)
	      buffer[i*Mat_w+j] = pm[j*Mat_h+i];
	  
	  for (i=0; i<Mat_h; i++) /* copy the buffer in the matrix */
	    for (j=0; j<Mat_w; j++)
	      pm[i*Mat_w+j] = buffer[i*Mat_w+j];
	  
	  mxSetM(m,Mat_w);
	  mxSetN(m,Mat_h);

	  if (MatrixIsComplex(m)) /* is it complex ? */
	    {
	      pm = mxGetPi(m);
	      for (i=0; i<Mat_h; i++) /* transpose in the buffer */
		for (j=0; j<Mat_w; j++)
		  buffer[i*Mat_w+j] = pm[j*Mat_h+i];
	      
	      for (i=0; i<Mat_h; i++) /* copy the buffer in the matrix */
		for (j=0; j<Mat_w; j++)
		  pm[i*Mat_w+j] = buffer[i*Mat_w+j];
	      
	    }
	 
	  free(buffer);
	}
    }
}


/********** StkAvail **********/
/* Returns the Available memory in the stack   */
/* the answer is in BYTE, not in "stck indexes" */
/* NOT IMPLEMENTED FOR MATLAB */
int StkAvail()
{
 return(0);
}




/********** InterfInit **********/
/* Allocate the memory to the Param array */
/* Puts the input parameters in the Param array   */
/* also computes the number of input and output   */
/* parameters. They are returned in the Interf.NbParamIn */
/* and Interf.NbParamOut */
/* Also puts the index of the called function */
/* within the interface in Interf.FuncIndex */
void InterfInit( nlhs, plhs, nrhs, prhs )
int nlhs;
Matrix *plhs[];
int nrhs;
Matrix *prhs[];
{

  
  Interf.NbParamIn = nrhs;
  Interf.NbParamOut = nlhs;
  Interf.FuncIndex = -1;
  Interf.ReturnCounter = 0;
  
  Interf.Param = prhs;
  Interf.Return = plhs;
   
}


/********** ReturnParam **********/
/* Push a parameter (matrix) on the return stack */
void ReturnParam(m)
     Matrix *m;
{


 
 
  if (Interf.ReturnCounter < Interf.NbParamOut)
    {
      Interf.Return[Interf.ReturnCounter] = m;
      Interf.ReturnCounter++;
    }
  
  
}


/********** InterfDone **********/
/* Useless with Matlab */
void InterfDone()
{
  
}

#endif  /* C_MATLAB4 */




#ifdef C_MATLAB 

/****************************************************************/
/*       ERRORS MANAGEMENT                                      */


/********** InterfError **********/
/* errare humanum est */
void InterfError( mesg )

     char *mesg;

{
  mexErrMsgTxt(mesg);
  
}


/****************************************************************/
/*       MATRIX MANAGEMENT                                      */

/********** MatrixMemSize **********/
/* Returns the size of a matrix in bytes */
int MatrixMemSize( m )
     Matrix *m;

{
    InterfError("MatrixMemSize is not implemented in C-MATLAB interface");
    return(-1);
}




Matrix *MatrixCreate( h,w,type )
     int h; /* number of lines */
     int w; /* number of columns */
     char *type; /* type of the matrix */

     /* type="real"    --> double (real part only )           */
     /* type="complex" --> double (real and imaginery part )  */
     
	
     
{
  Matrix *output=0;
  char Error[200];
  
  if (!strcmp(type,"real"))
    output = mxCreateDoubleMatrix(h, w, mxREAL);
  else
    {
      if (!strcmp("complex",type))
	output = mxCreateDoubleMatrix(h, w, mxCOMPLEX);
      else
	{
	  sprintf(Error,"%s is not an implemented type",type);
	  InterfError("Type not implemented in the interface");
	}
    }


  return(output);

}



/********** MatrixGetType **********/
/* returns the type of a matrix */
void MatrixGetType( m,type )
     Matrix *m;  /* the matrix which we want to know the type */
     char *type; /* the return value */

     /* the possible return values are:          */
     /* "real" for a real numeric matrix       */
     /* "complex" for a complex numeric matrix */
     /* "string" for a string matrix */

{


  if (mxIsNumeric(m)) /* the matrix is numeric */
    {
      if (!mxIsComplex(m)) /* is it complex ??? */
	strcpy(type,"real"); 
      else
	strcpy(type,"complex"); 
    }
  else
    {
      if (mxIsChar(m)) /* the matrix is string */
	strcpy(type,"string"); 
    }
  
}



/********** MatrixIsNumeric **********/
/* tests if a matrix is numeric ( real or complex ) */
int MatrixIsNumeric( m )
     Matrix *m; /* the matrix we want to test the type */

{

  return(mxIsNumeric(m));

}


/********** MatrixIsReal  **********/
/* tests if a matrix is real */
int MatrixIsReal( m )
     Matrix *m; /* the matrix we want to test the type */

{
  
  return( mxIsNumeric(m) && !mxIsComplex(m) );

}


/********** MatrixIsComplex  **********/
/* tests if a matrix is complex */
int MatrixIsComplex( m )
     Matrix *m; /* the matrix we want to test the type */

{
  
  return( mxIsNumeric(m) && mxIsComplex(m) );

}



/********** MatrixIsScalar  **********/
/* tests if a matrix is a real scalar */
int MatrixIsScalar( m )
     Matrix *m; /* the matrix we want to test the type */

{

  return ( mxIsNumeric(m) && !mxIsComplex(m) && 
	   (mxGetN(m) ==1) && (mxGetM(m) ==1) );

}


/********** MatrixIsString **********/
/* tests if a matrix is a string (but not a vector of strings) */
int MatrixIsString( m )
     Matrix *m; /* the matrix we want to test the type */

{

   return ( mxIsChar(m) && (mxGetM(m) ==1) );


}



/********** MatrixReadString  **********/
/* Reads the string contained in a string matrix */
/* Necessary because of scilab strange management of strings */
/* The returned pointer will need to be freed by the user !!!!!! */

char *MatrixReadString( m )
      Matrix *m; /* the matrix in which the string is stored */

{
  char *outstr;
  int strsiz;

  strsiz = mxGetN(m);
  outstr = (char *)malloc((strsiz+1) * sizeof(char));
  mxGetString(m, outstr, strsiz+1);	
  return(outstr);

}



/********** MatrixCreateString  **********/
/* Reads the string contained in a ascii  string and put it */
/* into a (1x1) Matlab string Matrix                        */

Matrix *MatrixCreateString( s )
      char *s;   /* the string to write */

{
  return ( mxCreateString(s) );
}







/********** MatrixFree **********/
/* Free a Matrix. For later compatibility */
void MatrixFree( m )
     Matrix *m; /* matrix to be freed */

{
  mxDestroyArray(m);
}



/********** MatrixGetPr **********/
/* Returns a pointer on the real part of a numeric matrix */
double *MatrixGetPr( m )
     Matrix *m; /* matrix that we want to get the real part */

{
  if ( !MatrixIsNumeric(m) )
    {
      InterfError("MatrixGetPr : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      return( mxGetPr(m) );
    }
}




/********** MatrixGetPi **********/
/* Returns a pointer on the imaginery part of a numeric complex  matrix */
double *MatrixGetPi( m )
     Matrix *m; /* matrix that we want to get the real part */

{

  if ( !MatrixIsNumeric(m) ) /* oh damn, what the f. is that "m" you gave me ??? */
    {
      InterfError("MatrixGetPi : Matrix is not numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* good boy */
    {
      if ( !MatrixIsComplex(m) ) /* come on boy, don't try it with me .. */
	{
	  InterfError("MatrixGetPi : Matrix is must be complex\n");
	  return(0); /* returns a NULL pointer */
	}
      else /* i knew you could */
	{
	  return( mxGetPi(m) ); 
	}
    }
}




/********** MatrixGetWidth **********/
/* returns the number of columns of the matrix */
int MatrixGetWidth( m )
     Matrix *m; /* The matrix ... */
     
{
  
  return ( mxGetN(m) );

}





/********** MatrixGetHeight **********/
/* returns the number of lines of the matrix */
int MatrixGetHeight( m )
     Matrix *m; /* The matrix ... */
     
{
  
  return ( mxGetM(m) );

}



/********** MatrixGetScalar **********/
/* return the real part of the (1,1) element of a matrix */
/* if offers a way to get the real parameters  without   */
/* handling all the matrix treatment */
double MatrixGetScalar( m )
     Matrix *m; /* the matrix containing the scalar */

{
  if ( !MatrixIsNumeric(m) ) /* It is not a numeric matrix !! */
    {
      InterfError("MatrixGetScalar : matrix must be numeric\n");
      return(0); /* returns a NULL pointer */
    }
  else /* OK */
    {
      return( mxGetScalar(m) );
    }
}

/********** MatrixTranspose **********/
/* Transpose a matrix but without conjugating the complexes */
/* This routine is SLOW but permits to interface C programs */
/* without much efforts. Remember the elements of a matrix  */
/* are written line per line in C whereas they are writen   */
/* column after column in Fortran.                          */
/* Unfortunately ( for us C-coder ), Scilab codes the       */
/* in the Fortran-like style                                */   
/* Why doesn't everybody code in JAVA ?????                 */
/* anyway IT IS STRONGLY RECOMMENDED TO HANDLE THE PROBLEM  */
/* IN YOUR C-ROUTINES                                       */
void MatrixTranspose( m )
     Matrix *m; /* the matrix to transpose */
     
{
  double *buffer; /* the buffer used to transpose the matrix */
  /* LATER, this routine will be optimized */
  /* TO DO : Use the temporary memory of scilab when avalaible    */
  /*         When the matrix is squared, do not use any buffer(?) */

  double *pm;
  int Mat_w, Mat_h, mat_size;
  int i,j;


  Mat_h = mxGetM(m);
  Mat_w = mxGetN(m);
  mat_size = Mat_h*Mat_w;

  if ( !mxIsNumeric(m) ) /* the matrix is not numeric */
    InterfError("MatrixTranspose : Unfortunately, the matrix must be numeric\n");
  else 
    {
      buffer = (double *)malloc( mat_size * sizeof(double) );
      if (buffer == 0) /* not enough memory, sorry */
	InterfError("MatrixTranspose : Not enough memory to allocate the buffer\nTry to quit applications\n");
      else
	{
	  pm = mxGetPr(m); /* to be on the real part */
	  for (i=0; i<Mat_h; i++) /* transpose in the buffer */
	    for (j=0; j<Mat_w; j++)
	      buffer[i*Mat_w+j] = pm[j*Mat_h+i];
	  
	  for (i=0; i<Mat_h; i++) /* copy the buffer in the matrix */
	    for (j=0; j<Mat_w; j++)
	      pm[i*Mat_w+j] = buffer[i*Mat_w+j];
	  
	  mxSetM(m,Mat_w);
	  mxSetN(m,Mat_h);

	  if (MatrixIsComplex(m)) /* is it complex ? */
	    {
	      pm = mxGetPi(m);
	      for (i=0; i<Mat_h; i++) /* transpose in the buffer */
		for (j=0; j<Mat_w; j++)
		  buffer[i*Mat_w+j] = pm[j*Mat_h+i];
	      
	      for (i=0; i<Mat_h; i++) /* copy the buffer in the matrix */
		for (j=0; j<Mat_w; j++)
		  pm[i*Mat_w+j] = buffer[i*Mat_w+j];
	      
	    }
	 
	  free(buffer);
	}
    }
}


/********** StkAvail **********/
/* Returns the Available memory in the stack   */
/* the answer is in BYTE, not in "stck indexes" */
/* NOT IMPLEMENTED FOR MATLAB */
int StkAvail()
{
 return(0);
}




/********** InterfInit **********/
/* Allocate the memory to the Param array */
/* Puts the input parameters in the Param array   */
/* also computes the number of input and output   */
/* parameters. They are returned in the Interf.NbParamIn */
/* and Interf.NbParamOut */
/* Also puts the index of the called function */
/* within the interface in Interf.FuncIndex */
void InterfInit( nlhs, plhs, nrhs, prhs )
int nlhs;
Matrix *plhs[];
int nrhs;
Matrix *prhs[];
{
  
  
  Interf.NbParamIn = nrhs;
  Interf.NbParamOut = nlhs;
  Interf.FuncIndex = -1;
  Interf.ReturnCounter = 0;
  
  Interf.Param = prhs;
  Interf.Return = plhs;
   
}


/********** ReturnParam **********/
/* Push a parameter (matrix) on the return stack */
void ReturnParam(m)
     Matrix *m;
{


 
 
  if (Interf.ReturnCounter < Interf.NbParamOut)
    {
      Interf.Return[Interf.ReturnCounter] = m;
      Interf.ReturnCounter++;
    }
  
  
}


/********** InterfDone **********/
/* Useless with Matlab */
void InterfDone()
{
  
}

#endif  /* C_MATLAB */







