/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAS/SRC/LAB_badic1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Thu Sep 12 12:17:09 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Sep 12 16:05:39 2001
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAM_include.h"

/************  LAB_badic1d  *********************************************/

void LAB_badic1d()
{ 
  register int k=0;
  /* input parameters */
  short n=12;
  short b=2;   
  int b_pow_n=(int)pow((double)b,(double)n);
  int Nx=b_pow_n;     
  double* badic1d=NULL;

  /* Matrix */
  Matrix* Mn;
  Matrix* Mb;
  Matrix* Mbadic1d;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("badic1d: You must give at least 1 input argument\n");
      return;
    }
  
   if(Interf.NbParamIn>2) 
    {
      InterfError("badic1d: You must give at most 2 input argument\n");
      return;
    }   
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
    {
      InterfError("badic1d: You must give 1 output argument\n");
      return;
    }
 
  /* Get the input resolution: n and verify it */
  Mn=Interf.Param[0];
  if(!MatrixIsNumeric(Mn))
    {
      InterfError("badic1d: The input resolution: n must be a numeric matrix\n");
      return;
    } 
  if(!MatrixIsReal(Mn))
    {
      InterfError("badic1d: The input resolution: n must be a real matrix\n");
      return;
    }
  if(!MatrixIsScalar(Mn))
    {
      InterfError("badic1d: The input resolution: n must be a scalar\n");
      return;
    }
  /* Get the input argument */ 
  n=(short)MatrixGetScalar(Mn);
 
  if(Interf.NbParamIn>1)
    {
      /* Get the input base: b and verify it */
      Mb=Interf.Param[1];
      if(!MatrixIsNumeric(Mb))
	{
	  InterfError("badic1d: The input base: b must be a numeric matrix\n");
	  return;
	} 
      if(!MatrixIsReal(Mb))
	{
	  InterfError("badic1d: The input base: b must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mb))
	{
	  InterfError("badic1d: The input base: b must be a scalar\n");
	  return;
	}

      /* Get the input argument */ 
      b=(short)MatrixGetScalar(Mb);
    }
  /* else b=2 */
   
 
  /* Create the output Matrixes */   
  b_pow_n=(int)pow((double)b,(double)n); 
  Nx=b_pow_n;
  Mbadic1d=MatrixCreate(Nx,1,"real"); 
  badic1d=MatrixGetPr(Mbadic1d);
 
  for(k=0;k<n;k++)  
    {  
      int i=(int)pow((double)b,(double)k);
      *(badic1d+i)=1.;
    }

  /* Return the output */  
  ReturnParam(Mbadic1d); 
}

