/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /a/linoleum.2/FRACLAB/SRC-DEV/INA/TML/SRC/LAB_dtmll.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_dtmll.c,v 1.1.1.1 2001/01/11 16:01:11 raynal Exp $
* Created by       : (Fractales Project INRIA) Kiran Kolwankar
* Creation date    : Oct 2000
* Last edited by   : (Fractales Project INRIA) Kiran Kolwankar
* Last edition date: Jan 9 2001
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Kiran Kolwankar 2001 */

#include "C_LAB_Interf.h"
#include "dtmll.h"

void LAB_dtmll()
{
  /* input parameters */
  int K, N_n, n_std;
  /* input */
  int Nx = 0, Ny=0, n_out=0, Nnby2 = 0;
  int N_in=0,n_incr=0,n_total=0; 
  double* F_n=NULL;

  /* output */
  double* sprime_s=NULL;
  double* len=NULL;

  /* Matrix */
  Matrix* MF_n;
  Matrix* Mn_incr;
  Matrix* Mn_std;
  Matrix* MN_n;
  Matrix* Msprime_s;
  Matrix* Mlen;

  /* Check the input arguments number */
  if(Interf.NbParamIn!=4) 
  {
    InterfError("dtmll: You must give 4 input arguments\n");
    return;
  }  



  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=2) 
  {
    InterfError("dtmll: You must give 2 output argument\n");
    return;
  }

  /* Get the input vector: F_n and verify it */
  MF_n=Interf.Param[0];
  if(!MatrixIsNumeric(MF_n)) 
  {
    InterfError("dtmll: The input must be a numeric \n");
    return;
  }

  if(!MatrixIsReal(MF_n)) 
  {
    InterfError("dtmlf: The input must be a real vector\n");
    return;
  }

  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(MF_n);
  Ny=(int)MatrixGetHeight(MF_n);
  if ((Nx<=0)||(Ny<=0))
  {
    InterfError("fd2d : The size (width,height) of the input pointwise Hoelder exponents image: p_alpha2d must be >0\n");
    return;
  }

 
  if(((int)MIN(MatrixGetWidth(MF_n),MatrixGetHeight(MF_n)))!=1)
  {
    InterfError("dtmlf: The input : F_n must be a vector\n");
    return;
  } 
 
  /* Get the input argument */  
  F_n=(double*)MatrixGetPr(MF_n);
 

  /* Get the size of the input argument */ 
  K=(int)MAX(MatrixGetWidth(MF_n),MatrixGetHeight(MF_n)); 
  if(K<1)
  {
    InterfError("dtmll: The length of the input: F_n must be >0\n");
    return;
  } 
 
  if(Interf.NbParamIn > 1)
  { 
    /* Get the input # of Hoelder exponents: N and verify it */
    MN_n=Interf.Param[1];
    if(!MatrixIsNumeric(MN_n))
    {
      InterfError("dtmll: The window: N_n  must be a numeric matrix\n");
      return;
    }  
    if(!MatrixIsScalar(MN_n))
    {
      InterfError("dtmll: The window: N_n  must be a scalar\n");
      return;
    }
    /* Get the input argument */
    N_n=(int)MatrixGetScalar(MN_n);
    if(N_n<1)
    {
      InterfError("dtmll: The window: N_n must be >0\n");
      return;
    }
   
    /* Get the input # of Hoelder exponents: n_incr and verify it */
    Mn_incr=Interf.Param[2];
    if(!MatrixIsNumeric(Mn_incr))
    {
      InterfError("dtmll: The step: n_incr must be a numeric \n");
      return;
    }  
    if(!MatrixIsScalar(Mn_incr))
    {
      InterfError("dtmll: The step: n_incr  must be a scalar\n");
      return;
    }
    /* Get the input argument */
    n_incr=(int)MatrixGetScalar(Mn_incr);
    if(n_incr<1)
    {
      InterfError("dtmll: The step: n_incr must be >0\n");
      return;
    }
    
    /* Get the input # of Hoelder exponents: n_std and verify it */
    Mn_std=Interf.Param[3];
    if(!MatrixIsNumeric(Mn_std))
    {
      InterfError("dtmll: : n_std must be a numeric matrix\n");
      return;
    }  
    if(!MatrixIsScalar(Mn_std))
    {
      InterfError("dtmll: : n_std must be a scalar\n");
      return;
    }
    /* Get the input argument */
    n_std=(int)MatrixGetScalar(Mn_std);
    if(n_std<1)
    {
      InterfError("dtmll: : n_std must be >0\n");
      return;
    }
  }

  n_out = (int)MIN(n_std,((K-N_n)/n_incr)+1);

  /* Create the output Matrix */  
  Msprime_s=MatrixCreate(1,n_out,"real");
  sprime_s=MatrixGetPr(Msprime_s); 
  Mlen=MatrixCreate(1,101,"real"); 
  len=MatrixGetPr(Mlen);



  /* Call of the C function */
  if(dtmll(F_n,K,N_n,n_std,n_incr,sprime_s,len)==0)
  { 
    InterfError("dtmll: Call of the C function dtmll failed\n"); 
    return; 
  }


 
  /* Return the output */  
  ReturnParam(Msprime_s); 
  ReturnParam(Mlen);

}
