/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_fd1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Sep  1 19:14:15 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 16:19:38 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/


/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_dimension.h"

void LAB_fd1d()
{ 
  /* input pointwise hoelder exponents */
  int Nx=0; 
  double* p_alpha1d=NULL;
  /* input parameters */ 
  double epsilon=0.;
  int K=0;
  double* B=NULL;
  MFADt_dim t_dim=MFAD_BOX;
  /* output images */
  unsigned char* iso_alpha1d=NULL;
  /* output spectrum */ 
  int N=50;
  double* alpha=NULL;
  double* fd1d_alpha=NULL;

  /* Matrix */
  Matrix* Mp_alpha1d;
  Matrix* MB;
  Matrix* MN;
  Matrix* Malpha;
  Matrix* Mfd1d_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
  {
    InterfError("fd1d: You must give at least 1 input arguments\n");
    return;
  }  

  if(Interf.NbParamIn>3) 
  {
    InterfError("fd1d: You must give at most 3 input arguments\n");
    return;
  }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=2) 
  {
    InterfError("fd1d: You must give 2 output argument\n");
    return;
  }

  /* Get the input pointwise Hoelder exponents image: p_alpha1d and verify it */
  Mp_alpha1d=Interf.Param[0];
  if(!MatrixIsNumeric(Mp_alpha1d)) 
  {
    InterfError("fd1d: The input pointwise Hoelder exponents vector: p_alpha1d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mp_alpha1d)) 
  {
    InterfError("fd1d: The input pointwise Hoelder exponents vector: p_alpha1d must be a real matrix\n");
    return;
  }

  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mp_alpha1d);
  if (Nx<=0)
  {
    InterfError("fd1d: The size of the input pointwise Hoelder exponents vector: p_alpha1d must be >0\n");
    return;
  }
 
  /* Get the input argument */  
  p_alpha1d=(double*)MatrixGetPr(Mp_alpha1d);
 
  if(Interf.NbParamIn>1) 
    {
      /* Get the input box sizes: B and verify it */
      MB=Interf.Param[1];
      if(!MatrixIsNumeric(MB))
	{
	  InterfError("fd1d: The input box sizes: B must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MB))
	{
	  InterfError("fd1d: The input box sizes: B must be a real matrix\n");
	  return;
	}
      if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
	{
	  InterfError("fd1d: The input box sizes: B must be a vector\n");
	  return;
	} 
      /* Get the size of the input argument */ 
      K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
      if(K<1)
	{
	  InterfError("fd1d: The length of the input box sizes vector: B must be >0\n");
	  return;
	} 
      /* Get the input argument */
      B=MatrixGetPr(MB);
      if(*B>*(B+K-1))
	{
	  InterfError("fd1d: The input box sizes vector: B must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int k=0;
      int b_ini=1,b_fin=33;   
      K=(b_fin-b_ini)/2+1; 
      /* dynamic memory allocation */
      if((B=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
	{
	  InterfError("fd1d: malloc error\n");
	  return;
	}
      for (k=0;k<K;k++)
	*(B+k)=b_ini+k*2;
    }

  if(Interf.NbParamIn>2)
  { 
    /* Get the input # of Hoelder exponents: N and verify it */
    MN=Interf.Param[2];
    if(!MatrixIsNumeric(MN))
    {
      InterfError("fd1d: The # of Hoelder exponents: N  must be a numeric matrix\n");
      return;
    }  
    if(!MatrixIsScalar(MN))
    {
      InterfError("fd1d: The # of Hoelder exponents: N  must be a scalar\n");
      return;
    }
    /* Get the input argument */
    N=(int)MatrixGetScalar(MN);
    if(N<1)
    {
      InterfError("fd1d: The # of Hoelder exponents: N must be >0\n");
      return;
    }
  }
  /* else N=100 */

  /* Create the output Matrix */  
  Malpha=MatrixCreate(1,N,"real");
  alpha=MatrixGetPr(Malpha); 
  Mfd1d_alpha=MatrixCreate(1,N,"real"); 
  fd1d_alpha=MatrixGetPr(Mfd1d_alpha);

  /* Transpose Matrix */
  MatrixTranspose(Mp_alpha1d);
 
  /* memory allocation */
  if((iso_alpha1d=(unsigned char*)malloc((size_t)(Nx*sizeof(unsigned char))))==NULL)
  {
    InterfError("fd1d: malloc error\n");
    return;
  }

  /* Call of the C function */
  if(MFAH_fd1d(Nx,p_alpha1d,epsilon,iso_alpha1d,K,B,t_dim,N,alpha,fd1d_alpha)==0)
  { 
    InterfError("fd1d: Call of the C function MFAH_fd1d failed\n");
    return; 
  }

  /* dynamic memory deallocation */
  free((char*)iso_alpha1d);

  if (Interf.NbParamIn<=1) 
    free((char*)B);

  /* Transpose input Matrixes */
  MatrixTranspose(Mp_alpha1d);
 
  /* Return the output */  
  ReturnParam(Malpha); 
  ReturnParam(Mfd1d_alpha);
}
