#include <math.h>
#include "C_LAB_Interf.h"
#include "Frontiere.h"

#define smin 0.0
#define smax 2.0
#define NS 100

void LAB_frontiere()
{
	double *coeff,*freq;
	Matrix *Mcoeff;
	Matrix *Mfreq;
	Matrix *Mtemp;
	int Sx,Sy,i;
	double x,epsilon;

	double *frontiere;
	Matrix *Mfrontiere;
	double *abscisse;
	Matrix *Mabscisse;

	if (Interf.NbParamIn != 4)
	{
		InterfError("Frontiere: il faut 4 parametre");
		return;
	}
	
	Mcoeff = Interf.Param[0];
	if (!MatrixIsNumeric(Mcoeff) || !MatrixIsReal(Mcoeff) )
	{
		InterfError("Frontiere: The first parameter (cwt) must be a real matrix\n");
		return;
	}
	Sx=MatrixGetWidth(Mcoeff);
	Sy=MatrixGetHeight(Mcoeff);
	coeff=MatrixGetPr(Mcoeff);
	
	Mfreq = Interf.Param[1];
	if ( MIN(MatrixGetWidth(Mfreq),MatrixGetHeight(Mfreq)) != 1 )
	{
		InterfError("Frontiere : The second parameter (freq) must be a (1xN) or (Nx1) matrix\n");
		return;
	}
	if ( MAX(MatrixGetWidth(Mfreq),MatrixGetHeight(Mfreq)) != Sy )
	{
		InterfError("Frontiere : The second parameter (freq) must be of the same height than cwt\n");
		return;
	}
	freq=MatrixGetPr(Mfreq);
	
	Mtemp = Interf.Param[2];
	if ( !MatrixIsScalar(Mtemp) )
	{
		InterfError("Frontiere : The third parameter (x) must be a scalar\n");
		return;
	}
	x=MatrixGetScalar(Mtemp);
	
	Mtemp = Interf.Param[3];
	if ( !MatrixIsScalar(Mtemp) )
	{
		InterfError("Frontiere : The fourth parameter (epsilon) must be a scalar\n");
		return;
	}
	epsilon=MatrixGetScalar(Mtemp);

	Mfrontiere= MatrixCreate(NS,1,"real");
	frontiere= MatrixGetPr(Mfrontiere);
	Mabscisse= MatrixCreate(NS,1,"real");
	abscisse= MatrixGetPr(Mabscisse);
	for(i=0;i<NS;i++) abscisse[i]=i*(smax-smin)/NS;
	if (Frontiere(coeff,freq,frontiere,Sx,Sy,(int)floor(x*Sx),smin,smax,NS,epsilon))
	{
		InterfError("Frontiere : Not enough memory\n");
		return;
	}

	ReturnParam(Mfrontiere);
	ReturnParam(Mabscisse);
	return;
}
