/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MYFRACLAB/MFAM/SRC/LAB_linearlt.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_linearlt.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:50:40 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 16:50:41 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "C_LAB_Interf.h"
#include "MFAM_concave_hull.h"
#include "MFAM_legendre.h"

/***********  LAB_linearlt  **************************************************/

void LAB_linearlt()
{ 
  register k=0;
  /* # of points of the input function */ 
  int N=20; 
  /* input abscissa & function */
  double* x=NULL;
  double* u_x=NULL;
  /* # of points of the regularized function */ 
  int M=0; 
  /* regularized function & abscissa */   
  double* rx=NULL;
  double* ru_x=NULL;
  /* conjugate function & abscissa */
  double* s=NULL; 
  double* u_star_s=NULL; 
  /* temporay pointers */       
  double* tmprx=NULL;
  double* tmpru_x=NULL; 

  Matrix *Mx;
  Matrix *Mu_x;
  Matrix *Mrx;
  Matrix *Mru_x;
  Matrix *Ms;
  Matrix *Mu_star_s;

  /* Check the input arguments number */
  if(Interf.NbParamIn!=2) 
  {
    InterfError("linearlt: You must give 2 input arguments\n");
    return;
  }
  
  /* Check the output arguments number */
  if((Interf.NbParamOut!=2)&&(Interf.NbParamOut!=4))
  {
    InterfError("linearlt: You must give 2 or 4 output arguments\n");
    return;
  }
   
  /* Get the input abscissa x and verify it */
  Mx=Interf.Param[0];
  if(!MatrixIsNumeric(Mx))
  {
    InterfError("linearlt: The abscissa: x  must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(Mx))
  {
    InterfError("linearlt: The abscissa: x must be a real matrix\n");
    return;
  } 
  /* Get the width of the input abscissa x */ 
  N=(int)MAX(MatrixGetWidth(Mx),MatrixGetHeight(Mx));
  /* Get the pointer on the input abscissa x */ 
  x=MatrixGetPr(Mx);
 
  /* Get the input function u_x and verify it */
  Mu_x=Interf.Param[1];
  if(!MatrixIsNumeric(Mu_x))
  {
    InterfError("linearlt: The function: u_x must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(Mu_x))
  {
    InterfError("linearlt: The function: u_x must be a real matrix\n");
    return;
  } 
  /* Check the size of the input function u_x */
  if(N!=(int)MAX(MatrixGetWidth(Mu_x),MatrixGetHeight(Mu_x)))
  {
    InterfError("linearlt: The abscissa: x and the function: u_x must be of the same size\n");
    return;
  } 
  /* Get the pointer on the input function u_x */ 
  u_x=MatrixGetPr(Mu_x);

  /* dynamic memory allocation */            
  if((tmprx=(double*)malloc((unsigned)(N*sizeof(double))))==NULL)
  {
    InterfError("linearlt: malloc error\n");
    return; 
  }               
  if((tmpru_x=(double*)malloc((unsigned)(N*sizeof(double))))==NULL)
  {  
    InterfError("linearlt: malloc error\n");
    return; 
  }  

  /* Fill in the input vectors rx and ru_x */
  for(k=0;k<N;k++)
  {
    *(tmprx+k)=*(x+k);
    *(tmpru_x+k)=*(u_x+k);
  }

  /* Call of the C function */  
  if(MFAM_bb_concave_hull(N,tmprx,tmpru_x,&M)==0)
  {
    InterfError("linearlt: Call of the C function MFAM_bb_concave_hull failed\n");
    return; 
  }
      
  /* Create the output Matrix */  
  Ms=MatrixCreate(1,M,"real");
  Mu_star_s=MatrixCreate(1,M,"real");
 
  /* Get the pointer on the output Matrix */  
  s=MatrixGetPr(Ms);
  u_star_s=MatrixGetPr(Mu_star_s);
 
  /* Call of the C function */  
  if(MFAM_llt(M,tmprx,tmpru_x,s,u_star_s)==0) 
  { 
    InterfError("linearlt: Call of the C function MFAM_llt failed\n"); 
    return;
  }

  if(Interf.NbParamOut==4)
  { 
    /* Create the output Matrix */  
    Mrx=MatrixCreate(1,M,"real");
    Mru_x=MatrixCreate(1,M,"real");
 
    /* Get the pointer on the output Matrix */  
    rx=MatrixGetPr(Mx);
    ru_x=MatrixGetPr(Mru_x); 
     
    /* Fill in the output vectors Prrx and Prru_x */
    for(k=0;k<M;k++)
    {  
      *(rx+k)=*(tmprx+k);
      *(ru_x+k)=*(tmpru_x+k);
    }
  }

  /* dynamic memory desallocation */       
  free((char*)tmprx);
  free((char*)tmpru_x);

  /* Return the output */
  ReturnParam(Ms);
  ReturnParam(Mu_star_s);
  if(Interf.NbParamOut==4)
  { 
    ReturnParam(Mrx);
    ReturnParam(Mru_x);
  }
}


