/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MYFRACLAB/MFAG/SRC/LAB_mdfg1d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_mdfg1d.c,v 1.1.1.1 1999/02/10 13:38:44 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:55:49 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 16:55:49 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997 */

#include "C_LAB_Interf.h"
#include "MFAG_discrete.h"

/************  LAB_mdfg  ***************************************************/

void LAB_mdfg()
{
  int nu_n=0;                                             /* # of badics */
  double *mu_n=NULL;                                          /* measure */
  int N=0;                                     /* # of Hoelder exponents */
  double epsilon=0.;                                        /* precision */
  double *alpha=NULL;                               /* Hoelder exponents */
  int normflg=0;                                   /* normalization flag */
  double *fg_n_alpha=NULL;                                 /* dimensions */
  double *p_n_alpha=NULL;                               /* probabilities */

  Matrix *Mnu_n;     /* matrix pointing on the # of badics               */
  Matrix *Mmu_n;     /* matrix pointing on the measure                   */ 
  Matrix *MN;        /* matrix pointing on the # of Hoelder exponents    */ 
  Matrix *Malpha;    /* matrix pointing on the Hoelder exponents         */
  Matrix *Mnormflg;  /* Matrix pointing on the normalization flag        */
  Matrix *Mfg_n_alpha;/* matrix pointing on the dimensions               */ 
  Matrix *Mp_n_alpha;/* matrix pointing on the probabilities             */ 

  /* Check the input arguments number */
  if(Interf.NbParamIn<2) 
  {
    InterfError("mdfg: You must give at least 2 input arguments\n");
    return;
  }
  if(Interf.NbParamIn>3) 
  {
    InterfError("mdfg: You must give at most 3 input arguments\n");
    return;
  }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut<2) 
  {
    InterfError("mdfg: You must give at least 2 output arguments\n");
    return;
  }
  if(Interf.NbParamOut>3) 
  {
    InterfError("mdfg: You must give at most 3 output arguments\n");
    return;
  }

  /* Get the input measure mu_n and verify it */
  Mmu_n=Interf.Param[0];
  if(!MatrixIsNumeric(Mmu_n))
  {
    InterfError("mdfg: The measure mu_n must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(Mmu_n))
  {
    InterfError("mdfg: The measure mu_n must be a real matrix\n");
    return;
  }
 
  nu_n=(int)MatrixGetWidth(Mmu_n);

  /* Get the input argument */ 
  mu_n=MatrixGetPr(Mmu_n);
 
  /* Get the input # of Hoelder exponents and verify it */
  MN=Interf.Param[1];
  if(!MatrixIsNumeric(MN))
  {
    InterfError("mdfg: The # of Hoelder exponents must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsScalar(MN))
  {
    InterfError("mdfg: The # of Hoelder exponents must be a scalar\n");
    return;
  }

  /* Cast the input argument */
  N=(int)MatrixGetScalar(MN);

  if(Interf.NbParamIn==3)
  {  
    /* Get the normalization flag and verify it */
    Mnormflg=Interf.Param[2];

    /* Get the input argument */
    normflg=(int)MatrixGetScalar(Mnormflg);
  }

  /* Create the output Matrix */  
  Malpha=MatrixCreate(1,N,"real");
  Mfg_n_alpha=MatrixCreate(1,N,"real");
  Mp_n_alpha=MatrixCreate(1,N,"real");

  /* Catch the pointer on the output Matrix */  
  alpha=MatrixGetPr(Malpha);
  fg_n_alpha=MatrixGetPr(Mfg_n_alpha);
  p_n_alpha=MatrixGetPr(Mp_n_alpha);

  /* Call of the C function */
  if(MFAG_mdfg(nu_n,mu_n,epsilon,alpha,normflg,p_n_alpha,fg_n_alpha,&N)==0) 
  { 
    InterfError("mdfg: Call of the C function MFAG_mdfg failed\n"); 
    return;
  }

  /* Return the output */
  ReturnParam(Malpha);
  ReturnParam(Mfg_n_alpha);
  if(Interf.NbParamOut==3) 
     ReturnParam(Mp_n_alpha);
  else
     MatrixFree(Mp_n_alpha);
}


