/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MYFRACLAB/MFAL/SRC/LAB_mdfl2d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_mdfl2d.c,v 1.1.1.1 1999/02/10 13:40:16 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:54:23 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 16:54:30 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997 */

#include "C_LAB_Interf.h"
#include "MFAL_discrete.h"

/************  LAB_mdfl2d  ***********************************************/

void LAB_mdfl2d()
{ 
  int nux_n=0;                                       /* # of x intervals */ 
  int nuy_n=0;                                       /* # of y intervals */
  double *mu_n=NULL;                                          /* measure */
  short N=0;                                   /* # of Hoelder exponents */
  short n_min=0;                                   /* minimal resolution */
  double *alpha=NULL;                               /* Hoelder exponents */
  double *f_alpha=NULL;                             /* Legendre spectrum */

  Matrix *Mmu_n;   /* matrix pointing on the measure                     */
  Matrix *MN;      /* matrix pointing on the # of Hoelder exponents      */
  Matrix *Mn_min;  /* matrix pointing on the minimal resolution          */
  Matrix *Malpha;  /* matrix pointing on the Hoelder exponents           */
  Matrix *Mf_alpha;/* matrix pointing on the Legendre spectrum           */

  /* Check the input arguments number */
  if(Interf.NbParamIn!=3) 
  {
    InterfError("mdfl2d: You must give 3 input arguments\n");
    return;
  }
  
  /* Check the output arguments number */
  if(Interf.NbParamOut!=2)
  {
    InterfError("mdfl2d: You must give 2 output arguments\n");
    return;
  }
   
  /* Get the input measure mu_n and verify it */
  Mmu_n=Interf.Param[0];
  if(!MatrixIsNumeric(Mmu_n))
  {
    InterfError("mdfl2d: The measure mu_n must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(Mmu_n))
  {
    InterfError("mdfl2d: The measure mu_n must be a real matrix\n");
    return;
  }

  nux_n=(int)MatrixGetWidth(Mmu_n);
  nuy_n=(int)MatrixGetHeight(Mmu_n);

  /* Get the input argument */ 
  mu_n=MatrixGetPr(Mmu_n);
 
  /* Get the # of Hoelder exponents N and verify it */
  MN=Interf.Param[1];
  if(!MatrixIsNumeric(MN))
  {
    InterfError("mdfl2d: The # of Hoelder exponents N must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(MN))
  {
    InterfError("mdfl2d: The # of Hoelder exponents N must be a real matrix\n");
    return;
  }
  if(!MatrixIsScalar(MN))
  {
    InterfError("mdfl2d: The # of Hoelder exponents must be a scalar\n");
    return;
  }
  N=(short)MatrixGetScalar(MN);
 
  /* Get the minimal resolution n_min and verify it */
  Mn_min=Interf.Param[2];
  if(!MatrixIsNumeric(Mn_min))
  {
    InterfError("mdfl2d: The minimal resolution n_min must be a numeric matrix\n");
    return;
  }
  if(!MatrixIsReal(Mn_min))
  {
    InterfError("mdfl2d: The minimal resolution n_min must be a real matrix\n");
    return;
  }
  if(!MatrixIsScalar(Mn_min))
  {
    InterfError("mdfl2d: The minimal resolution n_min must be a scalar\n");
    return;
  }
  n_min=(short)MatrixGetScalar(Mn_min);
 
  /* Create the output Matrix */  
  Malpha=MatrixCreate(1,N,"real");
  Mf_alpha=MatrixCreate(1,N,"real");

  /* Catch the pointer on the output Matrix */  
  alpha=MatrixGetPr(Malpha);
  f_alpha=MatrixGetPr(Mf_alpha);
 
  /* Call of the C function */
  if(MFAL_mdfl2d(nux_n,nuy_n,mu_n,n_min,N,alpha,f_alpha)==0) 
  { 
    InterfError("mdfl2d: Call of the C function MFAL_mdfl2d failed\n"); 
    return;
  }

  /* Return the output */
  ReturnParam(Malpha);
  ReturnParam(Mf_alpha);

}


