/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mfd2d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:53:31 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 13:41:30 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_dimension.h"

/************  LAB_mfd2d  ****************************************************/

void LAB_mfd2d()
{
  /* input 2d capacity and measure */
  int Nx=0,Ny=0; 
  double* c2d=NULL;
  double* mu2d=NULL;
  /* Pointwise hoelder exponents estimation parameters */
  int J=0;
  double* S=NULL;
  MFAMt_capa t_capa=MFAM_SUM;
  /* lp measure parameter */
  double P=2.;
  /* iso capacity parameter */
  int Q=50; 
  /* dimension estimation parameters */
  double epsilon=0.; 
  int K=5;
  int b_ini=1,b_fin=33;
  double* B=NULL; 
  MFADt_dim t_dim=MFAD_BOX; 
  /* output pointwise hoelder exponents */
  double* p_alpha2d=NULL;
  /* ouput iso-hoelder set */
  unsigned char* iso_alpha2d=NULL;
  /* output spectrum */
  int N=100;
  double* alpha=NULL;
  double* fd2d_alpha=NULL;

  /* Matrix */
  Matrix* Mc2d;  
  Matrix* Mmu2d; 
  Matrix* MS;
  Matrix* Mcapa; 
  Matrix* Mepsilon; 
  Matrix* MB; 
  Matrix* Mdim;
  Matrix* MN;
  Matrix* Mp_alpha2d;
  Matrix* Malpha;
  Matrix* Mfd2d_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("mfd2d: You must give at least 1 input argument\n");
      return;
    }  
  if(Interf.NbParamIn>8) 
    {
      InterfError("mfd2d: You must give at most 8 input arguments\n");
      return;
    }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut<2) 
    {
      InterfError("mfd2d: You must give at least 2 output arguments\n");
      return;
    }
  if(Interf.NbParamOut>6) 
    {
      InterfError("mfd2d: You must give at most 3 output arguments\n");
      return;
    }
  /* Get the input analyzed image: c2d and verify it */
  Mc2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc2d)) 
    {
      InterfError("mfd2d: The input analyzed image: c2d must be a numeric matrix\n");
      return;
    }

  if(!MatrixIsReal(Mc2d)) 
    {
      InterfError("mfd2d: The input analyzed image: c2d must be a real matrix\n");
      return;
    }

  /* Get the input argument */  
  c2d=(double*)MatrixGetPr(Mc2d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mc2d);
  Ny=(int)MatrixGetHeight(Mc2d);
  if ((Nx<=0)||(Ny<=0))
    {
      InterfError("mfd2d: The size (width,height) of the input analyzed image: c2d must be >0\n");
      return;
    }

  if(Interf.NbParamIn>1) 
    {
      /* Get the capacity and verify it */
      Mcapa=Interf.Param[1];
      if (!MatrixIsString(Mcapa)) 
	{
	  InterfError("mfd2d : the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc2d);
	  return;
	}

      /* Get the input argument */
      if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
	{
	  InterfError("mfd2d : invalid input capacity\n");
	  return;
	}
    }
  /* else t_capa=MFAM_SUM */

  if(Interf.NbParamIn>2) 
    {
      /* Get the input sizes: S and verify it */
      MS=Interf.Param[2];
      if(!MatrixIsNumeric(MS))
	{
	  InterfError("mfd2d: The input sizes: S must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MS))
	{
	  InterfError("mfd2d: The input sizes: S must be a real matrix\n");
	  return;
	}
      if(((int)MIN(MatrixGetWidth(MS),MatrixGetHeight(MS)))!=1)
	{
	  InterfError("mfd2d: The input sizes: S must be a vector\n");
	  return;
	} 
      /* Get the size of the input argument */ 
      J=(int)MAX(MatrixGetWidth(MS),MatrixGetHeight(MS)); 
      if(J<1)
	{
	  InterfError("mfd2d: The length of the input sizes vector: S must be >0\n");
	  return;
	} 
      /* Get the input argument */
      S=MatrixGetPr(MS);
      if(*S>*(S+J-1))
	{
	  InterfError("mfd2d: The input sizes vector: S must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int j=0;
      int s_ini=1,s_fin=5;   
      J=(s_fin-s_ini)/2+1; 
      /* dynamic memory allocation */
      if((S=(double*)malloc((size_t)(J*sizeof(double))))==NULL)
	{
	  InterfError("mfd1d: malloc error\n");
	  return;
	}
      for (j=0;j<J;j++)
	*(S+j)=s_ini+j*2;
    }

  if(Interf.NbParamIn>3) 
    { 
      /* Get the input precision: epsilon and verify it */
      Mepsilon=Interf.Param[3];
      if(!MatrixIsNumeric(Mepsilon)) 
	{
	  InterfError("mfd2d: The input precision: epsilon must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mepsilon)) 
	{
	  InterfError("mfd2d: The input precision: epsilon must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mepsilon))
	{
	  InterfError("mfd2d: The input precision: epsilon must be a scalar\n");
	  return;
	}

      /* Get the input argument */  
      epsilon=(double)MatrixGetScalar(Mepsilon);
    }
  /* else epsilon=.1; */

  if(Interf.NbParamIn>4) 
    {
      /* Get the input box sizes: B and verify it */
      MB=Interf.Param[4];

      if(!MatrixIsNumeric(MB))
	{
	  InterfError("mfd2d: The input box sizes: B must be a numeric matrix\n");
	  return;
	}

      if(!MatrixIsReal(MB))
	{
	  InterfError("mfd2d: The input box sizes: B must be a real matrix\n");
	  return;
	}

      if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
	{
	  InterfError("mfd2d: The input box sizes: B must be a vector\n");
	  return;
	} 

      /* Get the size of the input argument */ 
      K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
      if(K<1)
	{
	  InterfError("mfd2d: The length of the input box sizes vector: B must be >0\n");
	  return;
	} 
 
      /* Get the input argument */
      B=MatrixGetPr(MB);
      if(*B>*(B+K-1))
	{
	  InterfError("mfd2d: The input box sizes vector: B must be increasing\n");
	  return;
	}
    }
  else
    { 
      register int k=0;
      int b_ini=1,b_fin=33;   
      K=(b_fin-b_ini)/2+1; 
      /* dynamic memory allocation */
      if((B=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
	{
	  InterfError("mfd2d: malloc error\n");
	  return;
	}
      for (k=0;k<K;k++)
	*(B+k)=b_ini+k*2;
    }

  if(Interf.NbParamIn>5) 
    { 
      /* Get the dimension and verify it */
      Mdim=Interf.Param[5];
      if (!MatrixIsString(Mdim)) 
	{
	  InterfError("mfd2d: the input capacity: capa must be a string\n");
	  MatrixTranspose(Mc2d);
	  return;
	}

      /* Get the input argument */
      if(MFAD_dimstr(MatrixReadString(Mdim),&t_dim)==0) 
	{
	  InterfError("mfd2d: invalid input capacity\n");
	  return;
	}
    } 
  /* else t_dim=MFAH_BOX */
  
  if(Interf.NbParamIn>6) 
    { 
      /* Get the input reference image: mu2d */
      Mmu2d=Interf.Param[6];
      if(!MatrixIsNumeric(Mmu2d)) 
	{
	  InterfError("mfd2d : The input reference image: mu2d must be a numeric matrix\n");
	  return;
	}
    
      if(!MatrixIsReal(Mmu2d)) 
	{
	  InterfError("mfd2d: The input reference image: mu2d must be a real matrix\n");
	  return;
	}
  
      if((Nx!=(int)MatrixGetWidth(Mmu2d))||(Ny!=(int)MatrixGetHeight(Mmu2d)))
	{
	  InterfError("mfd2d: The size (width,height) of the input analysed image: c2d and reference image: mu2d must be =\n");
	  return;
	}

      /* Get the input argument */  
      mu2d=MatrixGetPr(Mmu2d);
    }
  /* else mu2d=NULL */
  
  if(Interf.NbParamIn>7) 
  { 
    /* Get the input # of Hoelder exponents: N and verify it */
    MN=Interf.Param[7];
    if(!MatrixIsNumeric(MN)) 
    {
      InterfError("mfd2d: The input # of Hoelder exponents: N must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(MN)) 
    {
      InterfError("mfd2d: The input # of Hoelder exponents: N must be a real matrix\n");
      return;
    }
    if(!MatrixIsScalar(MN))
      {
	InterfError("mfd2d: The input # of Hoelder exponents: N must be a scalar\n");
	return;
      }  

    /* Get the input argument */  
    N=(int)MatrixGetScalar(MN);
  } 
  /* else N=100 */

  /* Create the output Matrix */  
  Mp_alpha2d=MatrixCreate(Nx,Ny,"real"); 
  p_alpha2d=MatrixGetPr(Mp_alpha2d); 
  Malpha=MatrixCreate(N,1,"real"); 
  alpha=MatrixGetPr(Malpha); 
  Mfd2d_alpha=MatrixCreate(N,1,"real"); 
  fd2d_alpha=MatrixGetPr(Mfd2d_alpha);

  /* Transpose Matrix */
  MatrixTranspose(Mc2d);

  if(Interf.NbParamIn>6)
    MatrixTranspose(Mmu2d); 
  
  /* dynamic memory allocation */
  if((iso_alpha2d=(unsigned char*)malloc((size_t)(Nx*Ny*sizeof(unsigned char))))==NULL)
    {
      InterfError("mfd2d: malloc error\n");
      return;
    }

  /* Call of the C function */
  if(MFAH_mfd2d(Nx,Ny,c2d,mu2d,J,S,t_capa,P,Q,p_alpha2d,epsilon,iso_alpha2d,K,B,t_dim,N,alpha,fd2d_alpha)==0)
    { 
      InterfError("mfd2d: Call of the C function MFAH_mfd2d failed\n"); 
      return; 
    }

  /* dynamic memory deallocation */
  free((char*)iso_alpha2d);

  if (Interf.NbParamIn<=2) 
    free((char*)S); 

  if (Interf.NbParamIn<=4) 
    free((char*)B);

  /* Transpose input Matrixes */
  MatrixTranspose(Mc2d);
  if(Interf.NbParamIn>6) 
    MatrixTranspose(Mmu2d);

  /* Transpose output Matrixes */
  MatrixTranspose(Mp_alpha2d);
  
  /* Return the output */  
  ReturnParam(Malpha); 
  ReturnParam(Mfd2d_alpha);  

  if(Interf.NbParamOut>2) 
    ReturnParam(Mp_alpha2d);
  else
    MatrixFree(Mp_alpha2d); 
}
