/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mfdls2d.c
* File type        : C source file
* Revision         : 
* RCS status       : 
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sat Jul 20 17:17:18 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Tue Sep 17 16:42:13 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAH_mfdlimsup2d.h"

/************  LAB_mfdls2d  ****************************************************/

void LAB_mfdls2d()
{ 
  /* input 2d capacity and measure */
  int Nx=0,Ny=0;
  double* c2d=NULL;
  double* mu2d=NULL;
  /* input  arguments */ 
  short n_ini=0;
  short n_fin=0;
  MFAMt_capa t_capa=MFAM_SUM;
  double epsilon=.1;
  /* dimension estimation parameters */
  double* B=NULL;
  int K=11;
  int b_ini=1,b_fin=33;  
  MFADt_dim t_dim=MFAD_BOX; 
  /* output spectrum */ 
  int N=100;
  double* alpha=NULL;
  double* fdls2d_alpha=NULL;

  /* Matrix */
  Matrix* Mc2d; 
  Matrix* Mmu2d; 
  Matrix* Mn;  
  double* n; 
  Matrix* Mcapa;
  Matrix* Mepsilon; 
  Matrix* MB; 
  Matrix* Mdim;
  Matrix* MN;
  Matrix* Malpha;
  Matrix* Mfdls2d_alpha;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
  {
    InterfError("mfdls2d: You must give at least 1 input argument\n");
    return;
  }  
  if(Interf.NbParamIn>8) 
  {
    InterfError("mfdls2d: You must give at most 8 input arguments\n");
    return;
  }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=2) 
  {
    InterfError("mfdls2d: You must give 2 output argument\n");
    return;
  }

  /* Get the input analyzed image: c2d and verify it */
  Mc2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc2d)) 
  {
    InterfError("mfdls2d: The input analyzed image: c2d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mc2d)) 
  {
    InterfError("mfdls2d: The input analyzed image: c2d must be a real matrix\n");
    return;
  }

  /* Get the input argument */  
  c2d=(double*)MatrixGetPr(Mc2d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mc2d);
  Ny=(int)MatrixGetHeight(Mc2d);
  if ((Nx<=0)||(Ny<=0))
  {
    InterfError("mfdls2d: The size (width,height) of the input analyzed image: c2d must be >0\n");
    return;
  }

  if(Interf.NbParamIn>1) 
  {
    /* Get the input resolutions: n and verify it */
    Mn=Interf.Param[1];

    if(!MatrixIsNumeric(Mn))
      {
	InterfError("mfdls2d: The input resolutions: n must be a numeric matrix\n");
	return;
      }

    if(!MatrixIsReal(Mn))
      {
	InterfError("mfdls2d: The input resolutions: n must be a real matrix\n");
	return;
      }

    if(((int)MIN(MatrixGetWidth(Mn),MatrixGetHeight(Mn)))!=1)
      {
	InterfError("mfdls2d: The input resolutions: n must be a vector\n");
	return;
      } 

    /* Get the size of the input argument */ 
    K=(int)MAX(MatrixGetWidth(Mn),MatrixGetHeight(Mn)); 
    if(K!=2)
      {
	InterfError("mfdls2d: The length of the input resolutions: n vector must be = 2\n");
	return;
      } 
 
    /* Get the input argument */
    n=MatrixGetPr(Mn);
    if(*n>*(n+1))
      {
	InterfError("mfdls2d: The input box sizes vector: B must be increasing\n");
	return;
      }

    /* Get the input argument */  
    n_ini=(int)*n;
    n_fin=(int)*(n+1);
  }
  /* else n_ini=0; n_fin=0; */

  if(Interf.NbParamIn>2) 
  { 
    /* Get the capacity and verify it */
    Mcapa=Interf.Param[2];
    if (!MatrixIsString(Mcapa)) 
    {
      InterfError("mfdls2d: the input capacity: capa must be a string\n");
      MatrixTranspose(Mc2d);
      return;
    }

    /* Get the input argument */
    if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
    {
      InterfError("mfdls2d: invalid input capacity\n");
      return;
    }
  }
  /* else t_capa=MFAM_SUM */
 
  if(Interf.NbParamIn>3) 
  { 
    /* Get the input precision: epsilon and verify it */
    Mepsilon=Interf.Param[3];
    if(!MatrixIsNumeric(Mepsilon)) 
    {
      InterfError("mfdls2d: The input precision: epsilon must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(Mepsilon)) 
    {
      InterfError("mfdls2d: The input precision: epsilon must be a real matrix\n");
      return;
    }
    if(!MatrixIsScalar(Mepsilon))
      {
	InterfError("mfdls2d: The input precision: epsilon must be a scalar\n");
	return;
      }

    /* Get the input argument */  
    epsilon=(double)MatrixGetScalar(Mepsilon);
  }
  /* else epsilon=.1; */

  if(Interf.NbParamIn>4) 
  {
    /* Get the input box sizes: B and verify it */
    MB=Interf.Param[4];

    if(!MatrixIsNumeric(MB))
      {
	InterfError("mfdls2d: The input box sizes: B must be a numeric matrix\n");
	return;
      }

    if(!MatrixIsReal(MB))
      {
	InterfError("mfdls2d: The input box sizes: B must be a real matrix\n");
	return;
      }

    if(((int)MIN(MatrixGetWidth(MB),MatrixGetHeight(MB)))!=1)
      {
	InterfError("mfdls2d: The input box sizes: B must be a vector\n");
	return;
      } 

    /* Get the size of the input argument */ 
    K=(int)MAX(MatrixGetWidth(MB),MatrixGetHeight(MB)); 
    if(K<1)
      {
	InterfError("mfdls2d: The length of the input box sizes vector: B must be >0\n");
	return;
      } 
 
    /* Get the input argument */
    B=MatrixGetPr(MB);
    if(*B>*(B+K-1))
      {
	InterfError("mfdls2d: The input box sizes vector: B must be increasing\n");
	return;
      }
  }
  else
    { 
      register int k=0;
      int b_ini=1,b_fin=33;   
      K=(b_fin-b_ini)/2+1; 
      /* dynamic memory allocation */
      if((B=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
	{
	  InterfError("mfdls2d: malloc error\n");
	  return;
	}
      for (k=0;k<K;k++)
	*(B+k)=b_ini+k*2;
    }

  if(Interf.NbParamIn>5) 
  { 
    /* Get the dimension and verify it */
    Mdim=Interf.Param[5];
    if (!MatrixIsString(Mdim)) 
    {
      InterfError("mfdls2d: the input capacity: capa must be a string\n");
      MatrixTranspose(Mc2d);
      return;
    }

    /* Get the input argument */
    if(MFAD_dimstr(MatrixReadString(Mdim),&t_dim)==0) 
    {
      InterfError("mfdls2d: invalid input capacity\n");
      return;
    }
  } 
  /* else t_dim=MFAH_BOX */

  if(Interf.NbParamIn>6) 
  { 
    /* Get the input reference image: mu2d */
    Mmu2d=Interf.Param[6];
    if(!MatrixIsNumeric(Mmu2d)) 
    {
      InterfError("mfdls2d: The input reference image: mu2d must be a numeric matrix\n");
      return;
    }
    
    if(!MatrixIsReal(Mmu2d)) 
    {
      InterfError("mfdls2d: The input reference image: mu2d must be a real matrix\n");
      return;
    }
  
    if ((Nx!=(int)MatrixGetWidth(Mmu2d))||(Ny!=(int)MatrixGetHeight(Mmu2d)))
    {
      InterfError("mfdls2d: The size (width,height) of the input analysed image: c2d and reference image: mu2d must be =\n");
      return;
    }

    /* Get the input argument */  
    mu2d=MatrixGetPr(Mmu2d);
  }
  /* else mu2d=NULL */

  if(Interf.NbParamIn>7) 
  { 
    /* Get the input # of Hlder exponents: N and verify it */
    MN=Interf.Param[7];
    if(!MatrixIsNumeric(MN)) 
    {
      InterfError("mfdls2d: The input # of Hoelder exponents: N must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(MN)) 
    {
      InterfError("mfdls2d: The input # of Hoelder exponents: N must be a real matrix\n");
      return;
    }
    if(!MatrixIsScalar(MN))
      {
	InterfError("mfdls2d: The input # of Hoelder exponents: N must be a scalar\n");
	return;
      }  

    /* Get the input argument */  
    N=(int)MatrixGetScalar(MN);
  } 
  /* else N=100 */

  /* Create the output Matrixes */ 
  Malpha=MatrixCreate(N,1,"real"); 
  alpha=MatrixGetPr(Malpha);
  Mfdls2d_alpha=MatrixCreate(N,1,"real"); 
  fdls2d_alpha=MatrixGetPr(Mfdls2d_alpha);
  
  /* Transpose input Matrixes */
  MatrixTranspose(Mc2d);
 
  if(Interf.NbParamIn>6) 
     MatrixTranspose(Mmu2d);

  /* Call of the C function */
  if(MFAH_mfdls2d(Nx,Ny,c2d,mu2d,n_ini,n_ini,t_capa,epsilon,K,B,t_dim,N,alpha,fdls2d_alpha)==0)
  { 
    InterfError("mfdls2d: Call of the C function MFAH_mfdls2d failed\n"); 
    return; 
  }

  /* Transpose input Matrixes */
  MatrixTranspose(Mc2d);
  
  if(Interf.NbParamIn>6)
     MatrixTranspose(Mmu2d);

  /* dynamic memory deallocation */
  if (Interf.NbParamIn<=3) 
    free((char*)B);

  /* Return the output */  
  ReturnParam(Malpha); 
  ReturnParam(Mfdls2d_alpha);

}

