/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAH/SRC/LAB_mph2d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_mph2d.c,v 1.1.1.1 1999/02/10 13:39:49 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:53:27 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 16:14:17 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Frederic Raynal, Christophe Canus 1998-2002 */

#include "C_LAB_Interf.h"
#include "MFAH_hoelder2d.h"

void LAB_mph2d()
{ 
  /* input capacity and reference measure  */
  int Nx=0,Ny=0; 
  double* c2d=NULL;
  double* mu2d=NULL;
  /* input parameters */
  int J=0;
  double* S=NULL;
  MFAMt_capa t_capa=MFAM_SUM;
  /* lp measure parameter */
  double P=2.; 
  /* iso capacity parameter */
  int Q=50;

  /* ouput hoelder exponents */
  double* p_alpha2d=NULL;

  /* Matrix */
  Matrix* Mc2d; 
  Matrix* Mmu2d;
  Matrix* MS;
  Matrix* Mcapa;
  Matrix* Mp_alpha2d;

  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
  {
    InterfError("mph2d: You must give at least 1 input argument\n");
    return;
  }  
  if(Interf.NbParamIn>4) 
  {
    InterfError("mph2d: You must give at most 4 input arguments\n");
    return;
  }

  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
  {
    InterfError("mph2d: You must give 1 output argument\n");
    return;
  }

  /* Get the input analyzed image: c2d and verify it */
  Mc2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mc2d)) 
  {
    InterfError("mph2d: The input analyzed image: c2d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mc2d)) 
  {
    InterfError("mph2d: The input analyzed image: c2d must be a real matrix\n");
    return;
  }

  /* Get the input argument */  
  c2d=(double*)MatrixGetPr(Mc2d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mc2d);
  Ny=(int)MatrixGetHeight(Mc2d);
  if ((Nx<=0)||(Ny<=0))
  {
    InterfError("mph2d: The size (width,height) of the input analyzed image: c2d must be >0\n");
    return;
  }

  if(Interf.NbParamIn>1) 
  {
    /* Get the capacity and verify it */
    Mcapa=Interf.Param[1];
    if(!MatrixIsString(Mcapa)) 
    {
      InterfError("mph2d: the input capacity: capa must be a string\n");
      return;
    }

    /* Get the input argument */
    if(MFAM_capastr(MatrixReadString(Mcapa),&t_capa)==0) 
      {
	InterfError("mph2d: invalid input capacity\n");
	return;
      }
  }
  /* else t_capa=MFAM_SUM */

  if(Interf.NbParamIn>2) 
  {
    /* Get the input sizes: S and verify it */
    MS=Interf.Param[2];
    if(!MatrixIsNumeric(MS))
    {
      InterfError("mph2d: The input sizes: S must be a numeric matrix\n");
      return;
    }
    if(!MatrixIsReal(MS))
    {
      InterfError("mph2d: The input sizes: S must be a real matrix\n");
      return;
    }
    if(((int)MIN(MatrixGetWidth(MS),MatrixGetHeight(MS)))!=1)
    {
      InterfError("mph2d: The input sizes: S must be a vector\n");
      return;
    } 
    /* Get the size of the input argument */ 
    J=(int)MAX(MatrixGetWidth(MS),MatrixGetHeight(MS)); 
    if(J<1)
    {
      InterfError("mph2d: The length of the input sizes vector: S must be >0\n");
      return;
    } 
    /* Get the input argument */
    S=MatrixGetPr(MS);
    if(*S>*(S+J-1))
    {
      InterfError("mph2d: The input sizes vector: S must be increasing\n");
      return;
    }
  }
  else
    { 
      register int j=0;
      int s_ini=1,s_fin=5;   
      J=(s_fin-s_ini)/2+1; 
      /* dynamic memory allocation */
      if((S=(double*)malloc((unsigned int)(J*sizeof(double))))==NULL)
	{
	  InterfError("mfd1d: malloc error\n");
	  return;
	}
      for (j=0;j<J;j++)
	*(S+j)=s_ini+j*2;
    }

  if(Interf.NbParamIn>3) 
  { 
    /* Get the input reference image: mu2d */
    Mmu2d=Interf.Param[3];
    if(!MatrixIsNumeric(Mmu2d)) 
    {
      InterfError("mph2d: The input reference image: mu2d must be a numeric matrix\n");
      return;
    }
    
    if(!MatrixIsReal(Mmu2d)) 
    {
      InterfError("mph2d: The input reference image: mu2d must be a real matrix\n");
      return;
    }
  
    if ((Nx!=(int)MatrixGetWidth(Mmu2d))||(Ny!=(int)MatrixGetHeight(Mmu2d)))
    {
      InterfError("mph2d: The size (width,height) of the input analysed image: c2d and reference image: mu2d must be =\n");
      return;
    }

    /* Get the input argument */  
    mu2d=MatrixGetPr(Mmu2d);
  }
  /* else mu2d=NULL */

  /* Create the output Matrix */  
  Mp_alpha2d=MatrixCreate(Nx,Ny,"real"); 
  p_alpha2d=MatrixGetPr(Mp_alpha2d);
 
  /* Transpose Matrix */
  MatrixTranspose(Mc2d);
  if(Interf.NbParamIn>3) 
    MatrixTranspose(Mmu2d); 
  
  /* Call of the C function */
  if(MFAH_mph2d(Nx,Ny,c2d,mu2d,J,S,t_capa,P,Q,p_alpha2d)==0)
  { 
    InterfError("mph2d: Call of the C function MFAH_mph2d failed\n");
    return; 
  }

  /* dynamic memory deallocation */
  if (Interf.NbParamIn<=2) 
    free((char*)S); 

  /* Transpose input Matrixes */
  MatrixTranspose(Mc2d);
  if(Interf.NbParamIn>3) 
    MatrixTranspose(Mmu2d);

  /* Transpose output Matrix */
  MatrixTranspose(Mp_alpha2d);
  
  /* Return the output */
  ReturnParam(Mp_alpha2d);
}
