/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAM/SRC/LAB_msegmentation2d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: LAB_msegmentation2d.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:50:30 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Aug 11 19:22:06 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "C_LAB_Interf.h"
#include "MFAM_segmentation.h"

void LAB_msegmentation2d()
{
  /* input parameters */
  double f_alpha0=2.;
  double Delta_f_alpha0=.05;
  /* input spectrum */
  int N=0;
  double* alpha=NULL;
  double* f2d_alpha=NULL;
  /* input pointwise Hoelder exponents image */
  int Nx=0,Ny=0;
  double* p_alpha2d=NULL;
  /* ouput image */
  register int k=0;
  unsigned char* ucmsegmentation2d=NULL;
  double* msegmentation2d=NULL;

  /* Matrix */
  Matrix* Mp_alpha2d;
  Matrix* Malpha;
  Matrix* Mf2d_alpha; 
  Matrix* Mf_alpha0; 
  Matrix* MDelta_f_alpha0=0;
  Matrix* Mmsegmentation2d; 

  /* Check the input arguments number */
  if(Interf.NbParamIn<3) 
  {
    InterfError("msegmentation2d: You must a least 3 input arguments\n");
    return;
  }   
  if(Interf.NbParamIn>5) 
  {
    InterfError("msegmentation2d: You must a least 5 input arguments\n");
    return;
  } 
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
  {
    InterfError("msegmentation2d: You must give 1 output argument\n");
    return;
  }

  /* Get the input pointwise Hoelder exponents image: p_alpha2d and verify it */
  Mp_alpha2d=Interf.Param[0];
  if(!MatrixIsNumeric(Mp_alpha2d)) 
  {
    InterfError("msegmentation2d : The input pointwise Hoelder exponents image: p_alpha2d must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mp_alpha2d)) 
  {
    InterfError("msegmentation2d : The input pointwise Hoelder exponents image: p_alpha2d must be a real matrix\n");
    return;
  }

  /* Get the input argument */  
  p_alpha2d=(double*)MatrixGetPr(Mp_alpha2d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MatrixGetWidth(Mp_alpha2d);
  Ny=(int)MatrixGetHeight(Mp_alpha2d);
  if ((Nx<=0)||(Ny<=0))
  {
    InterfError("msegmentation2d : The size (width,height) of the input pointwise Hoelder exponents image: p_alpha2d must be >0\n");
    return;
  }

  /* Get the input spectrum graph abscissa: alpha and verify it */
  Malpha=Interf.Param[1];
  if(!MatrixIsNumeric(Malpha))
  {
    InterfError("msegmentation2d: The input spectrum graph abscissa: alpha must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Malpha))
  {
    InterfError("msegmentation2d: The input spectrum graph abscissa: alpha must be a real matrix\n");
    return;
  } 

  if(((int)MIN(MatrixGetWidth(Malpha),MatrixGetHeight(Malpha)))!=1)
  {
    InterfError("msegmentation2d: The input spectrum graph abscissa: alpha must be a vector\n");
    return;
  } 

  /* Get the size of the input argument */ 
  N=(int)MAX(MatrixGetWidth(Malpha),MatrixGetHeight(Malpha)); 
  if(N<1)
  {
    InterfError("msegmentation2d: The length of the input spectrum graph abscissa: alpha must be >0\n");
    return;
  }
 
  /* Get the input argument */  
  alpha=(double*)MatrixGetPr(Malpha);
 
  /* Get the input spectrum graph ordinates: f2d_alpha and verify it */
  Mf2d_alpha=Interf.Param[2];
  if(!MatrixIsNumeric(Mf2d_alpha))
  {
    InterfError("msegmentation2d: The input spectrum graph ordinates: f2d_alpha must be a numeric matrix\n");
    return;
  }

  if(!MatrixIsReal(Mf2d_alpha))
  {
    InterfError("msegmentation2d: The input spectrum graph ordinates: f2d_alpha must be a real matrix\n");
    return;
  } 

  if(((int)MIN(MatrixGetWidth(Mf2d_alpha),MatrixGetHeight(Mf2d_alpha)))!=1)
  {
    InterfError("msegmentation2d: The input spectrum graph ordinates: f2d_alpha must be a vector\n");
    return;
  } 
 
  if((int)MAX(MatrixGetWidth(Mf2d_alpha),MatrixGetHeight(Mf2d_alpha))!=N) 
  {
    InterfError("msegmentation2d: The input spectrum graph abscissa: alpha and ordinates: f2d_alpha must be same length\n");
    return;
  } 

  /* Get the input argument */  
  f2d_alpha=(double*)MatrixGetPr(Mf2d_alpha);

  if(Interf.NbParamIn>3)
  {
    /* Get the input point type: f_alpha0 and verify it */
    Mf_alpha0=Interf.Param[3];

    if(!MatrixIsNumeric(Mf_alpha0))
    {
      InterfError("msegmentation2d: The point type: f_alpha0 must be a numeric matrix\n");
      return;
    }  
    if(!MatrixIsScalar(Mf_alpha0))
    {
      InterfError("msegmentation2d: The point type: f_alpha0 must be a scalar\n");
      return;
    }
    /* Get the input argument */
    f_alpha0=(int)MatrixGetScalar(Mf_alpha0);
    if((f_alpha0<0.)||(f_alpha0>2.))
    {
      InterfError("msegmentation2d: The point type: f_alpha0 must be >0. and <2.\n");
      return;
    }
  }

  if(Interf.NbParamIn>4)
  { 
    /* Get the input spread: Delta_f_alpha0 and verify it */
    MDelta_f_alpha0=Interf.Param[4];

    if(!MatrixIsNumeric(MDelta_f_alpha0))
    {
      InterfError("msegmentation2d: The spread: Delta_f_alpha0 must be a numeric matrix\n");
      return;
    }

    if(!MatrixIsScalar(MDelta_f_alpha0))
    {
      InterfError("msegmentation2d: The spread: Delta_f_alpha0 must be a scalar\n");
      return;
    }

    /* Get the input argument */  
    Delta_f_alpha0=MatrixGetScalar(MDelta_f_alpha0);   
    if(Delta_f_alpha0<=0.)
    {
      InterfError("msegmentation2d: The spread: Delta_f_alpha0 must be >0.\n");
      return;
    }
  }
   
 
  /* Transpose input Matrix */
  MatrixTranspose(Mp_alpha2d);
  
   /* memory allocation */  
   if((ucmsegmentation2d=(unsigned char*)malloc((unsigned int)(Nx*Ny*sizeof(unsigned char))))==NULL)
   {
     InterfError("msegmentation2d: malloc error\n");
     return;
   }

   /* Call of the C function */
  if(MFAM_msegmentation2d(Nx,Ny,p_alpha2d,N,alpha,f2d_alpha,f_alpha0,Delta_f_alpha0,ucmsegmentation2d)==0)
  { 
    InterfError("msegmentation2d: Call of the C function MFAH_msegmentation2d failed\n"); 
    return; 
  }

  /* Transpose input Matrix */
  MatrixTranspose(Mp_alpha2d);
 
  /* Create the output Matrix */  
  Mmsegmentation2d=MatrixCreate(Nx,Ny,"real"); 
  msegmentation2d=MatrixGetPr(Mmsegmentation2d);
     
  for(k=0;k<Nx*Ny;k++)
    *(msegmentation2d+k)=(unsigned char)*(ucmsegmentation2d+k);
	
  /* Transpose output Matrix */
  MatrixTranspose(Mmsegmentation2d);
  
  /* Return the output */
  ReturnParam(Mmsegmentation2d);
}
