#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "C_LAB_Interf.h"
#include "McCulloch.h"
#include "Koutrouvelis.h"
#include "stable_sm.h"

#ifndef sign
#define sign(z) (z<0 ? (-1.0) : (1.0))
#endif
#ifndef M_PI
#define M_PI 3.1415927
#endif
#ifndef ROUND
#define ROUND(x) ((((x)-floor((double)(x)))<0.5)?floor((double)(x)):(ceil((double)(x))))
#endif 

void LAB_spectral_mesure()
{
   /* Input */ 
  
 Matrix *Mn, *Malpha, *Mdata1, *Mdata2;
 int    n;
 double *data1, *data2; 
 double alpha;

   /* Output */

 Matrix *Mtheta,  *MNSM;
 double *theta, *NSM;

    /* Work */
 int i, k, kopt,n2;
 double *rho, *rhoo, *valpha, mindiff;
 double PA_STAT,PA_SIGMA,NA_STAT,NA_SIGMA,T_PA_STATISTIC,T_NA_STATISTIC,COV;
   
   Mdata1 = (Matrix *)(Interf.Param[0]);
   Mdata2 = (Matrix *)(Interf.Param[1]);
   Mn = (Matrix *)(Interf.Param[2]);
   Malpha = (Matrix *)(Interf.Param[3]);
   

   /* get and verify the input parameters */

   data1 = MatrixGetPr(Mdata1);
   data2 = MatrixGetPr(Mdata2);
   n = (int)MatrixGetScalar(Mn);
   alpha = MatrixGetScalar(Malpha);

  if (Interf.NbParamIn != 4)
    {
      InterfError("spectral_mesure : you must give 4 input parameters\n");
      return;
    }
  n2= n/2;

  MNSM = MatrixCreate(100,1,"real");
  NSM = MatrixGetPr(MNSM);

  Mtheta = MatrixCreate(100,1,"real");
  theta = MatrixGetPr(Mtheta);

/*************************Allocation de memoire*****************************/	
	rho = (double *) malloc (sizeof (double) * n);
	rhoo = (double *) malloc (sizeof (double) * n);
	valpha = (double *) malloc (sizeof (double) * (n2-3));
/*************************On calcule le vecteur du module*******************/
	 for(i=0;i<n;i++) {
                rho[i] = stable_modu(data1,data2,i);
                rhoo[i]=rho[i];
        }
        stable_sort(n,rhoo);

/***********************finding optimal k *********************************/
	for(k=0;k< (n2-3);k++) {
                valpha[k]=stable_alpha(rhoo, k+3, n);
		
	}
	stable_root(valpha, n2-3, alpha, &mindiff, &kopt);
	
	kopt=kopt+3;
/***************************************************************************/

	for (i=0;i<100;i++){
	  NSM[i]= stable_mspect(data1,data2,n,2.0*i*M_PI/100,rhoo,kopt);
	  theta[i]=2.0*i*M_PI/100;
	}
/***************************************************************************/
	              /*Test for positive Association*/
/***************************************************************************/
 PA_STAT  = (stable_mspect(data1,data2,n,M_PI,rhoo,kopt)
	     -stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt)
	     -stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt)+1.0);

 PA_SIGMA=sqrt(stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt) 
	       - pow(stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt),2.0)
	       - stable_mspect(data1,data2,n,M_PI,rhoo,kopt) 
	       - pow(stable_mspect(data1,data2,n,M_PI,rhoo,kopt),2.0)
	       + stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt) 
	       - pow(stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt),2.0)
	       + 2.0 * stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt)
	       * stable_mspect(data1,data2,n,M_PI,rhoo,kopt)
	       - 2.0 * stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt) 
	       * stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt)
	       + 2.0 * stable_mspect(data1,data2,n,M_PI,rhoo,kopt) 
	       * stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt) + 1.0);

 T_PA_STATISTIC = sqrt(kopt)*(PA_STAT)/PA_SIGMA ;

/***************************************************************************/
	              /*Test for negative Association*/
/***************************************************************************/

 /*NA_STAT  = (stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt)
	     +stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt)
	     -stable_mspect(data1,data2,n,M_PI,rhoo,kopt));
 
 NA_SIGMA =(stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt) 
		- pow(stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt),2.0)
		+ stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt) 
		- pow(stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt),2.0)
		- stable_mspect(data1,data2,n,M_PI,rhoo,kopt)
	        - pow(stable_mspect(data1,data2,n,M_PI,rhoo,kopt),2.0)
		+ 2.0 * stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt) 
		* stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt)
		- 2.0 * stable_mspect(data1,data2,n,M_PI,rhoo,kopt) 
		* stable_mspect(data1,data2,n,M_PI/2.0,rhoo,kopt) 
		- 2.0 * stable_mspect(data1,data2,n,3.0*M_PI/2.0,rhoo,kopt) 
		* stable_mspect(data1,data2,n,M_PI,rhoo,kopt));
	
 T_NA_STATISTIC = sqrt(kopt)*(NA_STAT)/NA_SIGMA ;*/

 printf("%d %lf\n",kopt,T_PA_STATISTIC);
/***************************************************************************/

 COV = stable_cov(data1,data2,n,rhoo,kopt,alpha);
 printf(" %lf\n",COV);
	ReturnParam(Mtheta);
	ReturnParam(MNSM);
	
}

/***************************************************************************/




