/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAD/SRC/LAB_srandcover1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Wed Sep 11 13:58:48 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Sep 11 18:03:43 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "C_LAB_Interf.h"
#include "MFAD_hausdorff.h"

void LAB_srandcover1d()
{
  register int k=0;
  /* input 1d set */
  int Nx=0;  
  double* s1d=NULL;
  unsigned char* ucs1d=NULL;
  /* input parameters */  
  int rooted=1; 
  short n_fin=0;
  int M=10000;  
  /* input bernoulli parameters */ 
  int P=21;
  double p_ini=0.;
  double p_fin=1.; 
  double* p=NULL;
  /* output probability */
  double* x_n_p=NULL;
  short n_max=0;
  short b_opt=0;
 
  /* Matrix */
  Matrix* Ms1d;
  Matrix* Mn_fin;  
  Matrix* MM; 
  Matrix* Mp;
  Matrix* Mrooted;  
  Matrix* Mx_n_p; 
 
  /* Check the input arguments number */
  if(Interf.NbParamIn<1) 
    {
      InterfError("srandcover1d: You must give at least 1 input argument\n");
      return;
    }
  
   if(Interf.NbParamIn>5) 
    {
      InterfError("srandcover1d: You must give at most 5 input argument\n");
      return;
    }   
  
  /* Check the output arguments number */ 
  if(Interf.NbParamOut!=1) 
    {
      InterfError("srandcover1d: You must give 1 output argument\n");
      return;
    }

  /* Get the input 1d set image: s1d and verify it */
  Ms1d=Interf.Param[0];
  if(!MatrixIsNumeric(Ms1d)) 
    {
      InterfError("srandcover1d : The input 1d set image: s1d must be a numeric matrix\n");
      return;
    }

  if(!MatrixIsReal(Ms1d)) 
    {
      InterfError("srandcover1d : The input 1d set image: s1d must be a real matrix\n");
      return;
    }

  /* Get the input argument */  
  s1d=(double*)MatrixGetPr(Ms1d);
 
  /* Get the size of the input argument */ 
  Nx=(int)MAX(MatrixGetWidth(Ms1d),MatrixGetHeight(Ms1d));
  if(Nx<=0)
    {
      InterfError("srandcover1d : The size of the input 1d set image: s1d must be >0\n");
      return;
    }

  if(Interf.NbParamIn>1)
    { 
      /* Get the input final resolution: n_fin and verify it */
      Mn_fin=Interf.Param[1];
      if(!MatrixIsNumeric(Mn_fin))
	{
	  InterfError("srandcover1d: The input final resolution: n_fin must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mn_fin))
	{
	  InterfError("srandcover1d: The input final resolution: n_fin must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mn_fin))
	{
	  InterfError("srandcover1d: The input final resolution: n_fin must be a scalar\n");
	  return;
	}
      /* Get the input argument */ 
      n_fin=(int)MatrixGetScalar(Mn_fin);
    }
  /* else n_fin=n_max */

  if(Interf.NbParamIn>2)
    { 
      /* Get the input number of realizations: M and verify it */
      MM=Interf.Param[2];
      if(!MatrixIsNumeric(MM))
	{
	  InterfError("srandcover1d: The input number of realizations: M must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(MM))
	{
	  InterfError("srandcover1d: The input number of realizations: M must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(MM))
	{
	  InterfError("srandcover1d: The input number of realizations: M must be a scalar\n");
	  return;
	} 
      /* Get the input argument */ 
      M=(int)MatrixGetScalar(MM);
    }
  /* else M=10000 */
  
  if(Interf.NbParamIn>3)
    {
      /* Get the input Bernouilli coefficients: p and verify it */
      Mp=Interf.Param[3];
      if(!MatrixIsNumeric(Mp)) 
	{
	  InterfError("srandcover1d: The input Bernouilli coefficients: p must be a numeric matrix\n");
	  return;
	}

      if(!MatrixIsReal(Mp)) 
	{
	  InterfError("srandcover1d: The input Bernouilli coefficients: p must be a real matrix\n");
	  return;
	}

      /* Get the size of the input argument */ 
      P=(int)MAX(MatrixGetWidth(Mp),MatrixGetHeight(Mp));
      if(P<=0)
	{
	  InterfError("srandcover1d: The size of the input Bernouilli coefficients vector: p must be >0\n");
	  return;
	} 
      /* Get the input argument */  
      p=(double*)MatrixGetPr(Mp); 
    }
  else
    { 
      double p_ini=0.;
      double p_fin=1.;  
      /* dynamic memory allocation */
      if((p=(double*)malloc((unsigned)(P*sizeof(double))))==NULL)
	{
	  InterfError("srandcover1d: malloc error\n");
	  return;
	}  
      memset(p,0,P*sizeof(double)); 
      MFAM_linspace(P,p_ini,p_fin,p);
    }
 
  if(Interf.NbParamIn>4)
    {
      /* Get the input boolean: rooted and verify it */
      Mrooted=Interf.Param[4];
      if(!MatrixIsNumeric(Mrooted))
	{
	  InterfError("srandcover1d: The input boolean: rooted must be a numeric matrix\n");
	  return;
	}
      if(!MatrixIsReal(Mrooted))
	{
	  InterfError("srandcover1d: The input boolean: rooted must be a real matrix\n");
	  return;
	}
      if(!MatrixIsScalar(Mrooted))
	{
	  InterfError("srandcover1d: The input boolean: rooted must be a scalar\n");
	  return;
	}
      /* Get the input argument */ 
      rooted=(int)MatrixGetScalar(Mrooted);

      if((rooted!=1)&&(rooted!=0))
	{
	  InterfError("srandcover1d: The input boolean: rooted must be = to 0 or 1\n");
	  return;
	} 	
    }
  /* rooted=1 */

  /* Transpose input Matrix */
  MatrixTranspose(Ms1d);
 
  /* dynamic memory allocation */
  if((ucs1d=(unsigned char*)malloc((size_t)(Nx*sizeof(unsigned char))))==NULL)
    {
      InterfError("srandcover1d: malloc error\n");
      return;
    }
  for(k=0;k<Nx;k++)
    *(ucs1d+k)=(unsigned char)*(s1d+k);

  /* Create the output Matrixes */ 
  Mx_n_p=MatrixCreate(P,1,"real"); 
  x_n_p=MatrixGetPr(Mx_n_p);

  /* random fractal cover on 1d set */
  if(MFAD_x_n_p1d(Nx,ucs1d,rooted,n_fin,M,P,p,x_n_p,NULL,&n_max,&b_opt)==0)
    { 
      InterfError("srandcover1d: Call of the C function MFAD_x_n_p1d failed\n"); 
      return; 
    }
   
  /* Transpose input Matrix */
  MatrixTranspose(Ms1d);
    
  /* dynamic memory de-allocation */
  free((char*)ucs1d);
 
  if (Interf.NbParamIn<=3) 
    free((char*)p);

  /* Return the output */  
  ReturnParam(Mx_n_p); 
}
