/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAD.CL/src/MFAD_branching.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Sep  8 16:31:41 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Fri Sep 20 19:15:10 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAD_branching.h"

/*****************************************************************************/

int MFAD_node_get_progeny(MFAMt_node* node,MFAMt_tree* tree,short n_ini,short n_fin,int* tree_progeny,int* node_nb)
{
  if((node!=NULL)&&(tree!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(tree_progeny!=NULL)&&(node_nb!=NULL))
    {
      if(node->n<n_ini)   
 	{   
 	  int i=0; 

#ifdef MFADDEBUG
	  fprintf(stderr,"node->n=%d<n_ini=%d\n",node->n,n_ini);
	  MFAM_node_print(node,0);   
#endif /* MFADDEBUG */	
	  
 	  for(i=0;i<node->b;i++)  
 	    if(MFAM_node_sget(*(node->nodes+i),tree)==1) 
 	      MFAD_node_get_progeny(*(node->nodes+i),tree,n_ini,n_fin,tree_progeny,node_nb);    
 	} 
      else if(node->n<n_fin)  
	{
	  int i=0;
	  int progeny=0.; 	
	 
#ifdef MFADDEBUG
	  fprintf(stderr,"node->n=%d<n_fin=%d\n",node->n,n_fin); 
	  MFAM_node_print(node,0);
#endif /* MFADDEBUG */
	 
	  for(i=0;i<node->b;i++) 
	    if(MFAM_node_sget(*(node->nodes+i),tree)==1)
	      {
		progeny++;
		MFAD_node_get_progeny(*(node->nodes+i),tree,n_ini,n_fin,tree_progeny,node_nb);   
	      }
	  *tree_progeny+=progeny;
	  *node_nb+=1;

#ifdef MFADDEBUG
	  MFAM_node_print(node,0);
	  fprintf(stderr,"progeny=%d\n",progeny); 
	  fprintf(stderr,"tree_progeny=%d\n",*tree_progeny);
	  fprintf(stderr,"node_nb=%d\n",*node_nb);
#endif /* MFADDEBUG */

	}	  
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_tree_get_br_nb arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tnode=[7m%p[m\n",node);   
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);  
      return 0;
    }
}

/*****************************************************************************/

int MFAD_tree_get_br_nb(MFAMt_tree* tree,short n_ini,short n_fin,double* br_nb)
{
  if((tree!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(br_nb!=NULL))
    {
      int tree_progeny=0;
      int node_nb=0;
      MFAD_node_get_progeny(tree->root,tree,n_ini,n_fin,&tree_progeny,&node_nb);
      *br_nb=(double)tree_progeny/(double)node_nb;
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_tree_get_br_nb arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);  
      fprintf(stderr,"\tbr_nb=[7m%p[m\n",br_nb);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_br_nb1d(int Nx,unsigned char* s1d,short n_ini,short n_fin,double* br_nb,void* tree,short* n_opt,short* b_opt)
{ 
  if((Nx>0)&&(s1d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(br_nb!=NULL))
    {
      MFAMt_tree* tree_brnb=NULL;

      /* init tree */
      if((tree_brnb=MFAM_tree_s_level_init(Nx,s1d,tree,n_opt,b_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_tree_s_level_init function call error\n");
	  return 0;
	}
  
      if(n_fin==0)
	n_fin=tree_brnb->N;
       
      if(n_ini==0)
	n_ini=0;
   
      if(n_fin>tree_brnb->N)
	{ 
	  if(MFAM_tree_s_level_clear(Nx,s1d,tree_brnb,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	      return 0;
	    } 
	  return 0;
	}

      if(MFAD_tree_get_br_nb(tree_brnb,n_ini,n_fin,br_nb)==0)
	{
	  fprintf(stderr,"MFAM_tree_get_br_nb function call error\n");
	  return 0;
	}

      /* clear tree */ 
      if(MFAM_tree_s_level_clear(Nx,s1d,tree_brnb,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	  return 0;
	}
  
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_br_nb1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);   
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);  
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tbr_nb=[7m%p[m\n",br_nb); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb_opt=[7m%p[m\n",b_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_branching1d(int Nx,unsigned char* s1d,short n_ini,short n_fin,double* branchdim,void* tree,short* n_opt,short* b_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(branchdim!=NULL))
    {
      double br_nb=0.;
      if(MFAD_br_nb1d(Nx,s1d,n_ini,n_fin,&br_nb,tree,n_opt,b_opt)==0)
	{ 
	  fprintf(stderr,"MFAD_br_nb1d function call error\n");  
	  return 0;
	}
      if((br_nb>0.)&&(*b_opt>1))
	*branchdim=log(br_nb)/log((double)*b_opt); 
      else
	*branchdim=0.;
      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAD_branching1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);  
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tbranchdim=[7m%p[m\n",branchdim); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb=[7m%p[m\n",b_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_qnode_get_progeny(MFAMt_qnode* node,MFAMt_qtree* tree,short n_ini,short n_fin,int* tree_progeny,int* node_nb)
{
  if((node!=NULL)&&(tree!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(tree_progeny!=NULL)&&(node_nb!=NULL))
    {
      if(node->n<n_ini)   
 	{   
 	 
#ifdef MFADDEBUG
	  fprintf(stderr,"node->n=%d<n_ini=%d\n",node->n,n_ini);
	  MFAM_qnode_print(node,0);   
#endif /* MFADDEBUG */
	
	  if(MFAM_qnode_sget(node->cnode,tree)==1)  
	    MFAD_qnode_get_progeny(node->cnode,tree,n_ini,n_fin,tree_progeny,node_nb); 
	  if(MFAM_qnode_sget(node->hnode,tree)==1)  
	    MFAD_qnode_get_progeny(node->hnode,tree,n_ini,n_fin,tree_progeny,node_nb);  
	  if(MFAM_qnode_sget(node->vnode,tree)==1)  
	    MFAD_qnode_get_progeny(node->vnode,tree,n_ini,n_fin,tree_progeny,node_nb); 
	  if(MFAM_qnode_sget(node->hvnode,tree)==1)  
	    MFAD_qnode_get_progeny(node->hvnode,tree,n_ini,n_fin,tree_progeny,node_nb);
 	} 
      else if(node->n<n_fin)  
	{
    
	  int progeny=0.; 	
	 
#ifdef MFADDEBUG
	  fprintf(stderr,"node->n=%d<n_fin=%d\n",node->n,n_fin); 
	  MFAM_qnode_print(node,0);
#endif /* MFADDEBUG */
	 
	  if(MFAM_qnode_sget(node->cnode,tree)==1)
	    {
	      progeny++;
	      MFAD_qnode_get_progeny(node->cnode,tree,n_ini,n_fin,tree_progeny,node_nb);   
	    }
	  if(MFAM_qnode_sget(node->hnode,tree)==1)
	    {
	      progeny++;
	      MFAD_qnode_get_progeny(node->hnode,tree,n_ini,n_fin,tree_progeny,node_nb);   
	    } 
	  if(MFAM_qnode_sget(node->vnode,tree)==1)
	    {
	      progeny++;
	      MFAD_qnode_get_progeny(node->vnode,tree,n_ini,n_fin,tree_progeny,node_nb);   
	    } 
	  if(MFAM_qnode_sget(node->hvnode,tree)==1)
	    {
	      progeny++;
	      MFAD_qnode_get_progeny(node->hvnode,tree,n_ini,n_fin,tree_progeny,node_nb);   
	    }
	  *tree_progeny+=progeny;
	  *node_nb+=1;

#ifdef MFADDEBUG
	  MFAM_node_print(node,0);
	  fprintf(stderr,"progeny=%d\n",progeny); 
	  fprintf(stderr,"tree_progeny=%d\n",*tree_progeny);
	  fprintf(stderr,"node_nb=%d\n",*node_nb);
#endif /* MFADDEBUG */

	}	  
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_tree_get_br_nb arguments error\n"); 
      fprintf(stderr,"\tnode=[7m%p[m\n",node);  
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);  
      return 0;
    }
}

/*****************************************************************************/

int MFAD_qtree_get_br_nb(MFAMt_qtree* tree,short n_ini,short n_fin,double* br_nb)
{
  if((tree!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(br_nb!=NULL))
    {
      int tree_progeny=0;
      int node_nb=0;
      MFAD_qnode_get_progeny(tree->root,tree,n_ini,n_fin,&tree_progeny,&node_nb);

#ifdef MFADDEBUG
      fprintf(stderr,"tree_progeny=%d\n",tree_progeny);
      fprintf(stderr,"node_nb=%d\n",node_nb);
#endif /* MFADDEBUG */
    
      *br_nb=(double)tree_progeny/(double)node_nb;
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_qtree_get_br_nb arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);  
      fprintf(stderr,"\tbr_nb=[7m%p[m\n",br_nb);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_br_nb2d(int Nx,int Ny,unsigned char* s2d,short n_ini,short n_fin,double* br_nb,void* tree,short* n_opt)
{ 
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(br_nb!=NULL))
    {
      MFAMt_qtree* tree_brnb=NULL;

      /* init tree */
      if((tree_brnb=MFAM_qtree_s_level_init(Nx,Ny,s2d,tree,n_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_qtree_s_level_init function call error\n");
	  return 0;
	}
  
      if(n_fin==0)
	n_fin=*n_opt;
       
      if(n_ini==0)
	n_ini=0;
   
      if(n_fin>*n_opt)
	{ 
	  if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_brnb,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_qtree_s_level_clear function call error\n");
	      return 0;
	    } 
	  return 0;
	}

      if(MFAD_qtree_get_br_nb(tree_brnb,n_ini,n_fin,br_nb)==0)
	{
	  fprintf(stderr,"MFAM_qtree_get_br_nb function call error\n");
	  return 0;
	}

      /* clear tree */ 
      if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_brnb,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_qtree_s_level_clear function call error\n");
	  return 0;
	}
  
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAD_br_nb2d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny); 
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);   
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);  
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tbr_nb=[7m%p[m\n",br_nb); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return 0;
    }
}

/*****************************************************************************/

int MFAD_branching2d(int Nx,int Ny,unsigned char* s2d,short n_ini,short n_fin,double* branchdim,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(branchdim!=NULL))
    {
      double br_nb=0.;
      if(MFAD_br_nb2d(Nx,Ny,s2d,n_ini,n_fin,&br_nb,tree,n_opt)==0)
	{ 
	  fprintf(stderr,"MFAD_br_nb2d function call error\n");  
	  return 0;
	}
      if(br_nb>0.)
	*branchdim=log(br_nb)/log(2.);
      else
	*branchdim=0.;
      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAD_branching2d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);  
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tbranchdim=[7m%p[m\n",branchdim); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
       return 0;
    }
}
