/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAD.CL/src/MFAD_hausdorff.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Wed Sep  4 12:02:39 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Fri Sep 20 19:15:58 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/


/* Christophe Canus 2002 */

#include "MFAD_hausdorff.h"
#define MFADNOPROGESS 1

/*****************************************************************************/

int MFAD_node_s_level_cover(MFAMt_node* node,MFAMt_tree* tree,double p,short n_fin)
{ 
  if((tree!=NULL)&&(node!=NULL)&&(0<=p)&&(p<=1.)&&(0<n_fin)&&(n_fin<=tree->N))
    {
      if(((int)MFAM_node_sget(node,tree))==1)
	{
	  if(((int)MFAM_bernouilli(p))==1)
	    return 1;
	  else
	    { 
	      if(node->n<n_fin)
		{
		  int i=0;
		  int covered=1;
		  while((i<node->b)&&(covered==1))
		    {
		      covered&=MFAD_node_s_level_cover(*(node->nodes+i),tree,p,n_fin);
		      i++;
		    }
		  return covered;
		}
	      else
		return 0;
	    }
	}
      else
	return 1;
    }
  else 
    {
      fprintf(stderr,"MFAD_node_s_level_cover arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tnode=[7m%p[m\n",node);   
      fprintf(stderr,"\tp=[7m%f[m\n",p); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_tree_s_level_cover(MFAMt_tree* tree,double p,int rooted,short n_fin)
{
  if((tree!=NULL)&&(0<=p)&&(p<=1.)&&((rooted==0)||(rooted==1))&&(0<n_fin)&&(n_fin<=tree->N))
    {
      if(rooted==1) /* rooted==1 */
	return MFAD_node_s_level_cover(tree->root,tree,p,n_fin);
      else /* rooted==0 */
	{
	  int i=0;
	  int covered=1;
	  while((i<tree->b)&&(covered==1))
	    {
	      covered&=MFAD_node_s_level_cover(*(tree->root->nodes+i),tree,p,n_fin);
	      i++;
	    }
	  return covered;
	}
    }
  else
    {
      fprintf(stderr,"MFAD_tree_s_level_cover arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tp=[7m%f[m\n",p);   
      fprintf(stderr,"\trooted=[7m%d[m\n",rooted); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_x_n_p1d(int Nx,unsigned char* s1d,int rooted,short n_fin,int M,int P,double* p,double* x_n_p,void* tree,short* n_opt,short* b_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&((rooted==0)||(rooted==1))&&(0<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(x_n_p!=NULL))
    {
      register int i=0; 
      register int m=0;
      MFAMt_tree* tree_haus=NULL;
    
      /* random seed initialization */
      random_seed();
   
      /* init tree */
      if((tree_haus=MFAM_tree_s_level_init(Nx,s1d,tree,n_opt,b_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_tree_s_level_init function call error\n");
	  return 0;
	}

      if(n_fin==0)
	n_fin=tree_haus->N;

      if(n_fin>tree_haus->N)
	{
	  if(MFAM_tree_s_level_clear(Nx,s1d,tree_haus,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	      return 0;
	    }
	  return 0;
	}

      for(i=0;i<P;i++)
	{

#ifndef MFADNOPROGESS
	  (void)MFAM_progress(i,P);
#endif /* MFADNOPROGESS */

	  for(m=0;m<M;m++)
	    {
	      if(MFAD_tree_s_level_cover(tree_haus,*(p+i),rooted,n_fin)==1)
		*(x_n_p+i)+=1./(double)M;
	    }
	}

#ifndef MFADNOPROGESS
      (void)MFAM_progress(P,P);
#endif /* MFADNOPROGESS */
	
      /* clear tree */
      if(MFAM_tree_s_level_clear(Nx,s1d,tree_haus,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	  return 0;
	}

      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_x_n_p1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\tx_n_p=[7m%p[m\n",x_n_p); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb_opt=[7m%p[m\n",b_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_qnode_s_level_cover(MFAMt_qnode* node,MFAMt_qtree* tree,double p,short n_fin)
{ 
  if((tree!=NULL)&&(node!=NULL)&&(0<=p)&&(p<=1.)&&(0<n_fin)&&(n_fin<=tree->N))
    {
      if(((int)MFAM_qnode_sget(node,tree))==1)
	{
	  if(((int)MFAM_bernouilli(p))==1)
	    return 1;
	  else
	    { 
	      if(node->n<n_fin)
		{
		  int covered=1;
		  covered=MFAD_qnode_s_level_cover(node->cnode,tree,p,n_fin)&&
		    MFAD_qnode_s_level_cover(node->hnode,tree,p,n_fin)&&
		    MFAD_qnode_s_level_cover(node->vnode,tree,p,n_fin)&&
		    MFAD_qnode_s_level_cover(node->hvnode,tree,p,n_fin);
		  return covered;
		}
	      else
		return 0;
	    }
	}
      else
	return 1;
    }
  else 
    {
      fprintf(stderr,"MFAD_qnode_s_level_cover arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tnode=[7m%p[m\n",node);   
      fprintf(stderr,"\tp=[7m%f[m\n",p); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_qtree_s_level_cover(MFAMt_qtree* tree,double p,int rooted,short n_fin)
{
  if((tree!=NULL)&&(0<=p)&&(p<=1.)&&((rooted==0)||(rooted==1))&&(0<n_fin)&&(n_fin<=tree->N))
    {
      if(rooted==1) /* rooted==1 */
	return MFAD_qnode_s_level_cover(tree->root,tree,p,n_fin);
      else /* rooted==0 */
	{
	  int covered=1;
	  covered=MFAD_qnode_s_level_cover(tree->root->cnode,tree,p,n_fin)&&
	    MFAD_qnode_s_level_cover(tree->root->hnode,tree,p,n_fin)&&
	    MFAD_qnode_s_level_cover(tree->root->vnode,tree,p,n_fin)&&
	    MFAD_qnode_s_level_cover(tree->root->hvnode,tree,p,n_fin);
	  return covered;
	}
    }
  else
    {
      fprintf(stderr,"MFAD_qtree_s_level_cover arguments error\n");
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tp=[7m%f[m\n",p);   
      fprintf(stderr,"\trooted=[7m%d[m\n",rooted); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_x_n_p2d(int Nx,int Ny,unsigned char* s2d,int rooted,short n_fin,int M,int P,double* p,double* x_n_p,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&((rooted==0)||(rooted==1))&&(0<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(x_n_p!=NULL))
    {
      register int i=0; 
      register int m=0;
      MFAMt_qtree* tree_haus=NULL;
    
      /* random seed initialization */
      random_seed();
   
      /* init tree */
      if((tree_haus=MFAM_qtree_s_level_init(Nx,Ny,s2d,tree,n_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_qtree_s_level_init function call error\n");
	  return 0;
	}

      if(n_fin==0)
	n_fin=tree_haus->N;

      if(n_fin>tree_haus->N)
	{
	  if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_haus,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_qtree_s_level_clear function call error\n");
	      return 0;
	    }
	  return 0;
	}

      for(i=0;i<P;i++)
	{

#ifndef MFADNOPROGESS
	  (void)MFAM_progress(i,P);
#endif /* MFADNOPROGESS */

	  for(m=0;m<M;m++)
	    {
	      if(MFAD_qtree_s_level_cover(tree_haus,*(p+i),rooted,n_fin)==1)
		*(x_n_p+i)+=1./(double)M;
	    }
	}

#ifndef MFADNOPROGESS
      (void)MFAM_progress(P,P);
#endif /* MFADNOPROGESS */
	
      /* clear tree */
      if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_haus,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_qtree_s_level_clear function call error\n");
	  return 0;
	}

      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_x_n_p2d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);   
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);   
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\tx_n_p=[7m%p[m\n",x_n_p); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree); 
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return 0;
    }
}

/*****************************************************************************/
 
int MFAD_g_prime_1(int P,double* x_n_1_p,double* x_prime_n_p,int nb,double* x,double* y,double* g_prime_1)
{  
  if((P>0)&&(x_n_1_p!=NULL)&&(x_prime_n_p!=NULL)&&(0<nb)&&(nb<P)&&(x!=NULL)&&(y!=NULL)&&(g_prime_1!=NULL))
    {
      register int n=0;
      register int l=0; 
      double b=0.;
      double sigma2_e=0.;
      *x=1.;
      *y=1.;
      l=0;
      while((fabs(*(x_n_1_p+l)-1.)<1e-4)&&(l<P)) 
	l++; 
#ifdef MFADDEBUG
      fprintf(stderr,"l=%d\n",l); 
#endif /* MFADDEBUG */
      for(n=1;n<nb;n++)
	{  
	  *(x+n)=*(x_n_1_p+l+n);
	  *(y+n)=*(x_prime_n_p+l+n); 
#ifdef MFADDEBUG
	  fprintf(stderr,"(x,g_n_p(x))=(%f,%f)\n",*(x+n),*(y+n));
#endif /* MFADDEBUG */
	}
      (void)MFAM_lslr(nb,x,y,g_prime_1,&b,NULL,NULL,&sigma2_e);
#ifdef MFADDEBUG
      fprintf(stderr,"g_prime_1=%.2f\n",*g_prime_1);
#endif /* MFADDEBUG */    
      return 1;
    }
 else
    {
      fprintf(stderr,"MFAD_g_prime_1 arguments error\n");
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tx_n_1_p=[7m%p[m\n",x_n_1_p);   
      fprintf(stderr,"\tx_prime_n_p=[7m%p[m\n",x_prime_n_p);   
      fprintf(stderr,"\tnb=[7m%d[m\n",nb);
      fprintf(stderr,"\tx=[7m%p[m\n",x);   
      fprintf(stderr,"\ty=[7m%p[m\n",y);  
      fprintf(stderr,"\t_c=[7m%p[m\n",g_prime_1);  
      return 0;
    }
}

/*****************************************************************************/

int MFAD_p_c1d(int Nx,unsigned char* s1d,short n_ini,short n_fin,int M,int P,double* p,double* p_c,void* tree,short* n_opt,short* b_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(p_c!=NULL))
    {
      short n=0;
      double* x_n_1_p=NULL;
      double* x_prime_n_p=NULL;
      MFAMt_tree* tree_haus=NULL; 
      int nb=3;
      double* x=NULL;
      double* y=NULL;
      double g_prime_1=0.;
      double mean_g_prime_1=0.;
 
      /* dynamic memory allocation */
      if((x_n_1_p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x_n_1_p,0,P*sizeof(double)); 

      if((x_prime_n_p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x_prime_n_p,0,P*sizeof(double));
 
      if((x=(double*)malloc((size_t)(nb*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x,0,nb*sizeof(double)); 
      if((y=(double*)malloc((size_t)(nb*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(y,0,nb*sizeof(double));
 
      if((tree_haus=MFAM_tree_s_level_init(Nx,s1d,tree,n_opt,b_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_tree_s_level_init function call error\n");
	  return 0;
	}
    
      if(n_fin==0)
	n_fin=tree_haus->N;
   
      if(n_ini==0)
	n_ini=2;
      
      if(n_fin>tree_haus->N)
	{ 
	  if(MFAM_tree_s_level_clear(Nx,s1d,tree_haus,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	      return 0;
	    } 
	  return 0;
	}

      if(1)
	{
	  double p_ini=1./(double)*b_opt;
	  MFAM_linspace(P,p_ini,*(p+P-1),p);
	}

      for(n=n_ini;n<=n_fin;n++)
	{
	  memset(x_n_1_p,0,P*sizeof(double));
	  memset(x_prime_n_p,0,P*sizeof(double));
	  (void)MFAD_x_n_p1d(Nx,s1d,1,n-1,M,P,p,x_n_1_p,tree_haus,n_opt,b_opt); 
	  (void)MFAD_x_n_p1d(Nx,s1d,0,n,M,P,p,x_prime_n_p,tree_haus,n_opt,b_opt);  
#ifdef MFADDEBUG
	  {
	    register int k=0;
	    for(k=0;k<P;k++) 
	      fprintf(stdout,"%f %f\n",*(x_n_1_p+k),*(x_prime_n_p+k));
	  }
#endif /* MFADDEBUG */
	  (void)MFAD_g_prime_1(P,x_n_1_p,x_prime_n_p,nb,x,y,&g_prime_1);
	  mean_g_prime_1+=g_prime_1;
	}
      mean_g_prime_1/=(n_fin-n_ini+1);
      *p_c=1./mean_g_prime_1;
#ifdef MFADDEBUG
      fprintf(stderr,"p_c=%f\n",*p_c);
#endif /* MFADDEBUG */
      if(MFAM_tree_s_level_clear(Nx,s1d,tree_haus,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	  return 0;
	} 
     
      /* dynamic memory deallocation */
      free((char*)x_n_1_p);
      free((char*)x_prime_n_p);  
      free((char*)x);
      free((char*)y);
      return 1;
    }
   else
    {
      fprintf(stderr,"MFAD_p_c1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);  
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);    
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\tp_c=[7m%p[m\n",p_c); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb_opt=[7m%p[m\n",b_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_hausdorff1d(int Nx,unsigned char* s1d,short n_ini,short n_fin,int M,int P,double* p,double* hausdim,void* tree,short* n_opt,short* b_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(hausdim!=NULL))
    {
      double p_c=0.;
      if(MFAD_p_c1d(Nx,s1d,n_ini,n_fin,M,P,p,&p_c,tree,n_opt,b_opt)==0)
	{ 
	  fprintf(stderr,"MFAD_p_c1d function call error\n");  
	  return 0;
	}
      if((fabs(p_c-0.)>DBL_EPSILON)&&(*b_opt>1))
	*hausdim=-log(p_c)/log((double)*b_opt); 
      else
	*hausdim=0.; 
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_hausdorff1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);    
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);  
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\thausdim=[7m%p[m\n",hausdim); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb_opt=[7m%p[m\n",b_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_p_c2d(int Nx,int Ny,unsigned char* s2d,short n_ini,short n_fin,int M,int P,double* p,double* p_c,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(p_c!=NULL))
    {
      short n=0;
      double* x_n_1_p=NULL;
      double* x_prime_n_p=NULL;
      MFAMt_qtree* tree_haus=NULL; 
      int nb=3;
      double* x=NULL;
      double* y=NULL;
      double g_prime_1=0.;
      double mean_g_prime_1=0.;
 
      /* dynamic memory allocation */
      if((x_n_1_p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x_n_1_p,0,P*sizeof(double)); 

      if((x_prime_n_p=(double*)malloc((size_t)(P*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x_prime_n_p,0,P*sizeof(double));
 
      if((x=(double*)malloc((size_t)(nb*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(x,0,nb*sizeof(double)); 
      if((y=(double*)malloc((size_t)(nb*sizeof(double))))==NULL)
	{
	  fprintf(stderr,"malloc error\n"); 
	  return 0;
	}  
      memset(y,0,nb*sizeof(double));
 
      if((tree_haus=MFAM_qtree_s_level_init(Nx,Ny,s2d,tree,n_opt))==NULL)
	{
	  fprintf(stderr,"MFAM_qtree_s_level_init function call error\n");
	  return 0;
	}
    
      if(n_fin==0)
	n_fin=*n_opt;
   
      if(n_ini==0)
	n_ini=2;
      
      if(n_fin>tree_haus->N)
	{ 
	  if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_haus,tree,n_opt)==0)
	    {
	      fprintf(stderr,"MFAM_qtree_clear function call error\n");
	      return 0;
	    } 
	  return 0;
	}

      if(1)
	{
	  double p_ini=1./2.;
	  MFAM_linspace(P,p_ini,*(p+P-1),p);
	}

      for(n=n_ini;n<=n_fin;n++)
	{
	  memset(x_n_1_p,0,P*sizeof(double));
	  memset(x_prime_n_p,0,P*sizeof(double));
	  (void)MFAD_x_n_p2d(Nx,Ny,s2d,1,n-1,M,P,p,x_n_1_p,tree_haus,n_opt); 
	  (void)MFAD_x_n_p2d(Nx,Ny,s2d,0,n,M,P,p,x_prime_n_p,tree_haus,n_opt);  

#ifdef MFADDEBUG
	  {
	    register int k=0;
	    for(k=0;k<P;k++) 
	      fprintf(stdout,"%f %f\n",*(x_n_1_p+k),*(x_prime_n_p+k));
	  }
#endif /* MFADDEBUG */

	  (void)MFAD_g_prime_1(P,x_n_1_p,x_prime_n_p,nb,x,y,&g_prime_1);
	  mean_g_prime_1+=g_prime_1;
	}
      mean_g_prime_1/=(n_fin-n_ini+1);
      *p_c=1./mean_g_prime_1;

#ifdef MFADDEBUG
      fprintf(stderr,"p_c=%f\n",*p_c);
#endif /* MFADDEBUG */

      if(MFAM_qtree_s_level_clear(Nx,Ny,s2d,tree_haus,tree,n_opt)==0)
	{
	  fprintf(stderr,"MFAM_tree_s_level_clear function call error\n");
	  return 0;
	} 
     
      /* dynamic memory deallocation */
      free((char*)x_n_1_p);
      free((char*)x_prime_n_p);  
      free((char*)x);
      free((char*)y);
      return 1;
    }
   else
    {
      fprintf(stderr,"MFAD_p_c1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);  
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);    
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\tp_c=[7m%p[m\n",p_c); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);
      return 0;
    }
}

/*****************************************************************************/

int MFAD_hausdorff2d(int Nx,int Ny,unsigned char* s2d,short n_ini,short n_fin,int M,int P,double* p,double* hausdim,void* tree,short* n_opt)
{
  if((Nx>0)&&(Ny>0)&&(s2d!=NULL)&&(0<=n_ini)&&(n_ini<=n_fin)&&(M>1)&&(P>0)&&(p!=NULL)&&(hausdim!=NULL))
    {
      double p_c=0.;
      if(MFAD_p_c2d(Nx,Ny,s2d,n_ini,n_fin,M,P,p,&p_c,tree,n_opt)==0)
	{ 
	  fprintf(stderr,"MFAD_p_c2d function call error\n");  
	  return 0;
	}
      if(fabs(p_c-0.)>DBL_EPSILON)
	*hausdim=-log(p_c)/log(2.);  
      else
	*hausdim=0.;
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAD_hausdorff1d arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx); 
      fprintf(stderr,"\tNy=[7m%d[m\n",Ny);
      fprintf(stderr,"\ts2d=[7m%p[m\n",s2d);    
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini);  
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tM=[7m%d[m\n",M);
      fprintf(stderr,"\tP=[7m%d[m\n",P);
      fprintf(stderr,"\tp=[7m%p[m\n",p); 
      fprintf(stderr,"\thausdim=[7m%p[m\n",hausdim); 
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);  
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return 0;
    }
}
