/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAG.CL/src/MFAG_hoelder2d.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAG_hoelder2d.c,v 1.1.1.1 1999/02/10 13:38:44 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:42:26 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Sep 15 18:13:59 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998 */

#include "MFAG_hoelder2d.h"

/*****************************************************************************/
/* on measures                                                               */
/*****************************************************************************/

/*****************************************************************************/

#ifndef __STDC__
int MFAG_alpha_2_n2d(n, mu_n, alpha_n)
     short int n;
     double* mu_n;
     double* alpha_n;
#else /* __STDC__ */
     int MFAG_alpha_2_n2d(short n,double* mu_n,double* alpha_n)
#endif /* __STDC__ */
{
  if((n>0)&&(mu_n!=NULL)&&(alpha_n!=NULL))
    {
      register int i=0,j=0;  
      int two_pow_n=(int)pow(2.,(double)n);
      for(j=0;j<two_pow_n;j++)
	for(i=0;i<two_pow_n;i++)
	  *(alpha_n+two_pow_n*j+i)=(*(mu_n+two_pow_n*j+i)>0.?-log(*(mu_n+two_pow_n*j+i))/log((double)two_pow_n):HUGE);
      return 1;
    }
  else
    {
      fprintf(stdout,"MFAG_alpha_2_n2d arguments error\n");
      return 0;
    }   
}

/*****************************************************************************/

#ifndef __STDC__
int MFAG_alpha_n2d(Nx_n, Ny_n, mu_n, alpha_n)
     int Nx_n;
     int Ny_n;
     double* mu_n;
     double* alpha_n;
#else /* __STDC__ */
     int MFAG_alpha_n2d(int Nx_n,int Ny_n,double* mu_n,double* alpha_n)
#endif /* __STDC__ */
{
  if((Nx_n>0)&&(Ny_n>0)&&(mu_n!=NULL)&&(alpha_n!=NULL))
    {
      register int i=0,j=0;
      for(j=0;j<Ny_n;j++)
	for(i=0;i<Nx_n;i++)
	  *(alpha_n+Nx_n*j+i)=(*(mu_n+Nx_n*j+i)>0.?-2.*log(*(mu_n+Nx_n*j+i))/log((double)Nx_n*Ny_n):HUGE);
      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAG_alpha_n2d arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

#ifndef __STDC__
int MFAG_alpha_n2dmma(Nx_n, Ny_n, mu_n, nu_n, alpha_n)
     int Nx_n;
     int Ny_n;
     double* mu_n;
     double* nu_n;
     double* alpha_n;
#else /* __STDC__ */
     int MFAG_alpha_n2dmma(int Nx_n,int Ny_n,double* mu_n,double* nu_n,double* alpha_n)
#endif /* __STDC__ */
{
  if((Nx_n>0)&&(Ny_n>0)&&(mu_n!=NULL)&&(nu_n!=NULL)&&(alpha_n!=NULL))
    {
      register int i=0,j=0;
      for(j=0;j<Ny_n;j++)
	for(i=0;i<Nx_n;i++)
	  *(alpha_n+Nx_n*j+i)=((*(mu_n+Nx_n*j+i)>0.)&&(*(nu_n+Nx_n*j+i)>0.)?log(*(mu_n+Nx_n*j+i))/log(*(nu_n+Nx_n*j+i)):HUGE);
      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAG_alpha_n2dmma arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

#ifndef __STDC__
int MFAG_mch2d(N_nx, N_ny, I_nx, I_ny, mu_n2d, nu_n2d, t_capa, t_prog, t_ball, J, S_min, S_max, Nx, Ny, eta, x, y, alpha_eta_xy)
     int N_nx;
     int N_ny;
     double* I_nx;
     double* I_ny;
     double* mu_n2d;
     double* nu_n2d;
     MFAMt_capa t_capa;
     MFAMt_prog t_prog;
     MFAMt_ball t_ball;
     short int J;
     double S_min;
     double S_max;
     int Nx; 
     int Ny;
     double* eta;
     double* x;
     double* y;
     double* alpha_eta_xy;
#else /* __STDC__ */
     int MFAG_mch2d(int N_nx,int N_ny,double* I_nx,double* I_ny,double* mu_n2d,double* nu_n2d,MFAMt_capa t_capa,MFAMt_prog t_prog,MFAMt_ball t_ball,short J,double S_min,double S_max,int Nx,int Ny,double* eta,double* x,double* y,double* alpha_eta_xy)
#endif /* __STDC__ */
{ 
  if((N_nx>0)&&(N_ny>0)&&(mu_n2d!=NULL)&&(J>0)&&(S_min>=1.)&&(S_max>=S_min)&&(Nx*S_min>=N_nx)&&(eta!=NULL)&&(alpha_eta_xy!=NULL))
    { 
      double eta_nx=1./N_nx,eta_ny=1./N_ny;
      if(MFAM_scale(N_nx*N_ny,S_min,S_max,J,eta,t_prog,2)==0)
	{
	  fprintf(stderr,"MFAM_scale function call error\n"); 
	  return 0;
	}
      if((Nx==N_nx)&&(Ny==N_ny)&&(J==1)&&(t_ball==MFAM_ASYMMETRIC))
	{
	  if((*eta==eta_nx)&&(*eta==eta_ny))
	    {
	      if(nu_n2d==NULL)
		{
		  if(MFAG_alpha_n2d(N_nx,N_nx,mu_n2d,alpha_eta_xy)==0)
		    {  
		      fprintf(stderr,"MFAG_alpha_n2d function call error\n");  
		      return 0;  
		    }
		}
	      else
		{
		  if(MFAG_alpha_n2dmma(N_nx,N_nx,mu_n2d,nu_n2d,alpha_eta_xy)==0)
		    {  
		      fprintf(stderr,"MFAG_alpha_n2dmma function call error\n");  
		      return 0;  
		    }
		}
	    }
	  else /* *eta>eta_n */
	    { 
	      /* to be implemented */
	    } 
	  if(x!=NULL)
	    {
	      register int m=0;
	      *x=0.;
	      if(I_nx!=NULL)
		for(m=0;m<Nx;m++)
		  *(x+m)=*(I_nx+m);
	      else /* I_nx==NULL */
		for(m=1;m<Nx;m++)
		  *(x+m)=*(x+m-1)+*eta;
	      *y=0.;
	      if(I_ny!=NULL)
		for(m=0;m<Ny;m++)
		  *(y+m)=*(I_ny+m);
	      else /* I_ny==NULL */
		for(m=1;m<Ny;m++)
		  *(y+m)=*(y+m-1)+*eta;
	    }
	}  
      else if(t_prog==MFAM_DEC)
	{
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAG_mch2d arguments error\n");
      return 0;
    }
}
