/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAH.CL/src/MFAH_mfdlimsup1d.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 16:24:43 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Mon Oct 14 21:30:23 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

#include "MFAH_mfdlimsup1d.h"

/*****************************************************************************/

int MFAH_mfdls1d(int Nx,double* c1d,double* mu1d,short n_ini,short n_fin,MFAMt_capa t_capa,double epsilon,int K,double* B,MFADt_dim t_dim,int N,double* alpha,double* fdls1d_alpha)
{
  if((Nx)&&(c1d!=NULL)&&(n_ini>=0)&&(n_fin>=0)&&(t_capa<6)&&(epsilon>=0.)&&(K>0)&&(B!=NULL)&&(t_dim<4)&&(N>0)&&(alpha!=NULL)&(fdls1d_alpha!=NULL))
    {
      register int k=0;
      short n=0;
      short n_max=0; 
      int two_pow_n=0;
      double d_E_n_epsilon_alpha=0.;
      double sup_d_E_n_epsilon_alpha=0;
      double* log_epsilon=NULL;
      double* log_N_epsilon=NULL;
      double* alpha_n_k=NULL; 
      unsigned char* E_n_epsilon_alpha=NULL;
      MFAMt_btree* tree=0;
      short n_opt=0;
      short b_opt=0;
      MFAMt_tree* dim_tree=NULL;
      void* keep_tree=NULL;

      n_max=(short)ceil(log((double)Nx)/log(2.)); 
   
      if(n_fin==0)
	n_fin=n_max;
      if(n_ini==0)
	n_ini=n_max-1;
      else if(n_ini>n_max-1)
	{  
	  fprintf(stderr,"MFAH_mfdls1d arguments error\n");
	  return 0;
	}
  
#ifdef MFAHDEBUG 
      fprintf(stderr,"Nx=%d,n_max=%d,n_ini=%d,n_fin=%d\n",Nx,n_max,n_ini,n_fin);
#endif /* MFAHDEBUG */
      fprintf(stderr,"Nx=%d,n_max=%d,n_ini=%d,n_fin=%d\n",Nx,n_max,n_ini,n_fin);
   
      /* tree creation with n_max levels */
      tree=MFAM_btree_create(n_max);  
    
      two_pow_n=(int)pow(2.,(double)n_max); 
      /* capacities levels creation and computation
	 by attachment and decimation */
      if(Nx==two_pow_n) 
 	{
	  MFAM_btree_level_create(tree,MFAM_CAPACITY,n_ini,n_max-1);
	  MFAM_btree_level_set(tree,MFAM_CAPACITY,n_max,(void*)c1d); 
	} 
      else 
 	{
	  MFAM_btree_level_create(tree,MFAM_CAPACITY,n_ini,n_max); 
 	  MFAM_btree_level_convert(tree,MFAM_CAPACITY,n_max,Nx,(void*)c1d); 
 	} 

      MFAM_btree_c_level_decimate(tree,t_capa,n_ini,n_max);

#ifdef MFAHDEBUG
      MFAM_btree_level_print(tree,MFAM_CAPACITY,n_ini,n_max);
#endif /* MFAHDEBUG */

      /* reference measures levels creation and computation
	 by attachment and decimation */
      if(mu1d)
	{ 
	  if(Nx==two_pow_n) 
	    {
	      MFAM_btree_level_create(tree,MFAM_REFERENCE,n_ini,n_max-1);
	      MFAM_btree_level_set(tree,MFAM_REFERENCE,n_max,(void*)mu1d); 
	    } 
	  else
	    {  
	      MFAM_btree_level_create(tree,MFAM_REFERENCE,n_ini,n_max); 
	      MFAM_btree_level_convert(tree,MFAM_REFERENCE,n_max,Nx,(void*)mu1d); 
	    }
	  
	  MFAM_btree_r_level_decimate(tree,n_ini,n_max);
	}

#ifdef MFAHDEBUG
      MFAM_btree_level_print(tree,MFAM_REFERENCE,n_ini,n_max);
#endif /* MFAHDEBUG */

      /* coarse-grain hoelder exponents levels creation and computation */
      MFAM_btree_level_create(tree,MFAM_HOELDER,n_ini,n_max);
      if(mu1d) 
	MFAH_btree_mma_level_compute(tree,n_ini,n_max); 
      else
	MFAH_btree_h_level_compute(tree,n_ini,n_max); 

#ifdef MFAHDEBUG
        MFAM_btree_level_print(tree,MFAM_HOELDER,n_ini,n_max);
#endif /* MFAHDEBUG */ 
   
      /* coarse-grain hoelder exponents statistics computation */
      alpha_n_k=(double*)MFAM_btree_level_get(tree,MFAM_HOELDER,n_ini);  
      two_pow_n=(int)pow(2.,(double)n_ini);

      MFAH_hoelder(two_pow_n,alpha_n_k,N,alpha,&epsilon);

#ifdef MFAHDEBUG
      fprintf(stderr,"[alpha_min,alpha_max]=[7m[%g,%g][m\n",alpha_min,alpha_max);
#endif /* MFAHDEBUG */
    
      /* iso-hoelder sets level creation */
      MFAM_btree_level_create(tree,MFAM_SET,n_ini,n_fin);
    
      for(k=0;k<N;k++)      
	{ 

#ifndef MFAHNOPROGESS
	  MFAM_progress(k,N);
#endif /* MFAHNOPROGESS */

	  n=n_fin;	  
	  sup_d_E_n_epsilon_alpha=0.;
	  while(n>=n_ini)  
	    {   
	      MFAH_btree_E_n_epsilon_alpha(tree,n,epsilon,*(alpha+k));
	      E_n_epsilon_alpha=(unsigned char*)MFAM_btree_level_get(tree,MFAM_SET,n);
	      two_pow_n=(int)pow(2.,(double)n);
	     
	      /* dynamic memory allocation */	     
	      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
		{ 
		  if(k==0)
		    { 
		      if((log_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
			{
			  fprintf(stderr,"malloc error\n");
			  return 0;
			} 
		      memset(log_epsilon,0,K*sizeof(double));
		      if((log_N_epsilon=(double*)malloc((size_t)(K*sizeof(double))))==NULL)
			{
			  fprintf(stderr,"malloc error\n");
			  return 0;
			}     
		      memset(log_N_epsilon,0,K*sizeof(double)); 
		    }
		}
	      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
		{
		  if(k==0)
		    keep_tree=(void*)(dim_tree=MFAM_tree_s_level_init(two_pow_n,E_n_epsilon_alpha,NULL,&n_opt,&b_opt)); 
		  else
		    MFAM_tree_s_level_update(two_pow_n,E_n_epsilon_alpha,dim_tree,&n_opt);
		} 

	      if(MFAM_isempty(two_pow_n,E_n_epsilon_alpha)==0)
		{
		  if(MFAD_dimension1d(two_pow_n,E_n_epsilon_alpha,K,B,t_dim,log_epsilon,log_N_epsilon,&d_E_n_epsilon_alpha,keep_tree)==0) 
		    { 
		      fprintf(stderr,"MFAD_dimension1d function call error\n"); 
		      return 0; 
		    }
		} 
	      sup_d_E_n_epsilon_alpha=(sup_d_E_n_epsilon_alpha>d_E_n_epsilon_alpha?sup_d_E_n_epsilon_alpha:d_E_n_epsilon_alpha);   
	      d_E_n_epsilon_alpha=0;
	      n--;   
	    }
	  *(fdls1d_alpha+k)=sup_d_E_n_epsilon_alpha;

#ifdef MFAHDEBUG 
	  fprintf(stderr,"alpha=%f, fdls1d_alpha=%f\n",*(alpha+k),*(fdls1d_alpha+k));
#endif /* MFAHDEBUG */

	}

#ifndef MFAHNOPROGESS
      MFAM_progress(N,N);
#endif /* MFAHNOPROGESS */
     
      /* dynamic memory deallocation */
      if((t_dim==MFAD_BOX)||(t_dim==MFAD_VOSS))
	{
	  free((char*)log_N_epsilon);
	  free((char*)log_epsilon);
	}
      else if((t_dim==MFAD_BRANCHING)||(t_dim==MFAD_HAUSDORFF))
	MFAM_tree_s_level_clear(two_pow_n,E_n_epsilon_alpha,dim_tree,NULL,&n_opt);
        
      two_pow_n=(int)pow(2.,(double)n_max); 
      /* capacities levels deletion */  
      if(Nx==two_pow_n) 
	{
	  MFAM_btree_level_set(tree,MFAM_CAPACITY,n_max,(void*)NULL); 
	  MFAM_btree_level_delete(tree,MFAM_CAPACITY,n_ini,n_max-1); 
	}
      else
	MFAM_btree_level_delete(tree,MFAM_CAPACITY,n_ini,n_max);

      /* reference measures levels deletion */  
      if(mu1d)
	{ 
	  if(Nx==two_pow_n) 
	    {
	      MFAM_btree_level_set(tree,MFAM_REFERENCE,n_max,(void*)NULL); 
	      MFAM_btree_level_delete(tree,MFAM_REFERENCE,n_ini,n_max-1);
	    } 
	  else
	    MFAM_btree_level_delete(tree,MFAM_REFERENCE,n_ini,n_max);
	} 

      /* coarse-grain hoelder exponents levels deletion */
      MFAM_btree_level_delete(tree,MFAM_HOELDER,n_ini,n_max); 

      /* iso-hoelder sets levels deletion */
      MFAM_btree_level_delete(tree,MFAM_SET,n_ini,n_fin);

      /* tree deletion */
      MFAM_btree_delete(tree);
    
      /* spectrum normalization */
      if(MFAM_norm(N,fdls1d_alpha,NULL,1)==0)
	{ 
	  fprintf(stderr,"MFAM_norm fucntion call error\n"); 
	  return 0;
	}

      return 1; 
    }
  else
    {
      fprintf(stderr,"MFAH_mfdls1d arguments error\n");  
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
      fprintf(stderr,"\tc1d=[7m%p[m\n",c1d);  
      fprintf(stderr,"\tmu1d=[7m%p[m\n",mu1d); 
      fprintf(stderr,"\tn_ini=[7m%d[m\n",n_ini); 
      fprintf(stderr,"\tn_fin=[7m%d[m\n",n_fin); 
      fprintf(stderr,"\tt_capa=[7m%d[m\n",t_capa); 
      fprintf(stderr,"\tepsilon=[7m%f[m\n",epsilon); 
      fprintf(stderr,"\tK=[7m%d[m\n",K);   
      fprintf(stderr,"\tB=[7m%p[m\n",B);  
      fprintf(stderr,"\tt_dim=[7m%d[m\n",t_dim); 
      fprintf(stderr,"\tN=[7m%d[m\n",N);   
      fprintf(stderr,"\talpha=[7m%p[m\n",alpha);
      fprintf(stderr,"\tfdls1d_alpha=[7m%p[m\n",fdls1d_alpha); 
      return 0;
    }
}
