/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAM/SRC/MFAM_btree.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:53:31 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Wed Nov 20 23:21:46 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98-2002 */

#include "MFAM_btree.h"

extern double norm_meas;
extern double p;
extern double c_ref;
extern double q;
extern double sigma2;

/*****************************************************************************/

MFAMt_bnode* MFAM_bnode_create(MFAMt_bnode* node,char* x,short n,int k,short N)
{
  if(node==NULL)
  {	
    if((node=(MFAMt_bnode*)malloc(sizeof(MFAMt_bnode)))==NULL)
    { 
      fprintf(stderr,"MFAM_bnode_create: malloc error\n");
      return NULL;
    }
    node->x=x;
    node->n=n;
    node->k=k;
    node->lnode=NULL;
    node->rnode=NULL;
  }
  if (n!=N)
  {
    char* x0=NULL;
    char* x1=NULL;
    x0=(char*)malloc((strlen(x)+2)*sizeof(char));
    sprintf(x0,"%s%d",x,0); 
    node->lnode=MFAM_bnode_create(node->lnode,x0,n+1,2*k,N);
    x1=(char*)malloc((strlen(x)+2)*sizeof(char));
    sprintf(x1,"%s%d",x,1); 
    node->rnode=MFAM_bnode_create(node->rnode,x1,n+1,2*k+1,N);
  }
  return node;
}

/*****************************************************************************/

void MFAM_bnode_print(MFAMt_bnode* node,int recursive_print)
{ 
  if (node!=NULL)
  {
    fprintf(stderr,"bnode of type: ");
    if (!(node->k%2))
      fprintf(stdout,"l");
    else
      fprintf(stdout,"r");
    fprintf(stdout,"level n=%d, x=%s,k=%d\n",node->n,node->x,node->k);
    if (recursive_print)
      {
	MFAM_bnode_print(node->lnode,recursive_print);
	MFAM_bnode_print(node->rnode,recursive_print);
      }
  }
}

/*****************************************************************************/

void MFAM_bnode_delete(MFAMt_bnode* node)
{ 
  if(node!=NULL)
  {
    MFAM_bnode_delete(node->lnode);
    node->lnode=NULL;
    MFAM_bnode_delete(node->rnode);
    node->rnode=NULL;
    free(node->x);
    free((char*)node);
  }
}

/*****************************************************************************/

MFAMt_btree* MFAM_btree_create(short N)
{
  MFAMt_btree* tree=NULL; 
  char* x=NULL;
  if ((tree=(MFAMt_btree*)malloc(sizeof(MFAMt_btree)))==NULL) 
  { 
    fprintf(stderr,"MFAM_btree_create: malloc error\n");
    return NULL;
  }
  tree->N=N;
  tree->root=NULL;
  x=(char*)strdup("0.");
  if ((tree->root=MFAM_bnode_create(tree->root,x,0,0,N))==NULL)
  { 
    fprintf(stderr,"root node creation error\n");
    return NULL;
  }  
  if((tree->c_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->c_level,(int)NULL,(N+1)*sizeof(double*));
 
  if((tree->r_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->r_level,(int)NULL,(N+1)*sizeof(double*));

  if((tree->h_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->h_level,(int)NULL,(N+1)*sizeof(double*));

  if((tree->s_level=(unsigned char**)malloc((N+1)*sizeof(unsigned char*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    } 
  memset(tree->s_level,(int)NULL,(N+1)*sizeof(unsigned char*)); 
  
  return tree;
}

/*****************************************************************************/

void MFAM_btree_delete(MFAMt_btree* tree)
{ 
  if (tree!=NULL)
  {
    int n=0;
    MFAM_bnode_delete(tree->root);
    while(n<=tree->N)
      {
	if (*(tree->c_level+n)!=NULL)
	  {
	    free((char*)*(tree->c_level+n));
	    *(tree->c_level+n)=NULL;
	  }
	if (*(tree->r_level+n)!=NULL)
	  {
	    free((char*)*(tree->r_level+n));
	    *(tree->r_level+n)=NULL;
	  }
	if (*(tree->h_level+n)!=NULL)
	  {
	    free((char*)*(tree->h_level+n));
	    *(tree->h_level+n)=NULL;
	  }
	if (*(tree->s_level+n)!=NULL)  
	  {
	    free((char*)*(tree->s_level+n));
	    *(tree->s_level+n)=NULL;
	  }
	n++;
      }
    free((char*)tree->c_level);    
    free((char*)tree->r_level);   
    free((char*)tree->h_level);
    free((char*)tree->s_level);
    free((char*)tree);
  }
}

/*****************************************************************************/

void MFAM_btree_print(MFAMt_btree* tree)
{ 
  if (tree!=NULL)
  {
    fprintf(stderr,"btree with # N=%d levels\n",tree->N);
    MFAM_bnode_print(tree->root,1);
  }
}

/*****************************************************************************/

int MFAM_btree_level_create(MFAMt_btree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    { 
	      int two_pow_n=(int)pow(2.,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree capacity level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->c_level+n)=(double*)malloc(two_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_btree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    { 
	      int two_pow_n=(int)pow(2.,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree reference level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->r_level+n)=(double*)malloc(two_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_btree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int two_pow_n=(int)pow(2.,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree hoelder level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->h_level+n)=(double*)malloc(two_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_btree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int two_pow_n=(int)pow(2.,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree set level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->s_level+n)=(unsigned char*)malloc(two_pow_n*sizeof(unsigned char)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_btree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_btree_level_create arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_btree_level_create arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_btree_level_delete(MFAMt_btree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      if(*(tree->c_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"btree capacity level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->c_level+n));
		  *(tree->c_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_btree_level_delete (c) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      if(*(tree->r_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"btree reference level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->r_level+n));
		  *(tree->r_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_btree_level_delete (r) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      if(*(tree->h_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"btree hoelder level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->h_level+n));
		  *(tree->h_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_btree_level_delete (h) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      if(*(tree->s_level+n)!=NULL)
		{      
#ifdef MFAMDEBUG
		  fprintf(stderr,"btree set level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->s_level+n));
		  *(tree->s_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_btree_level_delete (s) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_btree_level_delete arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_btree_level_delete arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_btree_level_set(MFAMt_btree* tree,MFAMt_level level_type,short n,void* level)
{
  if((tree!=NULL)&&(n>0))
    {
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if(((*(tree->c_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->c_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree capacity level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->c_level+n)=(double*)level;
	    }
	  break;    
	case MFAM_REFERENCE:
	  if(((*(tree->r_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->r_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"btree reference level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->r_level+n)=(double*)level;
	    }
	  break;   
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET:  
	  *(tree->s_level+n)=(unsigned char*)level;
	  break;   
	default:
	  fprintf(stderr,"MFAM_btree_level_set arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_btree_level_set arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

int MFAM_btree_level_convert(MFAMt_btree* tree,MFAMt_level level_type,short n,int Nx,void* level)
{
  if((tree!=NULL)&&(n>0)&&(Nx>0)&&(level!=NULL))
    { 
      register int x=0;
      int two_pow_n=(int)pow(2.,(double)n);
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if((*(tree->c_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_c1d=NULL;
	      double* tmp_level1d=NULL;
	      tmp_c1d=(double*)MFAM_btree_level_get(tree,MFAM_CAPACITY,n);
	      tmp_level1d=(double*)level;
	      memset(tmp_c1d,0,(size_t)(two_pow_n*sizeof(double)));
	      for(x=0;x<Nx;x++)
		*(tmp_c1d+x)=(double)*(tmp_level1d+x);
	    }
	  break; 
	case MFAM_REFERENCE:
	  if((*(tree->r_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_mu1d=NULL;
	      double* tmp_level1d=NULL;
	      tmp_mu1d=(double*)MFAM_btree_level_get(tree,MFAM_REFERENCE,n);
	      tmp_level1d=(double*)level;
	      memset(tmp_mu1d,0,(size_t)(two_pow_n*sizeof(double)));
	      for(x=0;x<Nx;x++)
		  *(tmp_mu1d+x)=(double)*(tmp_level1d+x);
	    }
	  break;     
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET: 
	  if((*(tree->s_level+n)!=NULL)&&(level!=NULL))
  	    {     
	      unsigned char* tmp_set1d=NULL;
	      unsigned char* tmp_level1d=NULL;	    
	      tmp_set1d=(unsigned char*)MFAM_btree_level_get(tree,MFAM_SET,n);
	      tmp_level1d=(unsigned char*)level;
	      memset(tmp_set1d,0,(size_t)(two_pow_n*sizeof(unsigned char)));
	      for(x=0;x<Nx;x++)
		*(tmp_set1d+x)=(unsigned char)*(tmp_level1d+x);
	    }
	  break;   
	default:
	  fprintf(stderr,"MFAM_btree_level_convert arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_btree_level_convert arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

void* MFAM_btree_level_get(MFAMt_btree* tree,MFAMt_level level_type,short n)
{
  if((tree!=NULL)||(n>0))
    { 
      void* level=NULL;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  level=(void*)*(tree->c_level+n); 
	  break;  
	case MFAM_REFERENCE:
	  level=(void*)*(tree->r_level+n); 
	  break;   
	case MFAM_HOELDER:
	  level=(void*)*(tree->h_level+n);
	  break;    
	case MFAM_SET:  
	  level=(void*)*(tree->s_level+n);
	  break;   
	default:
	  fprintf(stderr,"MFAM_btree_level_get arguments error\n");
	  break;
	}
      return level;
    }
  else
    {
      fprintf(stderr,"MFAM_btree_level_get arguments error\n");
      return NULL;
   }   
}

/*****************************************************************************/

int MFAM_btree_level_print(MFAMt_btree *tree,MFAMt_level level_type,short n_ini,short n_fin)
{ 
  if(tree!=NULL)
    {
     int n=n_ini;
     switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      int k=0;
	      int two_pow_n=(int)pow(2.,(double)n); 
	      fprintf(stderr,"printing capacities of level # %d\n",n);
	      for(k=0;k<two_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->c_level+n)+k));
	      n++;
	    }
	  break;
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      int k=0;
	      int two_pow_n=(int)pow(2.,(double)n); 
	      fprintf(stderr,"printing reference measures of level # %d\n",n);
	      for(k=0;k<two_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->r_level+n)+k));
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int two_pow_n=(int)pow(2.,(double)n); 
	      fprintf(stderr,"printing hoelder exponents of level # %d\n",n);
	      for(k=0;k<two_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->h_level+n)+k));
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int two_pow_n=(int)pow(2.,(double)n); 
	      fprintf(stderr,"printing set of level # %d\n",n);
	      for(k=0;k<two_pow_n;k++)
		fprintf(stderr,"%d %d\n",k,*(*(tree->s_level+n)+k));
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_btree_level_print arguments error\n");
	  break;
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAM_btree_level_print error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_btree_cprint(MFAMt_btree *tree)
{ 
   if(tree!=NULL)
   {
      if(tree->c_level!=NULL)
	 MFAM_bnode_cprint(tree->root,tree);
      else
      {
	 fprintf(stderr,"error\n");
	 return 0;
      }
      return 1;
   }
   else
   { 
      fprintf(stderr,"error\n");
      return 0;
   }
}

/****************************************************************************/

void MFAM_bnode_cprint(MFAMt_bnode* node,MFAMt_btree* tree)
{ 
   if(node!=NULL)
   {
      fprintf(stdout,"node # x=%s,n=%d,k=%d\t",node->x,node->n,node->k);
      fprintf(stdout,"c=%f\n",*(*(tree->c_level+node->n)+node->k));
      MFAM_bnode_cprint(node->lnode,tree);
      MFAM_bnode_cprint(node->rnode,tree);
   }
}

/*****************************************************************************/

double MFAM_bnode_cget(MFAMt_bnode* node,MFAMt_btree* tree)
{ 
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+node->k!=NULL))
    return *(*(tree->c_level+node->n)+node->k);
  else
    {
      fprintf(stderr,"MFAM_qnode_cget error\n");
      return 0.;
    }
}

/*****************************************************************************/

void MFAM_bnode_cset(MFAMt_bnode* node,MFAMt_btree* tree,double c)
{  
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+node->k!=NULL))
    *(*(tree->c_level+node->n)+node->k)=c;
  else
    fprintf(stderr,"MFAM_qnode_cset error\n");
}

/*****************************************************************************/

double MFAM_bnode_rget(MFAMt_bnode* node,MFAMt_btree* tree)
{
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+node->k!=NULL))
    return *(*(tree->r_level+node->n)+node->k);
  else
    {
      fprintf(stderr,"MFAM_bnode_rget error\n");
      return 0.;
    }
}

/*****************************************************************************/

double MFAM_bnode_rset(MFAMt_bnode* node,MFAMt_btree* tree,double r)
{
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+node->k!=NULL))
    return (*(*(tree->r_level+node->n)+node->k)=r);
  else
    {
      fprintf(stderr,"MFAM_bnode_rset error\n");
      return 0.;
    }
}

/*****************************************************************************/

void MFAM_bnode_hprint(MFAMt_bnode* node,MFAMt_btree* tree,short n_ini, short n_fin)
{ 
   if (node!=NULL)
   {
      if ((node->n>=n_ini)&&(node->n<=n_fin))
      {
	 fprintf(stdout,"node # x=%s,n=%d,k=%d\t",node->x,node->n,node->k);
	 fprintf(stdout,"h=%f\n",*(*(tree->h_level+node->n)+node->k));
      }
      MFAM_bnode_hprint(node->lnode,tree,n_ini,n_fin);
      MFAM_bnode_hprint(node->rnode,tree,n_ini,n_fin);
   }
}
/*****************************************************************************/

int MFAM_btree_h_levelprint(MFAMt_btree *tree,short n)
{
  return MFAM_btree_level_print(tree,MFAM_HOELDER,n,n);
}

/*****************************************************************************/

int MFAM_btree_hprint(MFAMt_btree *tree,short n_ini,short n_fin)
{ 
   if(tree!=NULL)
   {
      if(tree->h_level!=NULL)
      {
	 int n=n_ini-1;
	 do  
	    n++;
	 while((*(tree->h_level+n)!=NULL)&&(n<=n_fin));
	 if (n<n_fin+1)
	 {
	    fprintf(stderr,"error\n");
	    return 0;
	 }
	 if (n_ini==n_fin)
	    MFAM_btree_h_levelprint(tree,n_ini);
	 else
	    MFAM_bnode_hprint(tree->root,tree,n_ini,n_fin);
      }
      else
      {
	 fprintf(stderr,"error\n");
	 return 0;
      }
      return 1;
   }
   else
   { 
      fprintf(stderr,"error\n");
      return 0;
   }
}

/*****************************************************************************/

double MFAM_bnode_c_level_decimate(MFAMt_bnode* node,MFAMt_btree* tree,double (*capa)(double,double),short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    { 
      if(node->n<n_ini)
	{
	  MFAM_bnode_c_level_decimate(node->lnode,tree,capa,n_ini,n_fin);
	  MFAM_bnode_c_level_decimate(node->rnode,tree,capa,n_ini,n_fin); 
	  return 0.;
	}
      else if(node->n<n_fin)
	{
	  double c=0.; 
	  double c_tmp=0.; 
	  double sum_c=0.; 
	  sum_c+=c=c_ref=MFAM_bnode_c_level_decimate(node->lnode,tree,capa,n_ini,n_fin);
	  sum_c+=c_tmp=MFAM_bnode_c_level_decimate(node->rnode,tree,capa,n_ini,n_fin);
	  c=capa(c,c_tmp);
	  MFAM_bnode_cset(node,tree,c);
	  return sum_c;
	}
      else /* if(node->n=n_fin) */
	return MFAM_bnode_cget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_bnode_c_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_btree_c_level_decimate(MFAMt_btree* tree,MFAMt_capa t_capa,short n_ini,short n_fin)
{
 if(tree!=NULL)
   { 
 
#ifndef __STDC__
     double (*capa)(); 
#else /* __STDC__ */
     double (*capa)(double,double);
#endif /* __STDC__ */

     switch(t_capa)
       { 
       case MFAM_SUM:
	 norm_meas=1.;
	 capa=MFAM_sum;
	 break;    
       case MFAM_LPSUM:
	 norm_meas=1.;
	 p=2;
	 capa=MFAM_lpsum;
	 break;
       case MFAM_MIN:
	 capa=MFAM_min;
	 break;   
       case MFAM_MAX:
	 capa=MFAM_max;
	 break;   
       case MFAM_ISO:
	 {
	   double Q=20;
	   double c_min=0,c_max=-HUGE;
	   int two_pow_n=(int)pow(2.,(double)n_fin);
	   double* c=NULL;
	   c=(double*)MFAM_btree_level_get(tree,MFAM_CAPACITY,n_fin);
	   if(MFAM_min_max(two_pow_n,c,&c_min,&c_max)==0)
	     { 
	       fprintf(stderr,"MFAM_min_max error\n"); 
	       return 0; 
	     }
	   q=(c_max-c_min)/Q;
	   capa=MFAM_iso;
	   break;
	 }
       case MFAM_ADAPISO:
	 {
	   int two_pow_n=(int)pow(2.,(double)n_fin);
	   double* c=NULL;
	   c=(double*)MFAM_btree_level_get(tree,MFAM_CAPACITY,n_fin);
	   if((sigma2=MFAM_variance(two_pow_n,c,MFAM_mean(two_pow_n,c)))==HUGE)
	     {
	       fprintf(stderr,"MFAM_variance error\n");
	       return 0;
	     }
	   capa=MFAM_adapiso;
	   break;
	 } 
       default:
	 return 0;
       }
     MFAM_bnode_c_level_decimate(tree->root,tree,capa,n_ini,n_fin);
     return 1;
   }
 else
   {
      fprintf(stderr,"MFAM_btree_c_level_decimate arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

double MFAM_bnode_r_level_decimate(MFAMt_bnode* node,MFAMt_btree* tree,short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{
	  MFAM_bnode_r_level_decimate(node->lnode,tree,n_ini,n_fin);
	  MFAM_bnode_r_level_decimate(node->rnode,tree,n_ini,n_fin);
	  return 0.;
	}
      else if(node->n<n_fin)
	return MFAM_bnode_rset(node,tree,
			       MFAM_bnode_r_level_decimate(node->lnode,tree,n_ini,n_fin)+
			       MFAM_bnode_r_level_decimate(node->rnode,tree,n_ini,n_fin));
      else /* if(node->n=n_fin) */
	return MFAM_bnode_rget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_bnode_r_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_btree_r_level_decimate(MFAMt_btree* tree,short n_ini,short n_fin)
{
 if(tree!=NULL)
   {  
     MFAM_bnode_r_level_decimate(tree->root,tree,n_ini,n_fin);
     return 1;
   }
 else
   {
     fprintf(stderr,"MFAM_btree_r_level_decimate arguments error\n");
     return 0;
   } 
}

/*****************************************************************************/

void MFAM_bnode_binomial(MFAMt_bnode* node,MFAMt_btree* tree,double p0,double mu)
{ 
   if(node!=NULL)
   {
      MFAM_bnode_cset(node,tree,mu);
      MFAM_bnode_binomial(node->lnode,tree,p0,mu*p0);
      MFAM_bnode_binomial(node->rnode,tree,p0,mu*(1-p0));
   } 
}

/*****************************************************************************/

MFAMt_btree* MFAM_btree_binomial(short N,double p0)
{ 
   MFAMt_btree* tree=NULL;
   if((tree=MFAM_btree_create(N))==NULL)
   {
      fprintf(stderr,"MFAM_btree create error\n");
      return NULL;
   } 
   if(MFAM_btree_level_create(tree,MFAM_CAPACITY,0,N)==0)
     {
      fprintf(stderr,"MFAM_btree level create error\n");
      return NULL;
     }  
   MFAM_bnode_binomial(tree->root,tree,p0,1.);
   return tree;
}
