/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/canus/MFA.CMP/MFAM.CL/src/MFAM_histogram.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAM_histogram.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:56:31 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Sun Feb  7 15:56:32 1999
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1997-98 */

#include "MFAM_histogram.h"

/*****************************************************************************/

int MFAM_histogram(int n,double* n_sample,int* N,double* bin_x,double* h_x,double* dev_x)
{ 
  if((n>0)&&(n_sample!=NULL)&&(N!=NULL)&&(bin_x!=NULL)&&(h_x!=NULL)&&(dev_x!=NULL))
  {
    register int k=0,l=0;
    static int (*compare)()=MFAM_compare;
    qsort(n_sample,n,sizeof(double),compare);
    *bin_x=*n_sample;
    *h_x=0.;
    *dev_x=0.;
    if(*N==0)
    {
      while(k<n)
      {
	if(*(n_sample+k)!=*(bin_x+l))
	{
	  l++;
	  *(bin_x+l)=*(n_sample+k);
	  *(h_x+l)=0.;
	  *(dev_x+l)=*(bin_x+l)-*(bin_x+l-1);
	}
	*(h_x+l)+=1.;
	k++; 
      }
      *N=l+1;
    }
    else
    { 
      double dev=(*(n_sample+n-1)-*n_sample)/(double)(*N-1);
      if(MFAM_linspace(*N,*n_sample,*(n_sample+n-1),bin_x)==0)
      {
	fprintf(stderr,"MFAM_linspace error\n");
	return 0;
      }
      for(l=0;l<*N;l++)
      {
	*(dev_x+l)=dev;
	for(k=0;k<n;k++)
	{
	  if((*(bin_x+l)-dev/2<=*(n_sample+k))&&
	     (*(n_sample+k)<*(bin_x+l)+dev/2))
	      *(h_x+l)+=1.;
	}
      }
    }
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_histogram arguments error\n");
    return 0;   
  } 
}

/*****************************************************************************/

int MFAM_khistogram(int n,double* n_sample,MFAMt_kern t_kern,int N,double* h,MFAMt_width t_width,double* x,double* p_x)
{
  if((n>0)&&(n_sample!=NULL)&&(N>0)&&(h!=NULL)&&(x!=NULL)&&(p_x!=NULL))
  { 
    register int k=0,l=0;
    double h_n=.1;
    int K=n;
    double* bin_x=NULL; 
    double* h_x=NULL;
    double* dev_x=NULL;
    double (*kern)(double);  
    if((bin_x=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0;
    }
    if((h_x=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0;
    }
    if((dev_x=(double*)malloc((unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0;
    } 
    if(MFAM_histogram(n,n_sample,&K,bin_x,h_x,dev_x)==0)
    { 
      fprintf(stderr,"MFAM_histogram error\n");
      return 0;
    } 
    if((bin_x=(double*)realloc(bin_x,(unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"realloc error\n");
      return 0;
    }
    if((h_x=(double*)realloc(h_x,(unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"realloc error\n");
      return 0;
    }
    if((dev_x=(double*)realloc(dev_x,(unsigned int)(K*sizeof(double))))==NULL)
    {
      fprintf(stderr,"realloc error\n");
      return 0;
    }
    if(MFAM_linspace(N,*bin_x,*(bin_x+K-1),x)==0)
    { 
      fprintf(stderr,"MFAM_linspace error\n");
      return 0;
    }
    switch(t_kern)
    {  
      case MFAM_BOXCAR:
	kern=MFAM_boxcar;
	break; 
      case MFAM_TRIANGLE:
	kern=MFAM_triangle;
	break; 
      case MFAM_GAUSSIAN:
	kern=MFAM_gauss;
	break;
      case MFAM_MOLLIFIER:
	kern=MFAM_mol;
	break; 
      case MFAM_EPANECHNIKOV:
	kern=MFAM_epa;
	break;    
      case MFAM_QUARTIC:
        kern=MFAM_quartic;
	break;   
      case MFAM_INTEGRATED: 
      default:
	fprintf(stderr,"MFAM_khistogram arguments error\n");
	break;
    }
    if(MFAM_adaption(n,n_sample,K,dev_x,kern,N,h,t_width,x,p_x)==0)
    {
      fprintf(stderr,"MFAM_adaption error\n");
      return 0;
    }
    for(l=0;l<N;l++)
    {
      *(p_x+l)=0.;
      /* first value */
      /* n-2 values */
      for(k=1;k<K-1;k++)
      {
	if (t_width==MFAM_LOCALDEV)
	  fprintf(stderr,"%f\n",h_n=*(dev_x+k));
	else
	  h_n=*(h+l);
	*(p_x+l)+=*(h_x+k)*kern((*(x+l)-*(bin_x+k))/h_n)/(n*h_n);
      }
      /* last value */
    }
    free((char*)bin_x);
    free((char*)h_x);
    free((char*)dev_x);
    return 1;   
  }
  else
  {
    fprintf(stderr,"MFAM_khistogram arguments error\n");
    return 0;   
  }
}
