/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/FRACLAB-WKSP/FRACLAB/MFAM/SRC/MFAM_misc.c
* File type        : C source file
* Revision         : $Revision: 1.2 $
* RCS status       : $Id: MFAM_misc.c,v 1.2 1999/02/15 10:34:01 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:54:45 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Mon Nov 18 20:40:22 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* christophe Canus 1997-98 */

#include "MFAM_misc.h"

/*****************************************************************************/

int MFAM_compare(double* d1,double* d2)
{ 
  if (*d1<*d2) 
    return -1;
  else if (*d1>*d2) 
    return 1;  
  else
    return 0;
}

/*****************************************************************************/

int MFAM_base(int N_n,short* b)
{
  if((N_n>0)&&(b!=NULL))
  {
    short i=2;
    int boolean=1;
    short n=0; 
    while((i<10)&&boolean)
    { 
      n=(short)ceil(log((double)N_n)/log((double)i));
      if(pow((double)i,(double)n)==N_n)
      {
	boolean=0;
	*b=i;
      }
      i++;
    }
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_base arguments error\n");
    return 0;   
  }
}

/*****************************************************************************/

int MFAM_zeros(int M,double* x)
{
  if((M>0)&&(x!=NULL))
  {
    register int m=0;
    for(m=0;m<M;m++)
      *(x+m)=0.;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_zeros arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_isempty(int M,unsigned char* s)
{
  int boolean=0;
  register int m=0;
  while((boolean=((int)*(s+m)==0))&&(m<M))
    m++;
  return boolean;
}

/*****************************************************************************/

int MFAM_iszeros(int M,double* x)
{
  int boolean=0;
  register int m=0;
  for(m=0;m<M;m++)
    boolean=(*(x+m)==0.);
  return boolean;
}

/*****************************************************************************/

int MFAM_linspace(int M,double x_min,double x_max,double* x)
{
  if((M>0)&&(x_min<=x_max)&&(x!=NULL))
  {
    register int m=0;
    double d_x=(x_max-x_min)/(double)(M-1);
    if(M!=1)
      for(m=0;m<M;m++)
	*(x+m)=x_min+m*d_x;
    else
      *x=x_min;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_linspace arguments error\n");
    fprintf(stderr,"\tM=[7m%d[m\n",M);
    fprintf(stderr,"\tx_min=[7m%f[m\n",x_min);
    fprintf(stderr,"\tx_max=[7m%f[m\n",x_max);
    fprintf(stderr,"\tx=[7m%p[m\n",x);
    return 0;
  }
}

/*****************************************************************************/

int MFAM_logspace(int M,double x_min,double x_max,double* x)
{
  if((M>0)&&(x_min<=x_max)&&(x!=NULL))
  {
    if(M!=1)
    {  
      register int m=0; 
      double r=pow(x_max/x_min,1./(M-1));
      *x=x_min;
      for(m=1;m<M;m++)
	*(x+m)=r**(x+m-1);
    }
    else
      *x=x_min;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_logspace arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_decspace(int N_n,int M,double x_min,double x_max,double* x)
{
  if((M>0)&&(0.<=x_min)&&(x_min<=x_max)&&(x!=NULL))
  {
    register int short n;
    short b=0,n_min=0,n_max=0;
    if(MFAM_base(N_n,&b)==0)
    {
      fprintf(stderr,"MFAM_base error\n");
      return 0;
    }  
    n_min=-(short)(log(x_min)/log((double)b));
    n_max=-(short)(log(x_max)/log((double)b));
    if(n_max-n_min)
      for(n=n_min;((n>=n_max)&&(n-n_min<=M));n--)
	*(x+n_min-n)=pow((double)b,(double)-n);
    else
      *x=x_min;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_decspace arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_progstr(char* progstr,MFAMt_prog* t_prog)
{
  if(strcmp(MFAM_LINSTR,progstr)==0)
    *t_prog=MFAM_LIN;
  else if(strcmp(MFAM_LOGSTR,progstr)==0)
    *t_prog=MFAM_LOG;
  else if(strcmp(MFAM_DECSTR,progstr)==0)
    *t_prog=MFAM_DEC;
  else 
    return 0;
  return 1;
}

/*****************************************************************************/

int MFAM_scale(int N_n,double S_min,double S_max,short J,double* eta,MFAMt_prog t_prog,int d)
{
  if((N_n>0)&&(S_min>=1)&&(S_max>=S_min)&&(S_max<=N_n)&&(J>0)&&(eta!=NULL))
  {
    double eta_1=S_min/N_n,eta_J=S_max/N_n;
    if(d==2)
    {
      eta_1=sqrt(eta_1);
      eta_J=sqrt(eta_J);
    }
    switch(t_prog)
    {
    case MFAM_LIN:
      if(MFAM_linspace(J,eta_1,eta_J,eta)==0)  
      {  
	fprintf(stderr,"MFAM_linspace error\n");  
	return 0;  
      }
      break;
    case MFAM_LOG:  
      if(MFAM_logspace(J,eta_1,eta_J,eta)==0)  
      {  
	fprintf(stderr,"MFAM_logspace error\n");  
	return 0;  
      }  
      break;
    case MFAM_DEC:
      if(MFAM_decspace(N_n,J,eta_1,eta_J,eta)==0)
      {
	fprintf(stderr,"MFAM_decspace error\n");  
	return 0;  
      }
      break;
    } 
  } 
  else
  { 
    fprintf(stderr,"MFAM_scale arguments error\n");
    return 0;
  }
  return 1;
}

/*****************************************************************************/

int MFAM_decsizespace(int N_n,int M,double x_min,double x_max,double* x)
{
  if((N_n>0)&&(M>0)&&(0.<=x_min)&&(x_min<=x_max)&&(x_max<=N_n/2.)&&(x!=NULL))
  {
    register int short n=0;
    short b=0,n_min=0,n_max=0;
    if(MFAM_base(N_n,&b)==0)
    {
      fprintf(stderr,"MFAM_base error\n");
      return 0;
    }  
    n_min=(short)ceil(log(x_min)/log((double)b));
    n_max=(short)ceil(log(x_max)/log((double)b));
    if(n_max-n_min+1==M)
    {
      if(n_max-n_min)
	for(n=n_min;n<=n_max;n++)
	  *(x+n)=pow((double)b,(double)n);
      else
	*x=x_max;
    }
    else
    {
      fprintf(stderr,"MFAM_decsizespace arguments error\n");
      return 0;
    }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_decsizespace arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_partstr(char* partstr,MFAMt_part* t_part)
{
  if(strcmp(MFAM_LINPARTSTR,partstr)==0)
    *t_part=MFAM_LINPART;
  else if(strcmp(MFAM_LOGPARTSTR,partstr)==0)
    *t_part=MFAM_LOGPART;
  else if(strcmp(MFAM_DECPARTSTR,partstr)==0)
    *t_part=MFAM_DECPART;
  else if(strcmp(MFAM_DECSIZESTR,partstr)==0)
    *t_part=MFAM_DECSIZE;
  else 
    return 0;
  return 1;
}

/*****************************************************************************/

int MFAM_partspace(int N_n,MFAMt_part t_part,int J,double P_min,double P_max,double* P)
{
  if((N_n>0)&&(J>0)&&(1<=P_min)&&(P_min<=P_max)&&(P_max<=N_n)&&(P!=NULL))
  { 
    register int j=0;
    switch(t_part)
    {
    case MFAM_LINPART:
      if(MFAM_linspace(J,P_min,P_max,P)==0)  
      {  
	fprintf(stderr,"MFAM_linspace error\n");  
	return 0;  
      }
      break;
    case MFAM_LOGPART:  
      if(MFAM_logspace(J,P_min,P_max,P)==0)  
      {  
	fprintf(stderr,"MFAM_logspace error\n");  
	return 0;  
      }  
      break;
    case MFAM_DECPART:
      if(MFAM_decspace(N_n,J,P_min,P_max,P)==0)
      {
	fprintf(stderr,"MFAM_decspace error\n");  
	return 0;  
      }
      break;
    default:
      fprintf(stderr,"MFAM_partspace arguments error\n");
      return 0;
    }
    if(J!=1)
      for(j=0;j<J;j++)
	*(P+j)=floor(*(P+j));
    else
      *P=P_min;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_partspace arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_bilogspace(int M,short d,double x_min,double x_max,double* x)
{
  if((M>0)&&(x_min<=x_max)&&(x!=NULL))
  {
    if(M!=1)
    {  
      register int m=0; 
      double r=pow(100,1./(M-1));
      if(M%2!=0)
      {
	*(x+M/2)=1;
	for(m=0;m<=M/2-1;m++)
	  *(x+M/2-1-m)=r**(x+M/2-m);
	for(m=M/2+1;m<M;m++)
	  *(x+m)=r**(x+m-1);
	/* for(m=0;m<=M/2-1;m++) */
/* 	  *(x+m)=(*(x+M/2)-*(x+m))/(*x-*(x+M/2)); */
	for(m=M/2;m<M;m++)
	  *(x+m)=d+(*(x+m)-*(x+M/2))/(*(x+M-1)-*(x+M/2))*(x_max-d);
      }
      else
      {
	*(x+M/2)=1;
	for(m=0;m<=M/2-1;m++)
	  *(x+M/2-1-m)=r**(x+M/2-m);
	for(m=M/2+1;m<M;m++)
	  *(x+m)=r**(x+m-1);
	fprintf(stderr,"%f %f %f\n",*x,*(x+M/2),*(x+M-1));
      }
    }
    else
      *x=x_min;
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_bilogspace arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_weights(short b,double* p)
{
  if((b>1)&&(p!=NULL))
  {
    register int i=0;
    double sum_p_1=0.;
    fprintf(stderr,"The chosen base is: b = %d\n",b);
    fprintf(stderr,"Enter the %d weights: \n",b-1);
    for(i=0;i<b-1;i++)
    {
      fprintf(stderr,"\tweight # %d: p_%d = ",i+1,i);
      fscanf(stdin,"%lf",p+i);
      sum_p_1+=*(p+i);
    }
    if(sum_p_1>1.)
    { 
      fprintf(stderr,"The sum of the weights can't be equal to 1.\n");
      return 0;
    }
    *(p+b-1)=1-sum_p_1; 
    fprintf(stderr,"\tweight # %d: p_%d = %.6f\n",b,b-1,*(p+b-1));
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_weights arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_weights2d(short bx,short by,double* p)
{
  if((bx>1)&&(by>1)&&(p!=NULL))
  {
    register int i=0,j=0; 
    double sum_p_1=0.;
    fprintf(stderr,"The chosen bases are: bx = %d, by = %d\n",bx,by);
    fprintf(stderr,"Enter the %d weights: \n",bx*by-1);
    for (i=0;i<bx;i++)
    {
      for (j=0;j<by;j++)
      {    
	if(!((i==bx-1)&&(j==by-1)))
	{   
	  fprintf(stderr,"\tweight # %d: p_%d%d = ",i*by+j+1,i,j);
	  fscanf(stdin,"%lf",p+i*by+j);
	  sum_p_1+=*(p+i*by+j);
	}   
      }
    } 
    if(sum_p_1>1.)
    { 
      fprintf(stderr,"The sum of the weights can't be equal to 1.\n");
      return 0;
    }
    *(p+bx*by-1)=1-sum_p_1; 
    fprintf(stderr,"\tweight # %d: p_%d%d = %.6f\n",bx*by,bx-1,by-1, *(p+bx*by-1));
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_weights2d arguments error\n");
    return 0;
  }
}

/*****************************************************************************/

int MFAM_min_max(int n,double* n_sample,double* min,double* max)
{
  if((n>0)&&(n_sample!=NULL)&&(min!=NULL)&&(max!=NULL))
  {
    register int k=0; 
    *min=HUGE;
    *max=-HUGE;
    while(k<n)
    {
      if(*(n_sample+k)!=HUGE)
      {
	*min=(*(n_sample+k)<*min?*(n_sample+k):*min);
	*max=(*(n_sample+k)>*max?*(n_sample+k):*max);
      }
      k++;
    }  
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_min_max arguments error\n");
    return 0;
  }   
}

/*****************************************************************************/

int MFAM_min_max_dev(int n,double* n_sample,double* min_dev,double* max_dev)
{
  if((n>0)&&(n_sample!=NULL)&&(max_dev!=NULL)&&(min_dev!=NULL))
  {
    register int k=0;
    double dev=0.;
    while(k<n-1)
    { 
      if((*(n_sample+k)!=HUGE)&&(*(n_sample+k+1)!=HUGE))
      {
	dev=fabs(*(n_sample+k)-*(n_sample+k+1));
	*min_dev=(dev<*min_dev?dev:*min_dev);
	*max_dev=(dev>*max_dev?dev:*max_dev);
      }
      k++;
    }  
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_min_max_dev arguments error\n");
    return 0;   
  }
}

/*****************************************************************************/

int MFAM_min_dev(int n,double* n_sample,double* min_dev)
{
  if((n>0)&&(n_sample!=NULL)&&(min_dev!=NULL))
  {
    register int k=0;
    double dev=0.;
    while(k<n-1)
    { 
      if((*(n_sample+k)!=HUGE)&&(*(n_sample+k+1)!=HUGE))
	if((dev=fabs(*(n_sample+k)-*(n_sample+k+1))))
	  *min_dev=(dev<*min_dev?dev:*min_dev);
      k++;
    }  
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_min_dev arguments error\n");
    return 0;   
  }
}

/*****************************************************************************/

int MFAM_max_dev(int n,double* n_sample,double* max_dev)
{
  if((n>0)&&(n_sample!=NULL)&&(max_dev!=NULL))
  {
    register int k=0;
    double dev=0.;
    while(k<n-1)
    { 
      if((*(n_sample+k)!=HUGE)&&(*(n_sample+k+1)!=HUGE))
	if((dev=fabs(*(n_sample+k)-*(n_sample+k+1))))
	  *max_dev=(dev>*max_dev?dev:*max_dev);
      k++;
    }  
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_max_dev arguments error\n");
    return 0;   
  }
}

/*****************************************************************************/

int MFAM_mean_dev(int n,double* n_sample,double* mean_dev)
{
  if((n>0)&&(n_sample!=NULL)&&(mean_dev!=NULL))
  {
    register int k=0;
    int N=0;
    double dev=0.;
    while(k<n-1)
    { 
      if((*(n_sample+k)!=HUGE)&&(*(n_sample+k+1)!=HUGE))
	if((dev=fabs(*(n_sample+k)-*(n_sample+k+1))))
	{
	  *mean_dev+=dev;
	  N++;
	}
      k++;
    } 
    *mean_dev/=N; 
    return 1;
  }
  else
  { 
    fprintf(stderr,"MFAM_mean_dev arguments error\n");
    return 0;   
  }
}

/*****************************************************************************/

int MFAM_norm(int M,double* s_x,double* norm_s_x,int d)
{
  if((M>0)&&(s_x!=NULL)&&((d==0)||(d==1)||(d==2)))
  {
    register int k=0;
    if(norm_s_x==NULL)
      norm_s_x=s_x;
    if(d==0)
      {  
	double sum_s_x=0.;
	for(k=0;k<M;k++)
	  sum_s_x+=*(s_x+k);
	for(k=0;k<M;k++)
	  *(norm_s_x+k)=*(s_x+k)/sum_s_x; 
      }
    else
      {
	double s_x_min=HUGE,s_x_max=-HUGE; 
	if(MFAM_min_max(M,s_x,&s_x_min,&s_x_max)==0)
	  { 
	    fprintf(stderr,"MFAM_min_max error\n");
	    return 0;
	  }
	if(s_x_min!=s_x_max)
	  for(k=0;k<M;k++)
	    *(norm_s_x+k)=d*(*(s_x+k)-s_x_min)/(s_x_max-s_x_min);
	else if(norm_s_x!=s_x)
	  for(k=0;k<M;k++)
	    *(norm_s_x+k)=*(s_x+k);
      }
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_norm arguments error\n");
    return 0;
  }
}


/*****************************************************************************/

int MFAM_progress(int k,int N)
{
  if (N>0)
    {

#if defined(_MSC_VER)

#else /* defined(_MSC_VER) */

      static struct timeval start;
      static struct timeval lastupdate;
      struct timeval now, td, wait; 
      double elapsed;
      const int ttywidth=80;
      int ratio=0,barlength=0,i=0,remaining=0;

      ratio=100.0*(double)k/(double)N;
      ratio=MIN(ratio,100);
      ratio=MAX(ratio,0);
      if(k==0)
	{
	  fprintf(stderr,"\nstart ...\n");
	  gettimeofday(&start,(struct timezone *)0);
	  lastupdate=start;
	}
      gettimeofday(&now, (struct timezone *) 0); 
      fprintf(stderr,"[7m\r%3d%%[m completed   ",ratio);  
      barlength=ttywidth-51;
      if(barlength>0)
	{
	  i=barlength*ratio/100;
	  fprintf(stderr,
		  "|%.*s%*s|",i,
		  "*******************************************************",
		  barlength-i,"");
	}
      i=0;
      TIMERSUB(&now,&lastupdate,&wait);
      lastupdate=now;
      wait.tv_sec = 0;
      TIMERSUB(&now,&start,&td);
      if(k)
	{
	  elapsed=td.tv_sec+(td.tv_usec/1000000.0);
	  remaining=(int)(N/(k/elapsed)-elapsed);
	  i=remaining/3600;
	  if(i)
	    fprintf(stderr,"%2d:",i);
	  else
	    fprintf(stderr,"    ");
	  i=remaining%3600;
	  fprintf(stderr,"%02d:%02d remaining (estim.)  ",i/60,i%60);
	}
      if(k==N)
	fprintf(stderr,"\nfinished.\n");

#endif /* defined(_MSC_VER) */

    }
  return 1;
}
