/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAM.CL/src/MFAM_segmentation.c
* File type        : C source file
* Revision         : $Revision: 1.1.1.1 $
* RCS status       : $Id: MFAM_segmentation.c,v 1.1.1.1 1999/02/10 13:29:04 guiheneu Exp $
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Sun Feb  7 15:53:59 1999
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Thu Sep 19 11:57:35 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 1998-2002 */

#include "MFAM_segmentation.h"

/*****************************************************************************/
  
int MFAM_segmentation1d(int Nx,double* signal1d,int L,double* min,double* max,unsigned char* set1d)
{
  if((Nx>0)&&(signal1d!=NULL)&&(L>0)&&(*min>=0.)&&(*max>=*min)&&(set1d!=NULL))
  {
    register int k=0,l=0;
    for(k=0;k<Nx;k++)
      *(set1d+k)=((((*min<=*(signal1d+k))&&(*(signal1d+k)<*max))==1)?1:0); 
    for(l=1;l<L;l++)
      for(k=0;k<Nx;k++)
	*(set1d+k)=(((*(min+l)<=*(signal1d+k))&&(*(signal1d+k)<*(max+l))==1)?1:*(set1d+k)); 
    return 1;  
  }
  else
  {
    fprintf(stderr,"MFAM_segmentation1d arguments error\n");
    fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
    fprintf(stderr,"\tsignal1d=[7m%p[m\n",signal1d);  
    fprintf(stderr,"\tL=[7m%d[m\n",L);  
    fprintf(stderr,"\t*min=[7m%f[m\n",*min);
    fprintf(stderr,"\t*max=[7m%f[m\n",*max);
    fprintf(stderr,"\tset1d=[7m%p[m\n",set1d);
    return 0;
  }
}

/*****************************************************************************/
  
int MFAM_segmentation2d(int N,int M,double* signal2d,int L,double* min,double* max,unsigned char* segmentation2d)
{
  if((N>0)&&(M>0)&&(signal2d!=NULL)&&(L>0)&&(*min>=0.)&&(*max>=*min)&&(segmentation2d!=NULL))
  {
    register int k=0,l=0;
    for(k=0;k<N*M;k++)
      *(segmentation2d+k)=((((*min<=*(signal2d+k))&&(*(signal2d+k)<*max))==1)?1:0); 
    for(l=1;l<L;l++)
      for(k=0;k<N*M;k++)
	*(segmentation2d+k)=(((*(min+l)<=*(signal2d+k))&&(*(signal2d+k)<*(max+l))==1)?1:*(segmentation2d+k)); 
    return 1;  
  }
  else
  {
    fprintf(stderr,"MFAM_segmentation2d arguments error\n");
    fprintf(stderr,"\tN=[7m%d[m\n",N);
    fprintf(stderr,"\tM=[7m%d[m\n",M);
    fprintf(stderr,"\tsignal2d=[7m%p[m\n",signal2d);  
    fprintf(stderr,"\tL=[7m%d[m\n",L);  
    fprintf(stderr,"\t*min=[7m%f[m\n",*min);
    fprintf(stderr,"\t*max=[7m%f[m\n",*max);
    fprintf(stderr,"\tsegmentation2d=[7m%p[m\n",segmentation2d);
    return 0;
  }
}

/*****************************************************************************/

int MFAM_type(double f_alpha,double f_alpha0,double Delta_f_alpha0)
{
  if(f_alpha<f_alpha0-Delta_f_alpha0/2.)
    return -1;
  else if((f_alpha0-Delta_f_alpha0/2.<=f_alpha)&&(f_alpha<=f_alpha0+Delta_f_alpha0/2.))
    return 0;
  else /* f_alpha>f_alpha0+Delta_f_alpha0/2. */
    return 1;
}

/*****************************************************************************/

int MFAM_msegmentation2d(int Nx,int Ny,double* p_alpha2d,int N,double* alpha,double* f_alpha,double f_alpha0,double Delta_f_alpha0,unsigned char* msegmentation2d)
{
  if((Nx>0)&&(Ny>0)&&(p_alpha2d!=NULL)&&(N>0)&&(alpha!=NULL)&&(f_alpha!=NULL)&&(f_alpha0>0.)&&(Delta_f_alpha0>0.)&&(msegmentation2d!=NULL))
  {
    register int n=0,l=0;
    int L=N;
    int type_n_1=0,type_n=0;
    double* alpha_min=NULL;
    double* alpha_max=NULL;
    if((alpha_min=(double*)malloc((unsigned int)(L*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0.;
    } 
    if((alpha_max=(double*)malloc((unsigned int)(L*sizeof(double))))==NULL)
    {
      fprintf(stderr,"malloc error\n");
      return 0.;
    }
    type_n_1=MFAM_type(*(f_alpha+n),f_alpha0,Delta_f_alpha0);
    fprintf(stderr,"f_alpha0-Delta_f_alpha0/2.=%f\n",f_alpha0-Delta_f_alpha0/2.);
    fprintf(stderr,"f_alpha0+Delta_f_alpha0/2.=%f\n",f_alpha0+Delta_f_alpha0/2.);

#ifndef MFAHNOPROGESS
	  MFAM_progress(0,N);
#endif /* MFAHNOPROGESS */

    n=1;
    while(n<N-1)
    { 
#ifndef MFAHNOPROGESS
	  MFAM_progress(n,N);
#endif /* MFAHNOPROGESS */

      type_n=MFAM_type(*(f_alpha+n),f_alpha0,Delta_f_alpha0);
      fprintf(stderr,"f_alpha(%d)=%f\n",n-1,*(f_alpha+n-1));
      fprintf(stderr,"f_alpha(%d)=%f\n",n,*(f_alpha+n));
      fprintf(stderr,"type_n_1=%d\n",type_n_1);
      fprintf(stderr,"type_n=%d\n",type_n);
      if(((type_n_1==-1)&&(type_n==1))||
	 ((type_n_1==1)&&(type_n==-1)))
      {
	double a=(*(f_alpha+n)-*(f_alpha+n-1))/(*(alpha+n)-*(alpha+n-1));
	fprintf(stderr,"a=%f\n",a);
	fprintf(stderr,"alpha(%d)=%f\n",n-1,*(alpha+n-1));
	fprintf(stderr,"alpha(%d)=%f\n",n,*(alpha+n));
	if(a>0.)
	{
	  *(alpha_min+l)=*(alpha+n-1)+(f_alpha0-Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	  *(alpha_max+l)=*(alpha+n-1)+(f_alpha0+Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	}
	else
	{
	  *(alpha_min+l)=*(alpha+n-1)+(f_alpha0+Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	  *(alpha_max+l)=*(alpha+n-1)+(f_alpha0-Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	}
	fprintf(stderr,"found min: n=%d,alpha_min=%f\n",n,*(alpha_min+l));
	fprintf(stderr,"found max: n=%d,alpha_max=%f\n",n,*(alpha_max+l));
	l++;
      } 
      if(type_n==0)
      {
	double a=(*(f_alpha+n)-*(f_alpha+n-1))/(*(alpha+n)-*(alpha+n-1));
	if (type_n_1==-1) 
	  *(alpha_min+l)=*(alpha+n-1)+(f_alpha0-Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	else
	  *(alpha_min+l)=*(alpha+n-1)+(f_alpha0+Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	fprintf(stderr,"found min: n=%d,alpha_min=%f\n",n,*(alpha_min+l));
	while((n<N-1)&&(type_n==0)) 
 	  { 
 	    fprintf(stderr,"f_alpha(%d)=%f\n",n,*(f_alpha+n)); 
 	    n++;  
	    type_n=MFAM_type(*(f_alpha+n),f_alpha0,Delta_f_alpha0);
	    fprintf(stderr,"type_n=%d\n",type_n);
 	  } 
	a=(*(f_alpha+n)-*(f_alpha+n-1))/(*(alpha+n)-*(alpha+n-1));
	if (type_n==-1)
	  *(alpha_max+l)=*(alpha+n-1)+(f_alpha0-Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	else
	  *(alpha_max+l)=*(alpha+n-1)+(f_alpha0+Delta_f_alpha0/2.-*(f_alpha+n-1))/a;
	fprintf(stderr,"found max: n=%d,alpha_max=%f\n",n,*(alpha_max+l));
	l++;
      }
      n++;
      type_n_1=type_n;
    }
    L=l;
    for(l=0;l<L;l++)
      fprintf(stderr,"min=%f max=%f\n",*(alpha_min+l),*(alpha_max+l));
    if(L)
    {
      if(MFAM_segmentation2d(Nx,Ny,p_alpha2d,L,alpha_min,alpha_max,msegmentation2d)==0)
      { 
	fprintf(stderr,"MFAM_segmentation2d error\n");
	return 0;
      } 
    }
    free((char*)alpha_min);
    free((char*)alpha_max);
    return 1;
  }
  else
  {
    fprintf(stderr,"MFAM_msegmentation2d arguments error\n");   
    fprintf(stderr,"\tNx=[7m%d[m\n",Nx);  
    fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
    fprintf(stderr,"\tp_alpha2d=[7m%p[m\n",p_alpha2d); 
    fprintf(stderr,"\tN=[7m%d[m\n",N);
    fprintf(stderr,"\talpha=[7m%p[m\n",alpha); 
    fprintf(stderr,"\tf_alpha=[7m%p[m\n",f_alpha); 
    fprintf(stderr,"\tf_alpha0=[7m%f[m\n",f_alpha0); 
    fprintf(stderr,"\tDelta_f_alpha0=[7m%f[m\n",Delta_f_alpha0);
    fprintf(stderr,"\tmsegmentation2d=[7m%p[m\n",msegmentation2d);
    return 0;
  }
}
