/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAM.CL/src/MFAM_tree.c
* File type        : C source file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Tue Sep  3 17:06:20 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Thu Sep 19 14:25:27 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#include "MFAM_tree.h"

extern double norm_meas;
extern double p;
extern double c_ref;
extern double q;
extern double sigma2;

/*****************************************************************************/

MFAMt_node* MFAM_node_create(MFAMt_node* node,char* x,short b,short i,short n,int k,short N)
{
  if(node==NULL)
    {	
      if((node=(MFAMt_node*)malloc(sizeof(MFAMt_node)))==NULL)
	{ 
	  fprintf(stderr,"MFAM_node_create: malloc error\n");
	  return NULL;
	}
      node->x=x; 
      node->b=b; 
      node->i=i;
      node->n=n;
      node->k=k;
      if((node->nodes=(MFAMt_node**)malloc(b*sizeof(MFAMt_node*)))==NULL)
	{ 
	  fprintf(stderr,"MFAM_node_create: malloc error\n");
	  return NULL;
	}
      memset(node->nodes,(int)NULL,b*sizeof(MFAMt_node*));
      if (n!=N)
	{
	  register int i=0;
	  for(i=0;i<node->b;i++)
	    {
	      char* xi=NULL;
	      xi=(char*)malloc((strlen(x)+2)*sizeof(char));
	      sprintf(xi,"%s%d",x,i); 
	      *(node->nodes+i)=MFAM_node_create(*(node->nodes+i),xi,b,i,n+1,b*k+i,N);
	    }
	}
    }
  return node;
}

/*****************************************************************************/

void MFAM_node_print(MFAMt_node* node,int recursive_print)
{ 
  if (node!=NULL)
    {
      fprintf(stderr,"%d_node of type: %d level n=%d, x=%s, k=%d\n",node->b,node->i,node->n,node->x,node->k);
      if (recursive_print)
	{
	  register int i=0;
	  for(i=0;i<node->b;i++)
	    MFAM_node_print(*(node->nodes+i),recursive_print);
	}
    }
}

/*****************************************************************************/

void MFAM_node_delete(MFAMt_node* node)
{ 
  if(node!=NULL)
    {
      register int i=0;
      for(i=0;i<node->b;i++)
	{
	  if(*(node->nodes+i)!=NULL)
	    MFAM_node_delete(*(node->nodes+i));
	  *(node->nodes+i)=NULL;
	}
      free(node->x);
      free((char*)node);
    }
}

/*****************************************************************************/

MFAMt_tree* MFAM_tree_create(short N,short b)
{
  MFAMt_tree* tree=NULL; 
  char* x=NULL;
  if ((tree=(MFAMt_tree*)malloc(sizeof(MFAMt_tree)))==NULL) 
    { 
      fprintf(stderr,"MFAM_tree_create: malloc error\n");
      return NULL;
    }
  tree->N=N;
  tree->b=b;
  tree->root=NULL;
  x=(char*)strdup("0.");
  if ((tree->root=MFAM_node_create(tree->root,x,b,0,0,0,N))==NULL)
    { 
      fprintf(stderr,"root node creation error\n");
      return NULL;
    }  
  if((tree->c_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->c_level,(int)NULL,(N+1)*sizeof(double*));
 
  if((tree->r_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->r_level,(int)NULL,(N+1)*sizeof(double*));

  if((tree->h_level=(double**)malloc((N+1)*sizeof(double*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    }  
  memset(tree->h_level,(int)NULL,(N+1)*sizeof(double*));

  if((tree->s_level=(unsigned char**)malloc((N+1)*sizeof(unsigned char*)))==NULL)
    { 
      fprintf(stderr,"malloc error\n");
      return 0;
    } 
  memset(tree->s_level,(int)NULL,(N+1)*sizeof(unsigned char*)); 
  
  return tree;
}

/*****************************************************************************/

void MFAM_tree_delete(MFAMt_tree* tree)
{ 
  if (tree!=NULL)
    {
      int n=0;
      MFAM_node_delete(tree->root);
      while(n<=tree->N)
	{
	  if (*(tree->c_level+n)!=NULL)
	    {
	      free((char*)*(tree->c_level+n));
	      *(tree->c_level+n)=NULL;
	    }
	  if (*(tree->r_level+n)!=NULL)
	    {
	      free((char*)*(tree->r_level+n));
	      *(tree->r_level+n)=NULL;
	    }
	  if (*(tree->h_level+n)!=NULL)
	    {
	      free((char*)*(tree->h_level+n));
	      *(tree->h_level+n)=NULL;
	    }
	  if (*(tree->s_level+n)!=NULL)  
	    {
	      free((char*)*(tree->s_level+n));
	      *(tree->s_level+n)=NULL;
	    }
	  n++;
	}
      free((char*)tree->c_level);    
      free((char*)tree->r_level);   
      free((char*)tree->h_level);
      free((char*)tree->s_level);
      free((char*)tree);
    }
}

/*****************************************************************************/

void MFAM_tree_print(MFAMt_tree* tree)
{ 
  if (tree!=NULL)
    {
      fprintf(stderr,"%d_tree with # N=%d levels\n",tree->b,tree->N);
      MFAM_node_print(tree->root,1);
    }
}

/*****************************************************************************/

int MFAM_tree_level_create(MFAMt_tree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    { 
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree capacity level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->c_level+n)=(double*)malloc(b_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_tree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    { 
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree reference level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->r_level+n)=(double*)malloc(b_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_tree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree hoelder level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->h_level+n)=(double*)malloc(b_pow_n*sizeof(double)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_tree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree set level create n=%d\n",n);
#endif /* MFAMDEBUG */
	      if((*(tree->s_level+n)=(unsigned char*)malloc(b_pow_n*sizeof(unsigned char)))==NULL) 
		{ 
		  fprintf(stderr,"MFAM_tree_level_create malloc error\n");
		  return 0;
		}
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_tree_level_create arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_tree_level_create arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_tree_level_delete(MFAMt_tree* tree,MFAMt_level level_type,short n_ini,short n_fin)
{
  if (tree!=NULL)
    {
      short n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      if(*(tree->c_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"tree capacity level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->c_level+n));
		  *(tree->c_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_tree_level_delete (c) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      if(*(tree->r_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"tree reference level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->r_level+n));
		  *(tree->r_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_tree_level_delete (r) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      if(*(tree->h_level+n)!=NULL)
		{  
#ifdef MFAMDEBUG
		  fprintf(stderr,"tree hoelder level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->h_level+n));
		  *(tree->h_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_tree_level_delete (h) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      if(*(tree->s_level+n)!=NULL)
		{      
#ifdef MFAMDEBUG
		  fprintf(stderr,"tree set level delete n=%d\n",n);
#endif /* MFAMDEBUG */
		  free((char*)*(tree->s_level+n));
		  *(tree->s_level+n)=NULL;
		}
	      else
		{ 
		  fprintf(stderr,"MFAM_tree_level_delete (s) free error\n");
		  return 0;
		}
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_tree_level_delete arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {  
      fprintf(stderr,"MFAM_tree_level_delete arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_tree_level_set(MFAMt_tree* tree,MFAMt_level level_type,short n,void* level)
{
  if((tree!=NULL)&&(n>0))
    {
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if(((*(tree->c_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->c_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree capacity level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->c_level+n)=(double*)level;
	    }
	  break;    
	case MFAM_REFERENCE:
	  if(((*(tree->r_level+n)==NULL)&&(level!=NULL))||
	     ((*(tree->r_level+n)!=NULL)&&(level==NULL)))
  	    { 
#ifdef MFAMDEBUG
	      fprintf(stderr,"tree reference level set n=%d\n",n);
#endif /* MFAMDEBUG */
	      *(tree->r_level+n)=(double*)level;
	    }
	  break;   
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET:  
	  *(tree->s_level+n)=(unsigned char*)level;
	  break;   
	default:
	  fprintf(stderr,"MFAM_tree_level_set arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_level_set arguments error\n");
      return 0;
    } 
}

/*****************************************************************************/

int MFAM_tree_level_convert(MFAMt_tree* tree,MFAMt_level level_type,short n,int Nx,void* level)
{
  if((tree!=NULL)&&(n>0)&&(Nx>0)&&(level!=NULL))
    { 
      register int x=0;
      int b_pow_n=(int)pow((double)tree->b,(double)n);
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  if((*(tree->c_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_c1d=NULL;
	      double* tmp_level1d=NULL;
	      tmp_c1d=(double*)MFAM_tree_level_get(tree,MFAM_CAPACITY,n);
	      tmp_level1d=(double*)level;
	      memset(tmp_c1d,0,(size_t)(b_pow_n*sizeof(double)));
	      for(x=0;x<Nx;x++)
		*(tmp_c1d+x)=(double)*(tmp_level1d+x);
	    }
	  break; 
	case MFAM_REFERENCE:
	  if((*(tree->r_level+n)!=NULL)&&(level!=NULL))
  	    {  
	      double* tmp_mu1d=NULL;
	      double* tmp_level1d=NULL;
	      tmp_mu1d=(double*)MFAM_tree_level_get(tree,MFAM_REFERENCE,n);
	      tmp_level1d=(double*)level;
	      memset(tmp_mu1d,0,(size_t)(b_pow_n*sizeof(double)));
	      for(x=0;x<Nx;x++)
		  *(tmp_mu1d+x)=(double)*(tmp_level1d+x);
	    }
	  break;     
	case MFAM_HOELDER:
	  *(tree->h_level+n)=(double*)level;
	  break;    
	case MFAM_SET: 
	  if((*(tree->s_level+n)!=NULL)&&(level!=NULL))
  	    {     
	      unsigned char* tmp_set1d=NULL;
	      unsigned char* tmp_level1d=NULL;	    
	      tmp_set1d=(unsigned char*)MFAM_tree_level_get(tree,MFAM_SET,n);
	      tmp_level1d=(unsigned char*)level;
	      memset(tmp_set1d,0,(size_t)(b_pow_n*sizeof(unsigned char)));
	      for(x=0;x<Nx;x++)
		*(tmp_set1d+x)=(unsigned char)*(tmp_level1d+x);
	    }
	  break;   
	default:
	  fprintf(stderr,"MFAM_tree_level_convert arguments error\n");
	  break;
	}
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_level_convert arguments error\n");
      return 0;
   } 
}

/*****************************************************************************/

void* MFAM_tree_level_get(MFAMt_tree* tree,MFAMt_level level_type,short n)
{
  if((tree!=NULL)||(n>0))
    { 
      void* level=NULL;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  level=(void*)*(tree->c_level+n); 
	  break;  
	case MFAM_REFERENCE:
	  level=(void*)*(tree->r_level+n); 
	  break;   
	case MFAM_HOELDER:
	  level=(void*)*(tree->h_level+n);
	  break;    
	case MFAM_SET:  
	  level=(void*)*(tree->s_level+n);
	  break;   
	default:
	  fprintf(stderr,"MFAM_tree_level_get arguments error\n");
	  break;
	}
      return level;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_level_get arguments error\n");
      return NULL;
    }   
}

/*****************************************************************************/

int MFAM_tree_level_print(MFAMt_tree *tree,MFAMt_level level_type,short n_ini,short n_fin)
{ 
  if(tree!=NULL)
    {
      int n=n_ini;
      switch (level_type)
	{	
	case MFAM_CAPACITY:
	  while(n<=n_fin)
	    {
	      int k=0;
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
	      fprintf(stderr,"printing capacities of level # %d\n",n);
	      for(k=0;k<b_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->c_level+n)+k));
	      n++;
	    }
	  break;
	case MFAM_REFERENCE:
	  while(n<=n_fin)
	    {
	      int k=0;
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
	      fprintf(stderr,"printing reference measures of level # %d\n",n);
	      for(k=0;k<b_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->r_level+n)+k));
	      n++;
	    }
	  break;    
	case MFAM_HOELDER:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
	      fprintf(stderr,"printing hoelder exponents of level # %d\n",n);
	      for(k=0;k<b_pow_n;k++)
		fprintf(stderr,"%d %g\n",k,*(*(tree->h_level+n)+k));
	      n++;
	    }
	  break;   
	case MFAM_SET:
	  while(n<=n_fin)
	    {
	      int k=0; 
	      int b_pow_n=(int)pow((double)tree->b,(double)n); 
	      fprintf(stderr,"printing set of level # %d\n",n);
	      for(k=0;k<b_pow_n;k++)
		fprintf(stderr,"%d %d\n",k,*(*(tree->s_level+n)+k));
	      n++;
	    }
	  break;
	default:
	  fprintf(stderr,"MFAM_tree_level_print arguments error\n");
	  break;
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"MFAM_tree_level_print error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_tree_cprint(MFAMt_tree *tree)
{ 
  if(tree!=NULL)
    {
      if(tree->c_level!=NULL)
	MFAM_node_cprint(tree->root,tree);
      else
	{
	  fprintf(stderr,"error\n");
	  return 0;
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"error\n");
      return 0;
    }
}

/****************************************************************************/

void MFAM_node_cprint(MFAMt_node* node,MFAMt_tree* tree)
{ 
  if(node!=NULL)
    {
      int i=0;
      fprintf(stdout,"node # x=%s,b=%d,i=%d,n=%d,k=%d\t",node->x,node->b,node->i,node->n,node->k);
      fprintf(stdout,"c=%f\n",*(*(tree->c_level+node->n)+node->k));  
      for(i=0;i<node->b;i++)
	MFAM_node_cprint(*(node->nodes+i),tree);
    }
}

/*****************************************************************************/

double MFAM_node_cget(MFAMt_node* node,MFAMt_tree* tree)
{ 
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+node->k!=NULL))
    return *(*(tree->c_level+node->n)+node->k);
  else
    {
      fprintf(stderr,"MFAM_qnode_cget error\n");
      return 0.;
    }
}

/*****************************************************************************/

void MFAM_node_cset(MFAMt_node* node,MFAMt_tree* tree,double c)
{  
  if((*(tree->c_level+node->n)!=NULL)&&(*(tree->c_level+node->n)+node->k!=NULL))
    *(*(tree->c_level+node->n)+node->k)=c;
  else
    fprintf(stderr,"MFAM_qnode_cset error\n");
}

/*****************************************************************************/

unsigned char MFAM_node_sget(MFAMt_node* node,MFAMt_tree* tree)
{
  if((*(tree->s_level+node->n)!=NULL)&&(*(tree->s_level+node->n)+node->k!=NULL))
    return *(*(tree->s_level+node->n)+node->k);
  else
    {
      fprintf(stderr,"MFAM_node_sget error\n");
      return 0;
    }
}

/*****************************************************************************/

unsigned char MFAM_node_sset(MFAMt_node* node,MFAMt_tree* tree,unsigned char s)
{
  if((*(tree->s_level+node->n)!=NULL)&&(*(tree->s_level+node->n)+node->k!=NULL))
    return (*(*(tree->s_level+node->n)+node->k)=s);
  else
    {
      fprintf(stderr,"MFAM_node_sset error\n");
      return 0;
    }
}

/*****************************************************************************/

double MFAM_node_rget(MFAMt_node* node,MFAMt_tree* tree)
{
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+node->k!=NULL))
    return *(*(tree->r_level+node->n)+node->k);
  else
    {
      fprintf(stderr,"MFAM_node_rget error\n");
      return 0.;
    }
}

/*****************************************************************************/

double MFAM_node_rset(MFAMt_node* node,MFAMt_tree* tree,double r)
{
  if((*(tree->r_level+node->n)!=NULL)&&(*(tree->r_level+node->n)+node->k!=NULL))
    return (*(*(tree->r_level+node->n)+node->k)=r);
  else
    {
      fprintf(stderr,"MFAM_node_rset error\n");
      return 0.;
    }
}

/*****************************************************************************/

void MFAM_node_hprint(MFAMt_node* node,MFAMt_tree* tree,short n_ini, short n_fin)
{ 
  if (node!=NULL)
    {
      int i=0;
      if ((node->n>=n_ini)&&(node->n<=n_fin))
	{ 
	  fprintf(stdout,"node # x=%s,b=%d,i=%d,n=%d,k=%d\t",node->x,node->b,node->i,node->n,node->k);
	  fprintf(stdout,"h=%f\n",*(*(tree->h_level+node->n)+node->k));
	}
      for(i=0;i<node->b;i++)
	MFAM_node_cprint(*(node->nodes+i),tree);
    }
}


/*****************************************************************************/

int MFAM_tree_h_levelprint(MFAMt_tree *tree,short n)
{
  return MFAM_tree_level_print(tree,MFAM_HOELDER,n,n);
}

/*****************************************************************************/

int MFAM_tree_hprint(MFAMt_tree *tree,short n_ini,short n_fin)
{ 
  if(tree!=NULL)
    {
      if(tree->h_level!=NULL)
	{
	  int n=n_ini-1;
	  do  
	    n++;
	  while((*(tree->h_level+n)!=NULL)&&(n<=n_fin));
	  if (n<n_fin+1)
	    {
	      fprintf(stderr,"error\n");
	      return 0;
	    }
	  if (n_ini==n_fin)
	    MFAM_tree_h_levelprint(tree,n_ini);
	  else
	    MFAM_node_hprint(tree->root,tree,n_ini,n_fin);
	}
      else
	{
	  fprintf(stderr,"error\n");
	  return 0;
	}
      return 1;
    }
  else
    { 
      fprintf(stderr,"error\n");
      return 0;
    }
}

/*****************************************************************************/

double MFAM_node_c_level_decimate(MFAMt_node* node,MFAMt_tree* tree,double (*capa)(double,double),short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    { 
      if(node->n<n_ini)
	{  
	  int i=0;
	  for(i=0;i<node->b;i++)
	    MFAM_node_c_level_decimate(*(node->nodes+i),tree,capa,n_ini,n_fin);
	  return 0.;
	}
      else if(node->n<n_fin)
	{
	  int i=0;
	  double c=0.; 
	  double c_tmp=0.; 
	  double sum_c=0.; 
	  sum_c+=c=c_ref=MFAM_node_c_level_decimate(*(node->nodes),tree,capa,n_ini,n_fin);
	  for(i=1;i<node->b;i++)
	    {
	      sum_c+=c_tmp=MFAM_node_c_level_decimate(*(node->nodes+i),tree,capa,n_ini,n_fin);
	      c=capa(c,c_tmp);
	    }
	  MFAM_node_cset(node,tree,c);
	  return sum_c;
	}
      else /* if(node->n=n_fin) */
	return MFAM_node_cget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_node_c_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_tree_c_level_decimate(MFAMt_tree* tree,MFAMt_capa t_capa,short n_ini,short n_fin)
{
  if(tree!=NULL)
    {  
      double (*capa)(double,double); 
      switch(t_capa)
	{ 
	case MFAM_SUM:
	  norm_meas=1.;
	  capa=MFAM_sum;
	  break;    
	case MFAM_LPSUM:
	  norm_meas=1.;
	  p=2;
	  capa=MFAM_lpsum;
	  break;
	case MFAM_MIN:
	  capa=MFAM_min;
	  break;   
	case MFAM_MAX:
	  capa=MFAM_max;
	  break;   
	case MFAM_ISO:
	  {
	    double Q=20;
	    double c_min=0,c_max=-HUGE;
	    int b_pow_n=(int)pow((double)tree->b,(double)n_fin);
	    double* c=NULL;
	    c=(double*)MFAM_tree_level_get(tree,MFAM_CAPACITY,n_fin);
	    if(MFAM_min_max(b_pow_n,c,&c_min,&c_max)==0)
	      { 
		fprintf(stderr,"MFAM_min_max error\n"); 
		return 0; 
	      }
	    q=(c_max-c_min)/Q;
	    capa=MFAM_iso;
	    break;
	  }
	case MFAM_ADAPISO:
	  {
	    int b_pow_n=(int)pow((double)tree->b,(double)n_fin);
	    double* c=NULL;
	    c=(double*)MFAM_tree_level_get(tree,MFAM_CAPACITY,n_fin);
	    if((sigma2=MFAM_variance(b_pow_n,c,MFAM_mean(b_pow_n,c)))==HUGE)
	      {
		fprintf(stderr,"MFAM_variance error\n");
		return 0;
	      }
	    capa=MFAM_adapiso;
	    break;
	  } 
	default:
	  return 0;
	}
      MFAM_node_c_level_decimate(tree->root,tree,capa,n_ini,n_fin);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_c_level_decimate arguments error\n");
      return 0;
    } 
}

/*****************************************************************************/

double MFAM_node_r_level_decimate(MFAMt_node* node,MFAMt_tree* tree,short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{ 
	  int i=0;
	  for(i=0;i<node->b;i++)
	    MFAM_node_r_level_decimate(*(node->nodes+i),tree,n_ini,n_fin);
	  return 0.;
	}
      else if(node->n<n_fin)
	{
	  int i=0; 
	  double r=0.;
	  for(i=0;i<node->b;i++)
	    r+=MFAM_node_r_level_decimate(*(node->nodes+i),tree,n_ini,n_fin);
	  return r;
	}
      else /* if(node->n=n_fin) */
	return MFAM_node_rget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_node_r_level_decimate arguments error\n");
      return 0.;
    }
}

/*****************************************************************************/

int MFAM_tree_r_level_decimate(MFAMt_tree* tree,short n_ini,short n_fin)
{
  if(tree!=NULL)
    {  
      MFAM_node_r_level_decimate(tree->root,tree,n_ini,n_fin);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_r_level_decimate arguments error\n");
      return 0;
    } 
}

/*****************************************************************************/

unsigned char MFAM_node_s_level_decimate(MFAMt_node* node,MFAMt_tree* tree,short n_ini,short n_fin)
{
  if((node!=NULL)&&(tree!=NULL))
    {
      if(node->n<n_ini)
	{ 
	  int i=0;
	  for(i=0;i<node->b;i++)
	    MFAM_node_s_level_decimate(*(node->nodes+i),tree,n_ini,n_fin);
	  return 0;
	}
      else if(node->n<n_fin)
	{
	  int i=0; 
	  unsigned char s=0;
	  for(i=0;i<node->b;i++)
	    if(MFAM_node_s_level_decimate(*(node->nodes+i),tree,n_ini,n_fin))
	      s=1;
	  MFAM_node_sset(node,tree,s);
	  return s;
	}
      else /* if(node->n=n_fin) */
	return MFAM_node_sget(node,tree);
    }
  else
    {
      fprintf(stderr,"MFAM_node_s_level_decimate arguments error\n");
      return 0;
    }
}

/*****************************************************************************/

int MFAM_tree_s_level_decimate(MFAMt_tree* tree,short n_ini,short n_fin)
{
  if(tree!=NULL)
    {  
      MFAM_node_s_level_decimate(tree->root,tree,n_ini,n_fin);
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_s_level_decimate arguments error\n");
      return 0;
    } 
}

/*****************************************************************************/

void MFAM_node_multinomial(MFAMt_node* node,MFAMt_tree* tree,short b,double* p,double mu)
{ 
  if(node!=NULL)
    {
      int i=0;
      MFAM_node_cset(node,tree,mu);
      for(i=0;i<b;i++)
	MFAM_node_multinomial(*(node->nodes+i),tree,b,p,mu**(p+i));
    } 
}

/*****************************************************************************/

MFAMt_tree* MFAM_tree_multinomial(short n,short b,double* p)
{ 
  MFAMt_tree* tree=NULL;
  if((tree=MFAM_tree_create(n,b))==NULL)
    {
      fprintf(stderr,"MFAM_tree create error\n");
      return NULL;
    } 
  if(MFAM_tree_level_create(tree,MFAM_CAPACITY,0,n)==0)
    {
      fprintf(stderr,"MFAM_tree level create error\n");
      return NULL;
    }  
  MFAM_node_multinomial(tree->root,tree,b,p,1.);
  return tree;
}

/*****************************************************************************/

void MFAM_node_cantor(MFAMt_node* node,MFAMt_tree* tree,short b,unsigned char* q,unsigned char s)
{ 
  if(node!=NULL)
    {
      int i=0;
      MFAM_node_sset(node,tree,s);
      for(i=0;i<b;i++)
	MFAM_node_cantor(*(node->nodes+i),tree,b,q,s**(q+i));
    } 
}

/*****************************************************************************/

MFAMt_tree* MFAM_tree_cantor(short n,short b,unsigned char* q)
{ 
  MFAMt_tree* tree=NULL;
  if((tree=MFAM_tree_create(n,b))==NULL)
    {
      fprintf(stderr,"MFAM_tree create error\n");
      return NULL;
    } 
  if(MFAM_tree_level_create(tree,MFAM_SET,0,n)==0)
    {
      fprintf(stderr,"MFAM_tree level create error\n");
      return NULL;
    }  
  MFAM_node_cantor(tree->root,tree,b,q,1);
  return tree;
}

/*****************************************************************************/

MFAMt_tree* MFAM_tree_s_level_init(int Nx,unsigned char* s1d,void* tree,short* n_opt,short* b_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(0<=*n_opt)&&(0<=*b_opt))
    {
      MFAMt_tree* tree_tmp=NULL;
      if(tree==NULL) /* tree's not already created */
	{  
	  int b_pow_n=0;

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's not already created\n"); 
#endif /* MFAMDEBUG */

	  /* find optimal decomposition base: b_opt */ 
	  if(*b_opt==0) 
	    MFAM_base(Nx,b_opt);

#ifdef MFAMDEBUG  
	  fprintf(stderr,"optimal decomposition base: b_opt=%d\n",*b_opt);
#endif /* MFAMDEBUG */

	  /* truncate it */
	  if((*b_opt==0)||(*b_opt>5))
	    *b_opt=2;
	  
#ifdef MFAMDEBUG
	  fprintf(stderr,"chosen decomposition base: b_opt=%d\n",*b_opt);
#endif /* MFAMDEBUG */

	  /* find maximum resolution: n */ 
	  if(*n_opt==0)
	    *n_opt=ceil(log((double)Nx)/log((double)*b_opt));
	
#ifdef MFAMDEBUG
	  fprintf(stderr,"Nx=%d\n",Nx);
	  fprintf(stderr,"maximum resolution: n_opt=%d\n",*n_opt);
#endif /* MFAMDEBUG */
	
	  tree_tmp=MFAM_tree_create(*n_opt,*b_opt);

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's created\n");
#endif /* MFAMDEBUG */

	  b_pow_n=(int)pow((double)*b_opt,(double)*n_opt);
	  if(Nx==b_pow_n)
 	    {   
	      MFAM_tree_level_create(tree_tmp,MFAM_SET,0,*n_opt-1);
	      MFAM_tree_level_set(tree_tmp,MFAM_SET,*n_opt,(void*)s1d);
	    } 
 	  else 
 	    {   
 	      MFAM_tree_level_create(tree_tmp,MFAM_SET,0,*n_opt); 
 	      MFAM_tree_level_convert(tree_tmp,MFAM_SET,*n_opt,Nx,(void*)s1d); 
 	    } 
	 
	  MFAM_tree_s_level_decimate(tree_tmp,0,*n_opt); 
	}  
      else /* tree's already created */
	{ 

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's already created\n");
#endif /* MFAMDEBUG */

	  /* pointers' affectation */
	  tree_tmp=(MFAMt_tree*)tree;
	  *b_opt=tree_tmp->b;
	  *n_opt=tree_tmp->N;
	}
      return tree_tmp;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_s_level_init arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      fprintf(stderr,"\tb_opt=[7m%p[m\n",b_opt);
      return NULL;
    }
}

/*****************************************************************************/

int MFAM_tree_s_level_update(int Nx,unsigned char* s1d,MFAMt_tree* tree,short* n_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(0<=*n_opt))
    {  
      int b_pow_n=0;
      b_pow_n=(int)pow((double)tree->b,(double)*n_opt);

      if(Nx==b_pow_n)
	{
	  MFAM_tree_level_set(tree,MFAM_SET,*n_opt,(void*)NULL); 
	  MFAM_tree_level_set(tree,MFAM_SET,*n_opt,(void*)s1d);
	}  
      else
	MFAM_tree_level_convert(tree,MFAM_SET,*n_opt,Nx,(void*)s1d);
      
      MFAM_tree_s_level_decimate(tree,0,*n_opt); 
 
#ifdef MFAMDEBUG
      fprintf(stderr,"tree's udpated\n"); 
#endif /* MFAMDEBUG */ 

      return 1;
    } 
  else
    {
      fprintf(stderr,"MFAM_tree_s_level_update arguments error\n");
      fprintf(stderr,"\tNx=[7m%d[m\n",Nx);
      fprintf(stderr,"\ts1d=[7m%p[m\n",s1d);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);  
      return 0;
    }
}

/*****************************************************************************/

int MFAM_tree_s_level_clear(int Nx,unsigned char* s1d,MFAMt_tree* tree_tmp,void* tree,short* n_opt)
{
  if((Nx>0)&&(s1d!=NULL)&&(tree_tmp!=NULL)&&(0<=*n_opt))
    {  
      if(tree==NULL) /* tree not already created */
	{
	  int b_pow_n=0;
	  b_pow_n=(int)pow((double)tree_tmp->b,(double)*n_opt);
	  
	  if(Nx==b_pow_n)
	    {
	      /* set levels deletion */  
	      MFAM_tree_level_set(tree_tmp,MFAM_SET,*n_opt,(void*)NULL); 
	      MFAM_tree_level_delete(tree_tmp,MFAM_SET,0,*n_opt-1);
	    }
	  else
	    MFAM_tree_level_delete(tree_tmp,MFAM_SET,0,*n_opt);

	  /* tree deletion */
	  MFAM_tree_delete(tree_tmp);
	  tree_tmp=NULL; 

#ifdef MFAMDEBUG
	  fprintf(stderr,"tree's cleared\n"); 
#endif /* MFAMDEBUG */ 

	} 
      else /* tree already created */
	 tree_tmp=NULL;
      return 1;
    }
  else
    {
      fprintf(stderr,"MFAM_tree_s_level_clear arguments error\n");
      fprintf(stderr,"\ttree_tmp=[7m%p[m\n",tree_tmp);
      fprintf(stderr,"\ttree=[7m%p[m\n",tree);
      fprintf(stderr,"\tn_opt=[7m%p[m\n",n_opt);
      return 0;
    }
}
