/*-----------------------------------------------------------------------------
*                                                                              
*                                    FracLab
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* File path        : /home/users/canus/MFA.CMP/MFAM.CL/inc/MFAM_tree.h
* File type        : C header file
* Revision         : $Revision$
* RCS status       : $Id$
* Created by       : (Fractales Project INRIA) Christophe Canus
* Creation date    : Tue Sep  3 17:06:24 2002
* Last edited by   : (Fractales Project INRIA) Christophe Canus
* Last edition date: Tue Sep 17 20:48:11 2002
*                                                                              
*------------------------------------------------------------------------------
*                                                                              
* Description : 
*                
*                
*                
*------------------------------------------------------------------------------
*/

/* Christophe Canus 2002 */

#ifndef _MFAM_tree_h_
#define _MFAM_tree_h_
#include "MFAM_include.h"
#include "MFAM_misc.h"
#include "MFAM_stats.h"
#include "MFAM_btree.h"
#include "MFAM_capacity.h"

typedef struct _node
{
  char* x;
  short b; 
  short i;
  short n;
  int   k;
  struct _node** nodes;
} MFAMt_node;

typedef struct _tree
{
  short N;
  short b;
  double** c_level;
  double** r_level;
  double** h_level;
  unsigned char** s_level;
  MFAMt_node* root;
} MFAMt_tree;

#ifndef __STDC__ 
MFAMt_node* MFAM_node_create();
void MFAM_node_delete();
void MFAM_node_print();
#else /* __STDC__ */
MFAMt_node* MFAM_node_create(MFAMt_node*,char*,short,short,short,int,short);
void MFAM_node_delete(MFAMt_node*);
void MFAM_node_print(MFAMt_node*,int); 
#endif /* __STDC__ */

#ifndef __STDC__
MFAMt_tree* MFAM_tree_create();
void MFAM_tree_delete();
void MFAM_tree_print();
#else /* __STDC__ */
MFAMt_tree* MFAM_tree_create(short,short);
void MFAM_tree_delete(MFAMt_tree*);
void MFAM_tree_print(MFAMt_tree*);
#endif /* __STDC__ */

#ifndef __STDC__
int MFAM_tree_level_create();
int MFAM_tree_level_delete();
int MFAM_tree_level_set();
int MFAM_tree_level_convert();
void* MFAM_tree_level_get();
int MFAM_tree_level_print();
#else /* __STDC__ */
int MFAM_tree_level_create(MFAMt_tree*,MFAMt_level,short,short);
int MFAM_tree_level_delete(MFAMt_tree*,MFAMt_level,short,short);
int MFAM_tree_level_set(MFAMt_tree*,MFAMt_level,short,void*);
int MFAM_tree_level_convert(MFAMt_tree*,MFAMt_level,short,int,void*);
void* MFAM_tree_level_get(MFAMt_tree*,MFAMt_level,short);
int MFAM_tree_level_print(MFAMt_tree*,MFAMt_level,short,short);
#endif /* __STDC__ */

#ifndef __STDC__
int MFAM_tree_cprint();
void MFAM_node_cprint();
#else /* __STDC__ */
int MFAM_tree_cprint(MFAMt_tree*);
void MFAM_node_cprint(MFAMt_node*,MFAMt_tree*);
#endif /* __STDC__ */

#ifndef __STDC__
double MFAM_node_cget();
void MFAM_node_cset();
double MFAM_node_rget();
double MFAM_node_rset();
unsigned char MFAM_node_sget();
unsigned char MFAM_node_sset();
#else /* __STDC__ */
double MFAM_node_cget(MFAMt_node*,MFAMt_tree*);
void MFAM_node_cset(MFAMt_node*,MFAMt_tree*,double);
double MFAM_node_rget(MFAMt_node*,MFAMt_tree*);
double MFAM_node_rset(MFAMt_node*,MFAMt_tree*,double);
unsigned char MFAM_node_sget(MFAMt_node*,MFAMt_tree*);
unsigned char MFAM_node_sset(MFAMt_node*,MFAMt_tree*,unsigned char);
#endif /* __STDC__ */

#ifndef __STDC__
void MFAM_node_hprint();
int MFAM_tree_hprint();
int MFAM_tree_h_levelprint();
#else /* __STDC__ */
void MFAM_node_hprint(MFAMt_node*,MFAMt_tree*,short,short);
int MFAM_tree_hprint(MFAMt_tree*,short,short);
int MFAM_tree_h_levelprint(MFAMt_tree*,short);
#endif /* __STDC__ */

#ifndef __STDC__
double MFAM_node_c_level_decimate();
int MFAM_tree_c_level_decimate();
double MFAM_node_r_level_decimate();
int MFAM_tree_r_level_decimate();
unsigned char MFAM_node_s_level_decimate();
int MFAM_tree_s_level_decimate();
#else /* __STDC__ */
double MFAM_node_c_level_decimate(MFAMt_node*,MFAMt_tree*,double (*)(double,double),short,short);
int MFAM_tree_c_level_decimate(MFAMt_tree*,MFAMt_capa,short,short);
double MFAM_node_r_level_decimate(MFAMt_node*,MFAMt_tree*,short,short);
int MFAM_tree_r_level_decimate(MFAMt_tree*,short,short);
unsigned char MFAM_node_s_level_decimate(MFAMt_node*,MFAMt_tree*,short,short);
int MFAM_tree_s_level_decimate(MFAMt_tree*,short,short);
#endif /* __STDC__ */

#ifndef __STDC__
void MFAM_node_multinomial();
MFAMt_tree* MFAM_tree_multinomial();
void MFAM_node_cantor();
MFAMt_tree* MFAM_tree_cantor();
#else /* __STDC__ */
void MFAM_node_multinomial(MFAMt_node*,MFAMt_tree*,short,double*,double);
MFAMt_tree* MFAM_tree_multinomial(short,short,double*);
void MFAM_node_cantor(MFAMt_node*,MFAMt_tree*,short,unsigned char*,unsigned char);
MFAMt_tree* MFAM_tree_cantor(short,short,unsigned char*);
#endif /* __STDC__ */

#ifndef __STDC__
MFAMt_tree* MFAM_tree_s_level_init();
int MFAM_tree_s_level_update();
int MFAM_tree_s_level_clear();
#else /* __STDC__ */
MFAMt_tree* MFAM_tree_s_level_init(int,unsigned char*,void*,short*,short*);
int MFAM_tree_s_level_update(int,unsigned char*,MFAMt_tree*,short*);
int MFAM_tree_s_level_clear(int,unsigned char*,MFAMt_tree*,void*,short*);
#endif /* __STDC__ */

#endif /* _MFAM_tree_h_ */
