/* This Software is ( Copyright INRIA . 1998  1 )                    */
/*                                                                   */
/* INRIA  holds all the ownership rights on the Software.            */
/* The scientific community is asked to use the SOFTWARE             */
/* in order to test and evaluate it.                                 */
/*                                                                   */
/* INRIA freely grants the right to use modify the Software,         */
/* integrate it in another Software.                                 */
/* Any use or reproduction of this Software to obtain profit or      */
/* for commercial ends being subject to obtaining the prior express  */
/* authorization of INRIA.                                           */
/*                                                                   */
/* INRIA authorizes any reproduction of this Software.               */
/*                                                                   */
/*    - in limits defined in clauses 9 and 10 of the Berne           */
/*    agreement for the protection of literary and artistic works    */
/*    respectively specify in their paragraphs 2 and 3 authorizing   */
/*    only the reproduction and quoting of works on the condition    */
/*    that :                                                         */
/*                                                                   */
/*    - "this reproduction does not adversely affect the normal      */
/*    exploitation of the work or cause any unjustified prejudice    */
/*    to the legitimate interests of the author".                    */
/*                                                                   */
/*    - that the quotations given by way of illustration and/or      */
/*    tuition conform to the proper uses and that it mentions        */
/*    the source and name of the author if this name features        */
/*    in the source",                                                */
/*                                                                   */
/*    - under the condition that this file is included with          */
/*    any reproduction.                                              */
/*                                                                   */
/* Any commercial use made without obtaining the prior express       */
/* agreement of INRIA would therefore constitute a fraudulent        */
/* imitation.                                                        */
/*                                                                   */
/* The Software beeing currently developed, INRIA is assuming no     */
/* liability, and should not be responsible, in any manner or any    */
/* case, for any direct or indirect dammages sustained by the user.  */
/*                                                                   */
/* Any user of the software shall notify at INRIA any comments       */
/* concerning the use of the Sofware (e-mail : FracLab@inria.fr)     */
/*                                                                   */
/* This file is part of FracLab, a Fractal Analysis Software         */

#include "C_LAB_Interf.h"
#include "CMPT1D_Convol.h"
#include "CMPT1D_DefWavelet.h"
#include "CMPT1D_MPTransform.h"
#include "CMPT1D_Wavelet.h"
#include <math.h>

#ifndef PI
#define PI 3.14159
#endif

void LAB_cmptsup()

{
  /* Input */
  Matrix *MSignal;
  double *Signal;
  int TSignal;

  Matrix *MSmin, *MSmax;
  double Smin, Smax;

  Matrix *MNscales;
  int Nscales;

  /* Output */
  int WTSize;
  
  Matrix *MWT;
  double *WTr;

  Matrix *MFreq;
  double *Freq;

  Matrix *MS;
  double *S;
  
  /* Work */
 
  CMPT1D_t_Filtre
      *IndicFilter;


  CMPT1D_t_Ondelette 
      *IndicWavelet;
 
  
  int i;

  int ech;
  double lSmin,
    lSmax,
    Sinc,
    lS;
 
  
  MSignal = (Matrix *)(Interf.Param[0]);
  MSmin = (Matrix *)(Interf.Param[1]);
  MSmax = (Matrix *)(Interf.Param[2]);
  MNscales = (Matrix *)(Interf.Param[3]);
  /* get and verify the input parameters */

  if (Interf.NbParamIn != 4)
    {
      InterfError("cmpt : you must give 4 input parameters\n");
      return;
    }
 
  if (Interf.NbParamOut > 2)
    {
      InterfError("cmpt : you must give at most 2 output parameter\n");
      return;
    }
  

  /* Signal type */
  if (!(MatrixIsNumeric(MSignal) && MatrixIsReal(MSignal)))
    {
      InterfError("cmpt : Input signal must be a real matrix\n");
      return;
    }
  /* Signal dimension */
  TSignal = MAX(MatrixGetWidth(MSignal),MatrixGetHeight(MSignal));
  Signal = MatrixGetPr(MSignal);
  if (MIN(MatrixGetWidth(MSignal),MatrixGetHeight(MSignal)) !=1)
    {
      InterfError("cmpt : Input signal must be a one_dimensional matrix\n");
      return;
    }
  
  /* Smin and Smax type */
  if (!(MatrixIsScalar(MSmin) && MatrixIsScalar(MSmax)))
    {
      InterfError("cmpt : Smin and Smax must be real scalars\n");
      return;
    }
  Smin = MatrixGetScalar(MSmin);
  Smax = MatrixGetScalar(MSmax);
  if (!((Smin>0) && (Smax>0)))
    {
      InterfError("cmpt : Smin and Smax must be positive\n");
      return;
    }
  
  
  if (!(MatrixIsScalar(MNscales)))
    {
      InterfError("cmpt : Nscales must be a real scalar\n");
      return;
    }
  Nscales = (int)floor(MatrixGetScalar(MNscales));
  if (!(Nscales>0))
    {
      InterfError("cmpt : Nscales must be positive\n");
      return;
    }

  

  WTSize = CMPT1D_Taille_MPT(TSignal,Nscales);
  MWT = MatrixCreate(TSignal, Nscales, "real");
  WTr = MatrixGetPr(MWT);

  IndicFilter = (CMPT1D_t_Filtre *)malloc ( sizeof( CMPT1D_t_Filtre ) );
  IndicWavelet = (CMPT1D_t_Ondelette *)malloc ( sizeof( CMPT1D_t_Ondelette ) );

  /* IndicWavelet->ValeurReelle = CMPT1D_Parab; */
  IndicWavelet->ValeurReelle = CMPT1D_Indic;
  IndicWavelet->ValeurComplexe = NULL;

  IndicWavelet->xmin = -1;
  IndicWavelet->xmax = 1;

  
 

  CMPT1D_MPT_max(
	   Signal,
	   TSignal,
	   WTr,
	   NULL,
	   IndicWavelet,
	   IndicFilter,
	   Smin,
	   Smax,
	   Nscales
	   );
  MatrixTranspose(MWT);
  free(IndicWavelet);
  free(IndicFilter);
  
  
  MS = MatrixCreate(1,Nscales,"real");
  S = MatrixGetPr(MS);
  
  
  lSmin = log(Smin);
  lSmax = log(Smax);
  Sinc = (lSmax-lSmin)/(double)Nscales;
  lS = lSmin;
  
  for (ech=0; ech<Nscales; ech++)
    {
      S[ech] = exp(lS);
      lS += Sinc;
    }
  
  
  ReturnParam(MWT);
  ReturnParam(MS);
  
}







